/* $Id: NamedElementRegister.java,v 1.1.1.1 2003/02/11 16:19:40 bures Exp $ */
package SOFA.Component;

/** Register for named objects.
  *
  * @author Petr Hnetynka
  */
public class NamedElementRegister extends java.util.Hashtable {
  
  /** Return element with given name.
    * @param name    name of element
    * @exception  NamingException   if object with given name does not exist
    */
  public Object getElementNamed(String name) throws NamingException {
    Object o = get(name);
    if (o==null)
      throw new NamingException("No such name");
    return o;
  }
  
  /** Register new element with given name.
    * @param name     name of element
    * @param element  element
    * @exception  NamingException   if object with given name already exists
    */
  public void registerElementNamed(String name, Object element) throws NamingException {
    try {
      if (containsKey(name)) {
        throw new NamingException("Duplicated name");
      } else {
        put(name, element);
      }
    } catch(Exception e) {
      throw new NamingException(e.toString());
    }
  }
  
  /** Register new element with given name. If object with given name exists, then reregister new one.
    * @param name     name of element
    * @param element  element
    * @exception  NamingException   
    */
  public void forceRegisterElementNamed(String name, Object element) throws NamingException {
    try {
      if (containsKey(name))
        remove(name);
      put(name, element);
    } catch(Exception e) {
      throw new NamingException(e.toString());
    }
  }

  /** Unregister element with given name.
    * @param name    name of element
    * @exception  NamingException   if object with given name does not exist
    */
  public void unregisterElementNamed(String name) throws NamingException {
    try {
      if (containsKey(name))
        remove(name);
      else
        throw new NamingException("No such name");
    } catch(Exception e) {
      throw new NamingException(e.toString());
    }
  }

  /** Returns all names in the register.
    * @return   enumeration with names
    */
  public java.util.Enumeration getAllNames() {
    return keys();
  }
}
