/* $Id: DeplDockConnector.java,v 1.2 2004/05/20 14:23:50 bures Exp $ */
package SOFA.Connector.Boot;

import SOFA.Connector.ConnectorException;
import SOFA.Connector.Linkable;
import SOFA.Connector.Reference;
import SOFA.Connector.RoleBase;
import SOFA.Connector.TaggedProfile;
import SOFA.SOFAnode.Run.Deployment.DeplDock;

/** Contains methods for creating connector to DeplDock.
  *
  * @author Petr Hnetynka
  */
public class DeplDockConnector {

  /** Creates server side of connector to DDock.
    * @param ddock     depldock
    * @param name      name of the depldock
    * @return          server side of connector
    */
  public static RoleBase createSrv(DeplDock ddock, String name) throws ConnectorException {
    RoleBase role = new SOFA.Connector.Boot.DeplDockSRole();
    SOFA.Connector.ECG.DeploymentDescriptor cDD = new SOFA.Connector.ECG.DeploymentDescriptor();
    cDD.unit="Server";
    role.setDeploymentDescriptor(cDD);
    
    Linkable skelRMI;
    String rmiName;
    String rmihost = System.getProperty("sofa.rmihost", "localhost");
    String rmiport = System.getProperty("sofa.rmiport", "1099");
    try {
      rmiName="//"+rmihost+":"+rmiport+"/SOFA/Connectors/Boot/DeplDockRMISkel/"+name;
      skelRMI = new SOFA.Connector.Boot.DeplDockRMISkel(name);
    } catch (Exception e) {
      throw new ConnectorException("Can't instantiate DeplDockRMISkel.",e);
    }
    skelRMI.link(role);

    // ???
    Reference ref=new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DeplDockConnector",
    new TaggedProfile[] { new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName) } );
    role.setSOFAReference(ref);
    role.link(ddock);
    return role;
  }
  
  /** Creates client side of connector to DDock
    * @param   ref    reference to depldock
    * @return         client side of connector
    */
  public static RoleBase createClt(SOFA.Connector.Reference ref) throws ConnectorException {
    RoleBase role = new SOFA.Connector.Boot.DeplDockCRole();
    try {
      role.link(ref);
    } catch (SOFA.Connector.LinkException e) {
      throw new ConnectorException("Can't link connector to DeplDock.", e);
    }
    return role;
  }
}
