/*
 * ElementOutputDescriptor.java
 *
 * Created on 2. duben 2002, 20:56
 */

package SOFA.Connector.ECG;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 *
 * @author  ghort
 * @version 
 */
public class ConnectorOutputDescriptor {

	public String name;
	public String unit;
	public String impl;

	public SOFA.Connector.Property[] params;

	public SOFA.Connector.EEG.ElementOutputDescriptor[] elements;

	/** Creates new ElementOutputDescriptor */
	public ConnectorOutputDescriptor() {
	}

	public ConnectorOutputDescriptor(Element element) throws ConnectorDescriptorXMLException {
		fromXML(element);
	}

	public Element toXML(Document parentDoc) throws ConnectorDescriptorXMLException {
		Element ret=parentDoc.createElement("sofa_connector");
		ret.setAttribute("name",name);
		ret.setAttribute("unit",unit);
		ret.setAttribute("impl",impl);

		for (int i=0;i<params.length;i++) {
			Element par=parentDoc.createElement("connector_param");
			par.setAttribute("name",params[i].name);
			par.setAttribute("value",params[i].value);
			ret.appendChild(par);
		}

		try {
			for (int i=0;i<elements.length;i++) {
				Element el=elements[i].toXML(parentDoc);
				ret.appendChild(el);
			}
		} catch (SOFA.Connector.EEG.ElementDescriptorXMLException e) {
			throw new ConnectorDescriptorXMLException("Can't create output descriptor XML.",e);
		}
		return ret;
	}

	public void fromXML(Element element) throws ConnectorDescriptorXMLException {
		java.util.LinkedList elems=new java.util.LinkedList();
		java.util.LinkedList pars=new java.util.LinkedList();

		if (!element.hasAttribute("name")) {
			throw new ConnectorDescriptorXMLException("Element doesn't have 'name' attribute.");
		}
		name=element.getAttribute("name");

		if (!element.hasAttribute("unit")) {
			throw new ConnectorDescriptorXMLException("Element doesn't have 'unit' attribute.");
		}
		unit=element.getAttribute("unit");

		if (!element.hasAttribute("impl")) {
			throw new ConnectorDescriptorXMLException("Element doesn't have 'impl' attribute.");
		}
		impl=element.getAttribute("impl");
		
		Node node=element.getFirstChild();
		while (node!=null) {
			if (node.getNodeType()==Node.ELEMENT_NODE) {
				Element el=(Element)node;
				if (el.getNodeName().equals("connector_element")) {
					try {
						elems.add(new SOFA.Connector.EEG.ElementOutputDescriptor(el));
					} catch (SOFA.Connector.EEG.ElementDescriptorXMLException e) {
						throw new ConnectorDescriptorXMLException("Can't read connector_element.",e);
					}
				} else if (el.getNodeName().equals("connector_param") && el.hasAttribute("name") && el.hasAttribute("value")) {
					pars.add(new SOFA.Connector.Property(el.getAttribute("name"),el.getAttribute("value")));
				}
			}
			node=node.getNextSibling();
		}
		
		int i;
		java.util.Iterator iter=elems.iterator();
		elements=new SOFA.Connector.EEG.ElementOutputDescriptor[elems.size()];
		for (i=0;iter.hasNext();i++) {
			elements[i]=(SOFA.Connector.EEG.ElementOutputDescriptor)iter.next();
		}

		iter=pars.iterator();
		params=new SOFA.Connector.Property[pars.size()];
		for (i=0;iter.hasNext();i++) {
			params[i]=(SOFA.Connector.Property)iter.next();
		}
	}
	
}
