/*
 * DataStreamImpl.java
 *
 * Created on 5. duben 2002, 13:50
 */

package SOFA.Connector.ECG.SGenerator.Connectors;

import SOFA.Connector.ConnectorException;
import SOFA.Connector.LinkException;
import SOFA.Connector.Linkable;
import SOFA.Connector.Reference;
import SOFA.Connector.RoleBase;
import SOFA.Connector.RoleBaseImpl;
import SOFA.Connector.EEG.Types.CRole;
import SOFA.Connector.EEG.Types.Interceptor;
import SOFA.Connector.EEG.Types.Receiver;
import SOFA.Connector.EEG.Types.SRole;
import SOFA.Connector.EEG.Types.Sender;

/**
 *
 * @author  ghort
 * @version 
 */
public class DataStreamImpl extends RoleBaseImpl implements SOFA.Connector.ECG.Types.DataStream, SOFA.Connector.EEG.ConnectorManagerInterface {

	protected CRole pushSRole;
	protected SRole pullSRole;
	protected Interceptor pushSInterceptor;
	protected Interceptor pullSInterceptor;
	protected Sender sender;

	protected SRole pushRRole;
	protected CRole pullRRole;
	protected Interceptor pushRInterceptor;
	protected Interceptor pullRInterceptor;
	protected Receiver receiver;

	protected final static int pullMode=0;
	protected final static int pushMode=1;
	
	/** Creates new DataStreamImpl */
	public DataStreamImpl() {
	}

	static public RoleBase createConnector(SOFA.Connector.ECG.DeploymentDescriptor dd, ClassLoader cl) throws ConnectorException {
		int sMode, rMode;
		String mode=SOFA.Connector.Property.findFirst(dd.params,"senderMode");
		if (mode!=null) {
			if (mode.equals("push")) {
				sMode=pushMode;
			} else if (mode.equals("pull")) {
				sMode=pullMode;
			} else {
				throw new ConnectorException("Parameter senderMode is incorrect '"+mode+"'.");
			}
		} else {
			throw new ConnectorException("Parameter senderMode is not specified");
		}

		mode=SOFA.Connector.Property.findFirst(dd.params,"receiverMode");
		if (mode!=null) {
			if (mode.equals("push")) {
				rMode=pushMode;
			} else if (mode.equals("pull")) {
				rMode=pullMode;
			} else {
				throw new ConnectorException("Parameter receiverMode is incorrect '"+mode+"'.");
			}
		} else {
			throw new ConnectorException("Parameter receiverMode is not specified");
		}
		
		DataStreamImpl pc=new DataStreamImpl();
		RoleBase role=null;
		if (dd.unit.equals("Sender")) {
			try {
				Linkable last;

				if (sMode==pushMode) {
					SOFA.Connector.EEG.ElementOutputDescriptor pushSRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pushSRole");
					pc.pushSRole=(CRole)Class.forName(pushSRoleClass.impl, true, cl).newInstance();
					pc.pushSRole.setManagerTie(pc);
					last=pc.pushSRole;

					SOFA.Connector.EEG.ElementOutputDescriptor pushSInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pushSInterceptor");
					if (pushSInterceptorClass!=null) {
						pc.pushSInterceptor=(Interceptor)Class.forName(pushSInterceptorClass.impl, true, cl).newInstance();
						pc.pushSInterceptor.setManagerTie(pc);
						last.link(pc.pushSInterceptor);
						last=pc.pushSInterceptor;
					}
				
					SOFA.Connector.EEG.ElementOutputDescriptor senderClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"sender");
					pc.sender=(Sender)Class.forName(senderClass.impl, true, cl).newInstance();
					pc.sender.setManagerTie(pc);
					last.link(pc.sender);

					role=pc.pushSRole;
				} else {
					SOFA.Connector.EEG.ElementOutputDescriptor senderClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"sender");
					pc.sender=(Sender)Class.forName(senderClass.impl, true, cl).newInstance();
					pc.sender.setManagerTie(pc);
					last=(Linkable)pc.sender;

					SOFA.Connector.EEG.ElementOutputDescriptor pullSInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pullSInterceptor");
					if (pullSInterceptorClass!=null) {
						pc.pullSInterceptor=(Interceptor)Class.forName(pullSInterceptorClass.impl, true, cl).newInstance();
						pc.pullSInterceptor.setManagerTie(pc);
						last.link(pc.pullSInterceptor);
						last=pc.pullSInterceptor;
					}
				
					SOFA.Connector.EEG.ElementOutputDescriptor pullSRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pullSRole");
					pc.pullSRole=(SRole)Class.forName(pullSRoleClass.impl, true, cl).newInstance();
					pc.pullSRole.setManagerTie(pc);
					last.link(pc.pullSRole);

					role=pc.pullSRole;
				}
				
				pc.setDeploymentDescriptor(dd);
			} catch (Exception e) {
				throw new ConnectorException("Can't instantiate senderRole.",e);
			}
		} else if (dd.unit.equals("Receiver")) {
			try {
				Linkable last;

				if (rMode==pushMode) {
					SOFA.Connector.EEG.ElementOutputDescriptor receiverClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"receiver");
					pc.receiver=(Receiver)Class.forName(receiverClass.impl, true, cl).newInstance();
					pc.receiver.setManagerTie(pc);
					last=(Linkable)pc.receiver;

					SOFA.Connector.EEG.ElementOutputDescriptor pushRInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pushRInterceptor");
					if (pushRInterceptorClass!=null) {
						pc.pushRInterceptor=(Interceptor)Class.forName(pushRInterceptorClass.impl, true, cl).newInstance();
						pc.pushRInterceptor.setManagerTie(pc);
						last.link(pc.pushRInterceptor);
						last=pc.pushRInterceptor;
					}
				
					SOFA.Connector.EEG.ElementOutputDescriptor pushRRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pushRRole");
					pc.pushRRole=(SRole)Class.forName(pushRRoleClass.impl, true, cl).newInstance();
					pc.pushRRole.setManagerTie(pc);
					last.link(pc.pushRRole);

					role=pc.pushRRole;
				} else {
					SOFA.Connector.EEG.ElementOutputDescriptor pullRRoleClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pullRRole");
					pc.pullRRole=(CRole)Class.forName(pullRRoleClass.impl, true, cl).newInstance();
					pc.pullRRole.setManagerTie(pc);
					last=pc.pullRRole;

					SOFA.Connector.EEG.ElementOutputDescriptor pullRInterceptorClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"pullRInterceptor");
					if (pullRInterceptorClass!=null) {
						pc.pullRInterceptor=(Interceptor)Class.forName(pullRInterceptorClass.impl, true, cl).newInstance();
						pc.pullRInterceptor.setManagerTie(pc);
						last.link(pc.pullRInterceptor);
						last=pc.pullRInterceptor;
					}
				
					SOFA.Connector.EEG.ElementOutputDescriptor receiverClass=SOFA.Connector.EEG.ElementOutputDescriptor.findFirst(dd.elements,"receiver");
					pc.receiver=(Receiver)Class.forName(receiverClass.impl, true, cl).newInstance();
					pc.receiver.setManagerTie(pc);
					last.link(pc.receiver);

					role=pc.pullRRole;
				}

				pc.setSOFAReference(new Reference(dd.impl,pc.receiver.getProfiles()));
				pc.setDeploymentDescriptor(dd);
			} catch (Exception e) {
				throw new ConnectorException("Can't instantiate receiverRole.",e);
			}
		} else {
			throw new ConnectorException("Unknown unit '"+dd.unit+"'.");
		}
		return role;
	}

	public void link(Object target) throws LinkException {
		Reference ref=(Reference)target;
		String[] profileNames=sender.getSupportedProfiles();
		int i, j;

		for (i=0;i<profileNames.length;i++) {
			for (j=0;j<ref.profiles.length;j++) {
				if (profileNames[i].equals(ref.profiles[j].tag)) {
					sender.link(ref.profiles[j]);
					return;
				}
			}
		}
	}

        public String getName() {
          return getConnectorName();
        }

}
