/* $Id: Reference.java,v 1.3 2004/05/20 14:23:50 bures Exp $ */
package SOFA.Connector;

import java.util.StringTokenizer;
import java.util.Vector;

import SOFA.Connector.ECG.Profiles.CORBAProfile;
import SOFA.Connector.ECG.Profiles.LocalProfile;
import SOFA.Connector.ECG.Profiles.RMIProfile;

/** Connector reference.
  */
public final class Reference implements java.io.Serializable, cz.cuni.sofa.lib.Streamable {
  
  /** Type of the connector - CSProcCall, EventPassing, DataStream */
  public String conn_type;
  /** Profiles */
  public TaggedProfile[] profiles;

  public Reference () {
  }

  public Reference (final String conn_type, final TaggedProfile[] profiles) {
    this.conn_type = conn_type;
    this.profiles = profiles;
  }

  public void _write(cz.cuni.sofa.lib.OutputStream os) throws java.io.IOException {
    os.write_string(conn_type);
    int len = 0;
    for (int i=0; i<profiles.length; i++) {
      if (profiles[i] instanceof RMIProfile || profiles[i] instanceof CORBAProfile) {
        len++;
      }
    }
    os.write_long(len);
    for (int i=0; i<profiles.length; i++) {
      if (profiles[i] instanceof RMIProfile) {
        os.write_string(profiles[i].tag);
        os.write_string( ((RMIProfile) profiles[i]).URL);
      }
      if (profiles[i] instanceof CORBAProfile) {
        os.write_string(profiles[i].tag);
        os.write_string( ((CORBAProfile) profiles[i]).IOR);
      }
    }
  }
  
  public void _read(cz.cuni.sofa.lib.InputStream is) throws java.io.IOException {
    conn_type = is.read_string();
    int len = is.read_long();
    profiles = new TaggedProfile [len];
    for (int i=0; i<len; i++) {
      String tag = is.read_string();
      if (tag.compareTo("RMI")==0) {
        profiles[i] = new RMIProfile(is.read_string());
      }
      if (tag.compareTo("CORBA")==0) {
        profiles[i] = new CORBAProfile(is.read_string());
      }
    }
  }

  public String toString() {
    StringBuffer sb = new StringBuffer(conn_type);
    sb.append("-");
    for (int i=0; i<profiles.length; i++) {
      if (profiles[i] instanceof RMIProfile) {
        sb.append(profiles[i].tag);
        sb.append(";");
        sb.append( ((RMIProfile) profiles[i]).URL);
        sb.append("-");
      }
      if (profiles[i] instanceof CORBAProfile) {
        sb.append(profiles[i].tag);
        sb.append("x"); 
        sb.append( ((CORBAProfile) profiles[i]).IOR);
        sb.append("-");
      }
      if (profiles[i] instanceof LocalProfile) {
        sb.append(profiles[i].tag);
        sb.append("-");
      }

    }
    return sb.toString();
  }

  /**
   * Reconstructs a reference from a string previously created by the {@link #toString()} method.
   * Added by PP.
   * @param s a string describing the reference in a defined format.
   * @return the reference being described by the string.
   */
  public static Reference fromString (String s) {
    StringTokenizer st = new StringTokenizer(s, "-");
    String type = st.nextToken();
    Vector profiles = new Vector();

    while (st.hasMoreTokens()) {
      String token = st.nextToken();
      if (token.equals("LOCAL")) {
        profiles.add(new LocalProfile());
        continue;
      }
      if (token.startsWith("RMI")) {
        String url = token.substring(4); // 4 == "RMI;".length(), see #toString()
        profiles.add(new RMIProfile(url));
        continue;
      }
      if (token.startsWith("CORBA")) {
        String ior = token.substring(6); // 6 == "CORBAx".length(), see #toString()
        profiles.add(new CORBAProfile(ior));
        continue;
      }
    }
    return new Reference(type, (TaggedProfile [])profiles.toArray(new TaggedProfile[profiles.size()]));
  }

}
