/*
 * RMIAdminCenter.java
 *
 * Created on 10. kvten 2004, 20:38
 */

package SOFA.SOFAnet.Admin.RMI;

import SOFA.SOFAnet.Core.*;
import SOFA.SOFAnet.Repository.Repository;
import SOFA.SOFAnet.Repository.NodeInfo;
import SOFA.SOFAnet.TR.TRInterface;
import SOFA.SOFAnet.Search.RMI.RMISearchCenter;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.net.MalformedURLException;

/**
 * Managing class of RMI implementation of "admin interface" of the SOFAnet.
 * This interface is used by SOFA runtime.
 *
 * @author  Ladislav Sobr
 */
public class RMIAdminCenter
{
  private TRInterface tr;
  private Repository rep;
  private LocalOps localOps;
  private NetOps netOps;
  private ShareOps shareOps;
  private SearchOps searchOps;
  private RMISearchCenter rmiSearchCenter;

  private RMIAdminServer server;
  
  boolean started;
  private String myNodeName;
  
  /** Creates a new instance of RMIAdminCenter */
  public RMIAdminCenter()
  {
    started = false;
    server = null;
    myNodeName = NodeInfo.getLocalNodeName();
  }

  public void setRepository(Repository repository)
  {
    rep = repository;
  }
  
  public void setTRInterface(TRInterface tr)
  {
    this.tr = tr;
  }
  
  public void setLocalOps(LocalOps localOps)
  {
    this.localOps = localOps;
  }

  public void setNetOps(NetOps netOps)
  {
    this.netOps = netOps;
  }
  
  public void setShareOps(ShareOps shareOps)
  {
    this.shareOps = shareOps;
  }
  
  public void setSearchOps(SearchOps searchOps)
  {
    this.searchOps = searchOps;
  }

  public void setRMISearchCenter(RMISearchCenter rmiSearchCenter)
  {
    this.rmiSearchCenter = rmiSearchCenter;
  }
  
  /** Starts RMI administration subsystem */
  public void start() throws RMIAdminException
  {
    started = true;

    //run RMI admin server
    
    NodeInfo myNodeInfo = new NodeInfo();
    try
    {
      myNodeInfo.setNodeName(myNodeName);
    }
    catch (NodeInfo.InvalidNodeNameException e)
    {
      throw new RMIAdminException("Invalid name of admin SOFA node: " + myNodeName, e);
    }
    
    try
    {
      server = new RMIAdminServer(tr, rep, localOps, netOps, shareOps, searchOps, rmiSearchCenter);
      Naming.rebind("//" + myNodeInfo.getAddressAndPort() + "/SOFAnet/RMIAdminServer", server);
    }
    catch (RemoteException e)
    {
      throw new RMIAdminException("RMI admin server failed to start", e);
    }
    catch (MalformedURLException e)
    {
      throw new RMIAdminException("RMI admin server failed to start", e);
    }
  }  

  boolean isStarted()
  {
    return started;
  }
}
