/*
 * RMIAdminInterface.java
 *
 * Created on 10. kvten 2004, 20:39
 */

package SOFA.SOFAnet.Admin.RMI;

import java.rmi.Remote;
import java.rmi.RemoteException;
import SOFA.SOFAnode.TR.ComponentInfo;
import SOFA.SOFAnet.Repository.Licence;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.ShareGroups;
import SOFA.SOFAnet.Repository.BundleOffer;
import SOFA.SOFAnet.Repository.LocalInfo;
import SOFA.SOFAnet.Repository.BundleContent;
import SOFA.SOFAnet.Repository.InputTrigger;
import SOFA.SOFAnet.Repository.OutputTrigger;
import SOFA.SOFAnet.Repository.Contract;
import SOFA.SOFAnet.Search.SearchPattern;
import SOFA.SOFAnet.Search.SearchReplyItem;
import SOFA.SOFAnet.Search.RMI.RMISearchConfiguration;

/**
 *
 * @author  Ladislav Sobr
 */
public interface RMIAdminInterface extends Remote
{
  String[] binBundlesList() throws RemoteException;
  String[] licencesList() throws RemoteException;
  String[] localInfosList() throws RemoteException;
  LocalInfoDesc[] localInfoDescsList() throws RemoteException;
  ComponentInfo[] componentsInTRList() throws RemoteException;
  String[] bundleOffersList() throws RemoteException;
  String[] inputTriggersList() throws RemoteException;
  InputTriggerDesc[] inputTriggerDescsList() throws RemoteException;
  String[] outputTriggersList() throws RemoteException;
  OutputTriggerDesc[] outputTriggerDescsList() throws RemoteException;
  String[] contractsList() throws RemoteException;
  ContractDesc[] contractDescsList() throws RemoteException;
  
  Licence getLicence(String bundleName) throws RemoteException;
  void updateLicence(String bundleName, Licence licence) throws RMIAdminException, RemoteException;

  BundleOffer getBundleOffer(String bundleName) throws RemoteException;
  void deleteBundleOffer(String bundleName) throws RemoteException;

  LocalInfo getLocalInfo(String bundleName) throws RemoteException;
  
  BundleContent getInstalledBundleContent(String bundleName) throws RemoteException;
  BundleContent getSharedBundleContent(String bundleName) throws RemoteException;
  
  void deleteComponentsFromTR(ComponentInfo[] components, boolean interfaces, boolean inferiors) throws RMIAdminException, RemoteException;

  InputTrigger getInputTrigger(String name) throws RemoteException;
  void updateInputTrigger(String name, InputTrigger inputTrigger) throws RMIAdminException, RemoteException;
  
  OutputTrigger getOutputTrigger(String name) throws RemoteException;
  void updateOutputTrigger(String name, OutputTrigger outputTrigger) throws RMIAdminException, RemoteException;

  Contract getContract(String name) throws RemoteException;
  void updateContract(String name, Contract contract) throws RMIAdminException, RemoteException;

  RMISearchConfiguration getRMISearchConfiguration() throws RemoteException;
  void setRMISearchConfiguration(RMISearchConfiguration configuration) throws RemoteException;
  void applyRMISearchConfiguration() throws RemoteException;
  
  void installBundle(String bundleName, boolean reinstall) throws RMIAdminException, RemoteException;
  void uninstallBundle(String bundleName, boolean deleteComponents, boolean includingInterfaces, boolean force) throws RMIAdminException, RemoteException;
  void createBinaryBundle(String bundleName, boolean overwrite) throws RMIAdminException, RemoteException;
  void deleteBinaryBundle(String bundleName) throws RMIAdminException, RemoteException;
  void deleteBundle(String bundleName) throws RMIAdminException, RemoteException;
  void createUserBundle(String bundleName, ComponentInfo[] components, boolean subcomponents) throws RMIAdminException, RemoteException;
  String createComponentBundle(ComponentInfo component, boolean subcomponents) throws RMIAdminException, RemoteException;
  
  boolean testPush(String bundleName, String nodeName, boolean offer) throws RMIAdminException, RemoteException;
  boolean testPull(String bundleName, String nodeName, String contractID) throws RMIAdminException, RemoteException;
  void pushBundle(String bundleName, String nodeName, int numberOfLicences) throws RMIAdminException, RemoteException;
  void pullBundle(String bundleName, String nodeName, String contractID, boolean synchro) throws RMIAdminException, RemoteException;
  void pushBundleOffer(String bundleName, String nodeName) throws RMIAdminException, RemoteException;
  void createBundleOfferFromBinBundle(String bundleName, boolean overwrite) throws RMIAdminException, RemoteException;
  void generateTriggersOfContract(String contractID, boolean firstDelete) throws RMIAdminException, RemoteException;
  void deleteTriggersOfContract(String contractID) throws RMIAdminException, RemoteException;
  
  void shareBundle(String bundleName, int numberOfLicences, ShareGroups shareGroups, NodeNameFilter nodeFilter, boolean equality, boolean reshare) throws RMIAdminException, RemoteException;
  void unshareBundle(String bundleName, boolean recoverLicences, boolean force) throws RMIAdminException, RemoteException;
  void acquireSharedBundle(String bundleName, String nodeName, boolean tryNoShareClient) throws RMIAdminException, RemoteException;
  void stopShareClientOfBundle(String bundleName, boolean returnLicence, boolean force) throws RMIAdminException, RemoteException;
  public void makeShareClientsReturnBundle(String bundleName) throws RMIAdminException, RemoteException;
  
  SearchReplyItem[] search(SearchPattern searchPattern, boolean firstOnly, String shareGroup, int timeout) throws RMIAdminException, RemoteException;
}
