/*
 * RMIAdminServer.java
 *
 * Created on 10. kvten 2004, 20:40
 */

package SOFA.SOFAnet.Admin.RMI;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.server.ServerNotActiveException;
import java.rmi.RemoteException;
import SOFA.SOFAnet.Core.*;
import SOFA.SOFAnet.Repository.*;
import SOFA.SOFAnet.Search.*;
import SOFA.SOFAnet.TR.TRInterface;
import SOFA.SOFAnet.Search.RMI.RMISearchCenter;
import SOFA.SOFAnet.Search.RMI.RMISearchConfiguration;
import SOFA.SOFAnode.TR.ComponentInfo;
import SOFA.SOFAnode.TR.TRException;
import java.util.*;

/**
 * RMI server that implemets "Admin interface" - AdminInterface - for Node Browser
 *
 * @author  Ladislav Sobr
 */
public class RMIAdminServer extends UnicastRemoteObject implements RMIAdminInterface
{
  private TRInterface tr;
  private Repository rep;
  private LocalOps localOps;
  private NetOps netOps;
  private ShareOps shareOps;
  private SearchOps searchOps;
  private RMISearchCenter rmiSearchCenter;
  
  /** Creates a new instance of RMIAdminServer */
  public RMIAdminServer(TRInterface tr, Repository rep, LocalOps localOps, NetOps netOps, ShareOps shareOps, SearchOps searchOps, RMISearchCenter rmiSearchCenter) throws RemoteException
  {
    this.tr = tr;
    this.rep = rep;
    this.localOps = localOps;
    this.netOps = netOps;
    this.shareOps = shareOps;
    this.searchOps = searchOps;
    this.rmiSearchCenter = rmiSearchCenter;
  }
  
  public String[] binBundlesList() throws RemoteException
  {
    BinBundles binBundles = rep.getBinBundles();
    synchronized (binBundles)
    {
      return (String[])binBundles.map.keySet().toArray(new String[binBundles.map.size()]);
    } //binBundles
  }
  
  public String[] licencesList() throws RemoteException
  {
    Licences licences = rep.getLicences();
    synchronized (licences)
    {
      return (String[])licences.map.keySet().toArray(new String[licences.map.size()]);
    } //licences
  }
  
  public String[] localInfosList() throws RemoteException
  {
    LocalInfos localInfos = rep.getLocalInfos();
    synchronized (localInfos)
    {
      return (String[])localInfos.map.keySet().toArray(new String[localInfos.map.size()]);
    } //localInfos
  }
  
  public LocalInfoDesc[] localInfoDescsList() throws RemoteException
  {
    LocalInfos localInfos = rep.getLocalInfos();
    synchronized (localInfos)
    {
      LocalInfoDesc[] localInfoDescs = new LocalInfoDesc[localInfos.map.size()];
      Iterator it = localInfos.map.values().iterator();
      
      for (int i = 0; i < localInfoDescs.length; i++)
      {
        localInfoDescs[i] = new LocalInfoDesc((LocalInfo)it.next());
      }
      
      return localInfoDescs;
    } //localInfos
  }

  public ComponentInfo[] componentsInTRList() throws RemoteException
  {
    return tr.list().getComponents();
  }

  public String[] bundleOffersList() throws RemoteException
  {
    BundleOffers bundleOffers = rep.getBundleOffers();
    synchronized (bundleOffers)
    {
      return (String[])bundleOffers.map.keySet().toArray(new String[bundleOffers.map.size()]);
    } //bundleOffers
  }

  public String[] inputTriggersList() throws RemoteException
  {
    InputTriggers inputTriggers = rep.getInputTriggers();
    synchronized (inputTriggers)
    {
      return (String[])inputTriggers.map.keySet().toArray(new String[inputTriggers.map.size()]);
    } //inputTriggers
  }

  public InputTriggerDesc[] inputTriggerDescsList() throws RemoteException
  {
    InputTriggers inputTriggers = rep.getInputTriggers();
    synchronized (inputTriggers)
    {
      InputTriggerDesc[] inputTriggerDescs = new InputTriggerDesc[inputTriggers.map.size()];
      Iterator it = inputTriggers.map.values().iterator();
      
      for (int i = 0; i < inputTriggerDescs.length; i++)
      {
        inputTriggerDescs[i] = new InputTriggerDesc((InputTrigger)it.next());
      }
      
      return inputTriggerDescs;
    } //inputTriggers
  }
  
  public String[] outputTriggersList() throws RemoteException
  {
    OutputTriggers outputTriggers = rep.getOutputTriggers();
    synchronized (outputTriggers)
    {
      return (String[])outputTriggers.map.keySet().toArray(new String[outputTriggers.map.size()]);
    } //outputTriggers
  }

  public OutputTriggerDesc[] outputTriggerDescsList() throws RemoteException
  {
    OutputTriggers outputTriggers = rep.getOutputTriggers();
    synchronized (outputTriggers)
    {
      OutputTriggerDesc[] outputTriggerDescs = new OutputTriggerDesc[outputTriggers.map.size()];
      Iterator it = outputTriggers.map.values().iterator();
      
      for (int i = 0; i < outputTriggerDescs.length; i++)
      {
        outputTriggerDescs[i] = new OutputTriggerDesc((OutputTrigger)it.next());
      }
      
      return outputTriggerDescs;
    } //outputTriggers
  }
  
  public String[] contractsList() throws RemoteException
  {
    Contracts contracts = rep.getContracts();
    synchronized (contracts)
    {
      return (String[])contracts.map.keySet().toArray(new String[contracts.map.size()]);
    } //contracts
  }
  
  public ContractDesc[] contractDescsList() throws RemoteException
  {
    Contracts contracts = rep.getContracts();
    synchronized (contracts)
    {
      ContractDesc[] contractDescs = new ContractDesc[contracts.map.size()];
      Iterator it = contracts.map.values().iterator();
      
      for (int i = 0; i < contractDescs.length; i++)
      {
        contractDescs[i] = new ContractDesc((Contract)it.next());
      }
      
      return contractDescs;
    } //contracts
  }
  
  public Licence getLicence(String bundleName) throws RemoteException
  {
    Licences licences = rep.getLicences();
    return licences.getLicence(bundleName);
  }
  
  public void updateLicence(String bundleName, Licence licence) throws RMIAdminException, RemoteException
  {
    Licences licences = rep.getLicences();
    
    if (licence == null)
    {
      licences.deleteLicence(bundleName);
      return;
    }
    
    synchronized (licences)
    {
      Licence lic = licences.getLicence(bundleName);
      if (lic != null)
      {
        lic.setType(licence.getType());
        lic.setValidFrom(licence.getValidFrom());
        lic.setValidTo(licence.getValidTo());
        lic.setNumberOfCopies(licence.getNumberOfCopies());
        lic.setText(licence.getText());
        lic.saveToStorage();
      }
      else
      {
        lic = licences.addLicence(bundleName, licence.getType(), licence.getNumberOfCopies(), licence.getValidFrom(), licence.getValidTo(), licence.getText());
        if (lic == null) throw new RMIAdminException("Cannot add Licence File for bundle '" + bundleName + "'");
      }
    } //licences
  }

  public BundleOffer getBundleOffer(String bundleName) throws RemoteException
  {
    BundleOffers bundleOffers = rep.getBundleOffers();
    return bundleOffers.getBundleOffer(bundleName);
  }
  
  public void deleteBundleOffer(String bundleName) throws RemoteException
  {
    BundleOffers bundleOffers = rep.getBundleOffers();
    bundleOffers.deleteBundleOffer(bundleName);
  }
  
  public LocalInfo getLocalInfo(String bundleName) throws RemoteException
  {
    LocalInfos localInfos = rep.getLocalInfos();
    return localInfos.getLocalInfo(bundleName);
  }

  public BundleContent getInstalledBundleContent(String bundleName) throws RemoteException
  {
    BundleContents installedBundleContents = rep.getInstalledBundleContents();
    return installedBundleContents.getBundleContent(bundleName);
  }
  
  public BundleContent getSharedBundleContent(String bundleName) throws RemoteException
  {
    BundleContents sharedBundleContents = rep.getSharedBundleContents();
    return sharedBundleContents.getBundleContent(bundleName);
  }

  public void deleteComponentsFromTR(ComponentInfo[] components, boolean interfaces, boolean inferiors) throws RMIAdminException, RemoteException
  {
    try
    {
      tr.deleteComponents(components, interfaces, inferiors);
    }
    catch (TRException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public InputTrigger getInputTrigger(String name) throws RemoteException
  {
    InputTriggers inputTriggers = rep.getInputTriggers();
    return inputTriggers.getInputTrigger(name);
  }  
  
  public void updateInputTrigger(String name, InputTrigger inputTrigger) throws RMIAdminException, RemoteException
  {
    InputTriggers inputTriggers = rep.getInputTriggers();
    
    if (inputTrigger == null)
    {
      inputTriggers.deleteInputTrigger(name);
      return;
    }
    
    synchronized (inputTriggers)
    {
      InputTrigger iTrigger = inputTriggers.getInputTrigger(name);
      if (iTrigger != null)
      {
        iTrigger.setSource(inputTrigger.getSource());
        iTrigger.setBundleType(inputTrigger.getBundleType());
        iTrigger.setDescription(inputTrigger.getDescription());
        iTrigger.setAutoDelete(inputTrigger.getAutoDelete());
        iTrigger.setBundleFilter(inputTrigger.getBundleFilter());
        iTrigger.setNodeFilter(inputTrigger.getNodeFilter());
        iTrigger.setValidFrom(inputTrigger.getValidFrom());
        iTrigger.setValidTo(inputTrigger.getValidTo());
        if (inputTrigger.isContract()) iTrigger.setContract(inputTrigger.getContractID(), inputTrigger.getContractRule());
        else iTrigger.setContract(null, null);
        iTrigger.setActions(inputTrigger.getActions());
        iTrigger.setAsNoLicences(inputTrigger.getAsNoLicences());
        iTrigger.setAsShareGroups(inputTrigger.getAsShareGroups());
        iTrigger.setAsNodeFilter(inputTrigger.getAsNodeFilter());
        iTrigger.setAsEquality(inputTrigger.getAsEquality());
        iTrigger.setAlpOutputTrigger(inputTrigger.getAlpOutputTrigger());
        iTrigger.setApNodeNameLicList(inputTrigger.getApNodeNameLicList());
        iTrigger.setApoNodeNameList(inputTrigger.getApoNodeNameList());
        iTrigger.saveToStorage();
      }
      else
      {
        iTrigger = inputTriggers.addInputTrigger(
              name,
              true,
              inputTrigger.getSource(),
              inputTrigger.getBundleType(),
              inputTrigger.getDescription(),
              inputTrigger.getAutoDelete(),
              inputTrigger.getBundleFilter(),
              inputTrigger.getNodeFilter(),
              inputTrigger.getValidFrom(),
              inputTrigger.getValidTo(),
              inputTrigger.isContract() ? inputTrigger.getContractID() : null,
              inputTrigger.isContract() ? inputTrigger.getContractRule() : null,
              inputTrigger.getActions(),
              inputTrigger.getAsNoLicences(),
              inputTrigger.getAsShareGroups(),
              inputTrigger.getAsNodeFilter(),
              inputTrigger.getAsEquality(),
              inputTrigger.getAlpOutputTrigger(),
              inputTrigger.getApNodeNameLicList(),
              inputTrigger.getApoNodeNameList());
        
        if (iTrigger == null) throw new RMIAdminException("Cannot add Input Trigger '" + name + "'");
      }
    } //inputTriggers
  }
  
  public OutputTrigger getOutputTrigger(String name) throws RemoteException
  {
    OutputTriggers outputTriggers = rep.getOutputTriggers();
    return outputTriggers.getOutputTrigger(name);
  }
  
  public void updateOutputTrigger(String name, OutputTrigger outputTrigger) throws RMIAdminException, RemoteException
  {
    OutputTriggers outputTriggers = rep.getOutputTriggers();
    
    if (outputTrigger == null)
    {
      outputTriggers.deleteOutputTrigger(name);
      return;
    }
    
    synchronized (outputTriggers)
    {
      OutputTrigger oTrigger = outputTriggers.getOutputTrigger(name);
      if (oTrigger != null)
      {
        oTrigger.setDescription(outputTrigger.getDescription());
        oTrigger.setAutoDelete(outputTrigger.getAutoDelete());
        oTrigger.setBundleFilter(outputTrigger.getBundleFilter());
        oTrigger.setNodeFilter(outputTrigger.getNodeFilter());
        oTrigger.setValidFrom(outputTrigger.getValidFrom());
        oTrigger.setValidTo(outputTrigger.getValidTo());
        if (outputTrigger.isContract()) oTrigger.setContract(outputTrigger.getContractID(), outputTrigger.getContractRule());
        else oTrigger.setContract(null, null);
        oTrigger.saveToStorage();
      }
      else
      {
        oTrigger = outputTriggers.addOutputTrigger(
              name,
              true,
              outputTrigger.getDescription(),
              outputTrigger.getAutoDelete(),
              outputTrigger.getBundleFilter(),
              outputTrigger.getNodeFilter(),
              outputTrigger.getValidFrom(),
              outputTrigger.getValidTo(),
              outputTrigger.getContractID(),
              outputTrigger.getContractRule());
        
        if (oTrigger == null) throw new RMIAdminException("Cannot add Output Trigger '" + name + "'");
      }
    } //outputTriggers
  }

  public Contract getContract(String name) throws RemoteException
  {
    Contracts contracts = rep.getContracts();
    return contracts.getContract(name);
  }
  
  public void updateContract(String name, Contract contract) throws RMIAdminException, RemoteException
  {
    Contracts contracts = rep.getContracts();
    
    if (contract == null)
    {
      contracts.deleteContract(name);
      return;
    }
    
    synchronized (contracts)
    {
      Contract myContract = contracts.getContract(name);
      if (myContract != null)
      {
        myContract.setContractID(contract.getContractID());
        myContract.setDescription(contract.getDescription());
        myContract.setAutoDelete(contract.getAutoDelete());
        myContract.setValidFrom(contract.getValidFrom());
        myContract.setValidTo(contract.getValidTo());
        myContract.setDefaultPushDestinationNodes(contract.getDefaultPushDestinationNodes());
        myContract.setDefaultPushNodeFilter(contract.getDefaultPushNodeFilter());
        myContract.setDefaultPullNodeFilter(contract.getDefaultPullNodeFilter());
        myContract.setDefaultLicenceText(contract.getDefaultLicenceText());
        myContract.getRules().clear();
        myContract.getRules().putAll(contract.getRules());
        myContract.saveToStorage();
      }
      else
      {
        myContract = contracts.addContract(
              name,
              true,
              contract.getDescription(),
              contract.getAutoDelete(),
              contract.getValidFrom(),
              contract.getValidTo(),
              contract.getDefaultPushDestinationNodes(),
              contract.getDefaultPushNodeFilter(), 
              contract.getDefaultPullNodeFilter(), 
              contract.getDefaultLicenceText(),
              contract.getRules().values());
        
        if (myContract == null) throw new RMIAdminException("Cannot add Contract '" + name + "'");
      }
    } //contracts
  }

  public RMISearchConfiguration getRMISearchConfiguration() throws RemoteException
  {
    return rmiSearchCenter.getConfiguration();
  }
  public void setRMISearchConfiguration(RMISearchConfiguration configuration) throws RemoteException
  {
    rmiSearchCenter.setConfiguration(configuration);
  }

  public void applyRMISearchConfiguration() throws RemoteException
  {
    rmiSearchCenter.applyConfiguration();
  }
  
  public void installBundle(String bundleName, boolean reinstall) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.installBundle(bundleName, reinstall);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void uninstallBundle(String bundleName, boolean deleteComponents, boolean includingInterfaces, boolean force) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.uninstallBundle(bundleName, deleteComponents, includingInterfaces, force);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void createBinaryBundle(String bundleName, boolean overwrite) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.createBinaryBundle(bundleName, overwrite);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void deleteBinaryBundle(String bundleName) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.deleteBinaryBundle(bundleName);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
 
  public void deleteBundle(String bundleName) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.deleteBundle(bundleName);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void createUserBundle(String bundleName, ComponentInfo[] components, boolean subcomponents) throws RMIAdminException, RemoteException
  {
    try
    {
      localOps.createUserBundle(bundleName, components, subcomponents);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public String createComponentBundle(ComponentInfo component, boolean subcomponents) throws RMIAdminException, RemoteException
  {
    try
    {
      if (subcomponents) return localOps.createCompleteComponentBundle(component);
      else return localOps.createSingleComponentBundle(component);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public boolean testPush(String bundleName, String nodeName, boolean offer) throws RMIAdminException, RemoteException
  {
    try
    {
      return netOps.testPush(bundleName, nodeName, offer);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public boolean testPull(String bundleName, String nodeName, String contractID) throws RMIAdminException, RemoteException
  {
    try
    {
      return netOps.testPull(bundleName, nodeName, contractID);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void pushBundle(String bundleName, String nodeName, int numberOfLicences) throws RMIAdminException, RemoteException
  {
    try
    {
      netOps.pushBundle(bundleName, nodeName, numberOfLicences);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void pullBundle(String bundleName, String nodeName, String contractID, boolean synchro) throws RMIAdminException, RemoteException
  {
    try
    {
      if (synchro) netOps.synchroPullBundle(bundleName, nodeName, contractID);
      else netOps.pullBundle(bundleName, nodeName, contractID);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void pushBundleOffer(String bundleName, String nodeName) throws RMIAdminException, RemoteException
  {
    try
    {
      netOps.pushBundleOffer(bundleName, nodeName);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void createBundleOfferFromBinBundle(String bundleName, boolean overwrite) throws RMIAdminException, RemoteException
  {
    try
    {
      netOps.createBundleOfferFromBinBundle(bundleName, overwrite);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void generateTriggersOfContract(String contractID, boolean firstDelete) throws RMIAdminException, RemoteException
  {
    try
    {
      netOps.generateTriggersOfContract(contractID, firstDelete);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void deleteTriggersOfContract(String contractID) throws RMIAdminException, RemoteException
  {
    try
    {
      netOps.deleteTriggersOfContract(contractID);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void shareBundle(String bundleName, int numberOfLicences, ShareGroups shareGroups, NodeNameFilter nodeFilter, boolean equality, boolean reshare) throws RMIAdminException, RemoteException
  {
    try
    {
      shareOps.shareBundle(bundleName, numberOfLicences, shareGroups, nodeFilter, equality, reshare);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void unshareBundle(String bundleName, boolean recoverLicences, boolean force) throws RMIAdminException, RemoteException
  {
    try
    {
      shareOps.stopSharingBundle(bundleName, recoverLicences, force);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void acquireSharedBundle(String bundleName, String nodeName, boolean tryNoShareClient) throws RMIAdminException, RemoteException
  {
    try
    {
      shareOps.acquireSharedBundle(bundleName, nodeName, tryNoShareClient);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public void stopShareClientOfBundle(String bundleName, boolean returnLicence, boolean force) throws RMIAdminException, RemoteException
  {
    try
    {
      shareOps.stopShareClientOfBundle(bundleName, returnLicence, force);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }

  public void makeShareClientsReturnBundle(String bundleName) throws RMIAdminException, RemoteException
  {
    try
    {
      shareOps.makeShareClientsReturnBundle(bundleName);
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
  public SearchReplyItem[] search(SearchPattern searchPattern, boolean firstOnly, String shareGroup, int timeout) throws RMIAdminException, RemoteException
  {
    try
    {
      List replies = searchOps.search(searchPattern, firstOnly, shareGroup, timeout);
      if (replies != null && replies.size() > 0)
      {
        return (SearchReplyItem[])replies.toArray(new SearchReplyItem[replies.size()]);
      }
      else return null;
    }
    catch (CoreException e)
    {
      throw new RMIAdminException(e.getMessage(), e);
    }
  }
  
}
