/*
 * ContractEditDialog.java
 *
 * Created on 17. kvten 2004, 22:47
 */

package SOFA.SOFAnet.Browser;

import SOFA.SOFAnet.Repository.Contract;
import SOFA.SOFAnet.Repository.ContractRule;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.NodeNameLicList;
import java.util.*;
import java.text.DateFormat;
import java.text.ParseException;
import javax.swing.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class ContractEditDialog extends javax.swing.JDialog
{
  private Contract contract;
  private NodeNameLicList defaultPushDestinationNodes;
  private NodeNameFilter defaultPushNodeFilter;
  private NodeNameFilter defaultPullNodeFilter;
  private String defaultLicenceText;
  private DefaultListModel content;
  private boolean isOK;
  
  public class ContractRuleHolder
  {
    public ContractRule contractRule;

    /** Creates a new instance of ContractRuleHolder */
    public ContractRuleHolder(ContractRule contractRule)
    {
      this.contractRule = contractRule;
    }

    public String toString()
    {
      if (contractRule.getDescription().length() == 0) return contractRule.getName();
      else return contractRule.getName() + " (" + contractRule.getDescription() + ")";
    }
  }
  
  public static boolean showDialog(Component frameComp, Contract contract, String title)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    ContractEditDialog dialog = new ContractEditDialog(frame, true, contract, title);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form ContractEditDialog */
  public ContractEditDialog(java.awt.Frame parent, boolean modal, Contract contract, String title)
  {
    super(parent, modal);
    this.contract = contract;
    content = new DefaultListModel();
    defaultPushDestinationNodes = contract.getDefaultPushDestinationNodes();
    defaultPushNodeFilter = contract.getDefaultPushNodeFilter();
    defaultPullNodeFilter = contract.getDefaultPullNodeFilter();
    defaultLicenceText = contract.getDefaultLicenceText();
    isOK = false;

    content.ensureCapacity(contract.getRules().size());
    Iterator it = contract.getRules().values().iterator();
    while (it.hasNext()) content.addElement(new ContractRuleHolder((ContractRule)((ContractRule)it.next()).clone()));
    
    initComponents();
    setTitle(title);
    
    nameTextField.setText(contract.getName());
    descriptionTextField.setText(contract.getDescription());

    if (defaultPushNodeFilter != null)
    {
      defaultPushNodeFilterCheckBox.setSelected(true);
      defaultPushNodeFilterButton.setEnabled(true);
    }
    else
    {
      defaultPushNodeFilterCheckBox.setSelected(false);
      defaultPushNodeFilterButton.setEnabled(false);
    }

    if (defaultPushDestinationNodes != null)
    {
      defaultPushDestinationNodesCheckBox.setSelected(true);
      defaultPushDestinationNodesButton.setEnabled(true);
    }
    else
    {
      defaultPushDestinationNodesCheckBox.setSelected(false);
      defaultPushDestinationNodesButton.setEnabled(false);
    }
    
    if (defaultPullNodeFilter != null)
    {
      defaultPullNodeFilterCheckBox.setSelected(true);
      defaultPullNodeFilterButton.setEnabled(true);
    }
    else
    {
      defaultPullNodeFilterCheckBox.setSelected(false);
      defaultPullNodeFilterButton.setEnabled(false);
    }

    if (defaultLicenceText.length() != 0)
    {
      defaultLicenceTextCheckBox.setSelected(true);
      defaultLicenceTextButton.setEnabled(true);
    }
    else
    {
      defaultLicenceTextCheckBox.setSelected(false);
      defaultLicenceTextButton.setEnabled(false);
    }

    if (defaultPushDestinationNodes != null) defaultPushDestinationNodes = (NodeNameLicList)defaultPushDestinationNodes.clone();
    else defaultPushDestinationNodes = new NodeNameLicList();
    if (defaultPushNodeFilter != null) defaultPushNodeFilter = (NodeNameFilter)defaultPushNodeFilter.clone();
    else defaultPushNodeFilter = new NodeNameFilter();
    if (defaultPullNodeFilter != null) defaultPullNodeFilter = (NodeNameFilter)defaultPullNodeFilter.clone();
    else defaultPullNodeFilter = new NodeNameFilter();
    
    Date validFrom = contract.getValidFrom();
    Date validTo = contract.getValidTo();
    DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);

    if (validFrom != null) validFromTextField.setText(dateFormat.format(validFrom));
    else validFromTextField.setText("");

    if (validTo != null) validToTextField.setText(dateFormat.format(validTo));
    else validToTextField.setText("");
    
    autodeleteCheckBox.setSelected(contract.getAutoDelete());
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    nameAndDescriptionPanel = new javax.swing.JPanel();
    nameLabel = new javax.swing.JLabel();
    nameTextField = new javax.swing.JTextField();
    descriptionLabel = new javax.swing.JLabel();
    descriptionTextField = new javax.swing.JTextField();
    jPanel7 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    parametersPanel = new javax.swing.JPanel();
    validFromPanel = new javax.swing.JPanel();
    validFromLabel = new javax.swing.JLabel();
    validFromTextField = new javax.swing.JTextField();
    validToPanel = new javax.swing.JPanel();
    validToLabel = new javax.swing.JLabel();
    validToTextField = new javax.swing.JTextField();
    autodeleteCheckBox = new javax.swing.JCheckBox();
    defaultsPanel = new javax.swing.JPanel();
    defaultPushNodeFilterPanel = new javax.swing.JPanel();
    defaultPushNodeFilterCheckBox = new javax.swing.JCheckBox();
    defaultPushNodeFilterButton = new javax.swing.JButton();
    defaultPushDestinationNodesPanel = new javax.swing.JPanel();
    defaultPushDestinationNodesCheckBox = new javax.swing.JCheckBox();
    defaultPushDestinationNodesButton = new javax.swing.JButton();
    defaultPullNodeFilterPanel = new javax.swing.JPanel();
    defaultPullNodeFilterCheckBox = new javax.swing.JCheckBox();
    defaultPullNodeFilterButton = new javax.swing.JButton();
    defaultLicenceTextPanel = new javax.swing.JPanel();
    defaultLicenceTextCheckBox = new javax.swing.JCheckBox();
    defaultLicenceTextButton = new javax.swing.JButton();
    jScrollPane1 = new javax.swing.JScrollPane();
    rulesList = new javax.swing.JList();
    jPanel2 = new javax.swing.JPanel();
    jPanel4 = new javax.swing.JPanel();
    editButton = new javax.swing.JButton();
    addButton = new javax.swing.JButton();
    deleteSelectedButton = new javax.swing.JButton();
    jPanel6 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    nameAndDescriptionPanel.setLayout(new java.awt.GridLayout(4, 0));

    nameAndDescriptionPanel.setBorder(new javax.swing.border.TitledBorder("Name (ID) and Description of Contract"));
    nameLabel.setText("Name (Contract ID):");
    nameAndDescriptionPanel.add(nameLabel);

    nameTextField.setEditable(false);
    nameAndDescriptionPanel.add(nameTextField);

    descriptionLabel.setText("Description:");
    nameAndDescriptionPanel.add(descriptionLabel);

    nameAndDescriptionPanel.add(descriptionTextField);

    getContentPane().add(nameAndDescriptionPanel, java.awt.BorderLayout.NORTH);

    jPanel7.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.GridLayout(1, 0));

    parametersPanel.setLayout(new java.awt.GridLayout(4, 0));

    parametersPanel.setBorder(new javax.swing.border.TitledBorder("Parameters"));
    validFromPanel.setLayout(new javax.swing.BoxLayout(validFromPanel, javax.swing.BoxLayout.X_AXIS));

    validFromLabel.setText("Valid From:");
    validFromPanel.add(validFromLabel);

    validFromPanel.add(validFromTextField);

    parametersPanel.add(validFromPanel);

    validToPanel.setLayout(new javax.swing.BoxLayout(validToPanel, javax.swing.BoxLayout.X_AXIS));

    validToLabel.setText("Valid To:");
    validToPanel.add(validToLabel);

    validToPanel.add(validToTextField);

    parametersPanel.add(validToPanel);

    autodeleteCheckBox.setText("Automatically Delete When Expired");
    parametersPanel.add(autodeleteCheckBox);

    jPanel3.add(parametersPanel);

    defaultsPanel.setLayout(new java.awt.GridLayout(4, 0));

    defaultsPanel.setBorder(new javax.swing.border.TitledBorder("Defaults"));
    defaultsPanel.setMinimumSize(new java.awt.Dimension(200, 111));
    defaultsPanel.setPreferredSize(new java.awt.Dimension(200, 111));
    defaultPushNodeFilterPanel.setLayout(new javax.swing.BoxLayout(defaultPushNodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    defaultPushNodeFilterCheckBox.setText("Push Node Filter");
    defaultPushNodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPushNodeFilterCheckBoxActionPerformed(evt);
      }
    });

    defaultPushNodeFilterPanel.add(defaultPushNodeFilterCheckBox);

    defaultPushNodeFilterButton.setText("Edit Push Node Filter");
    defaultPushNodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPushNodeFilterButtonActionPerformed(evt);
      }
    });

    defaultPushNodeFilterPanel.add(defaultPushNodeFilterButton);

    defaultsPanel.add(defaultPushNodeFilterPanel);

    defaultPushDestinationNodesPanel.setLayout(new javax.swing.BoxLayout(defaultPushDestinationNodesPanel, javax.swing.BoxLayout.X_AXIS));

    defaultPushDestinationNodesCheckBox.setText("Push Destination Nodes");
    defaultPushDestinationNodesCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPushDestinationNodesCheckBoxActionPerformed(evt);
      }
    });

    defaultPushDestinationNodesPanel.add(defaultPushDestinationNodesCheckBox);

    defaultPushDestinationNodesButton.setText("Edit Nodes");
    defaultPushDestinationNodesButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPushDestinationNodesButtonActionPerformed(evt);
      }
    });

    defaultPushDestinationNodesPanel.add(defaultPushDestinationNodesButton);

    defaultsPanel.add(defaultPushDestinationNodesPanel);

    defaultPullNodeFilterPanel.setLayout(new javax.swing.BoxLayout(defaultPullNodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    defaultPullNodeFilterCheckBox.setText("Pull Node Filter");
    defaultPullNodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPullNodeFilterCheckBoxActionPerformed(evt);
      }
    });

    defaultPullNodeFilterPanel.add(defaultPullNodeFilterCheckBox);

    defaultPullNodeFilterButton.setText("Edit Pull Node Filter");
    defaultPullNodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultPullNodeFilterButtonActionPerformed(evt);
      }
    });

    defaultPullNodeFilterPanel.add(defaultPullNodeFilterButton);

    defaultsPanel.add(defaultPullNodeFilterPanel);

    defaultLicenceTextPanel.setLayout(new javax.swing.BoxLayout(defaultLicenceTextPanel, javax.swing.BoxLayout.X_AXIS));

    defaultLicenceTextCheckBox.setText("Licence Text");
    defaultLicenceTextCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultLicenceTextCheckBoxActionPerformed(evt);
      }
    });

    defaultLicenceTextPanel.add(defaultLicenceTextCheckBox);

    defaultLicenceTextButton.setText("Edit Licence Text");
    defaultLicenceTextButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        defaultLicenceTextButtonActionPerformed(evt);
      }
    });

    defaultLicenceTextPanel.add(defaultLicenceTextButton);

    defaultsPanel.add(defaultLicenceTextPanel);

    jPanel3.add(defaultsPanel);

    jPanel7.add(jPanel3, java.awt.BorderLayout.NORTH);

    jScrollPane1.setMinimumSize(new java.awt.Dimension(260, 132));
    rulesList.setModel(content);
    rulesList.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(java.awt.event.MouseEvent evt)
      {
        rulesListMouseClicked(evt);
      }
    });

    jScrollPane1.setViewportView(rulesList);

    jPanel7.add(jScrollPane1, java.awt.BorderLayout.CENTER);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel4.setLayout(new java.awt.GridLayout(3, 0));

    jPanel4.setBorder(new javax.swing.border.TitledBorder("Contract Rules"));
    editButton.setText("Edit Rule");
    editButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        editButtonActionPerformed(evt);
      }
    });

    jPanel4.add(editButton);

    addButton.setText("Add New Rule");
    addButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        addButtonActionPerformed(evt);
      }
    });

    jPanel4.add(addButton);

    deleteSelectedButton.setText("Delete Selected");
    deleteSelectedButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        deleteSelectedButtonActionPerformed(evt);
      }
    });

    jPanel4.add(deleteSelectedButton);

    jPanel2.add(jPanel4, java.awt.BorderLayout.NORTH);

    jPanel7.add(jPanel2, java.awt.BorderLayout.EAST);

    getContentPane().add(jPanel7, java.awt.BorderLayout.CENTER);

    jPanel6.setPreferredSize(new java.awt.Dimension(500, 35));
    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel6.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel6.add(cancelButton);

    getContentPane().add(jPanel6, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void defaultPushNodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPushNodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_defaultPushNodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, defaultPushNodeFilter.getPatterns(), "Node Name Pattern", "Default Push Filter Editor");
  }//GEN-LAST:event_defaultPushNodeFilterButtonActionPerformed

  private void defaultPushNodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPushNodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_defaultPushNodeFilterCheckBoxActionPerformed
    defaultPushNodeFilterButton.setEnabled(defaultPushNodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_defaultPushNodeFilterCheckBoxActionPerformed

  private void rulesListMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_rulesListMouseClicked
  {//GEN-HEADEREND:event_rulesListMouseClicked
    if (evt.getClickCount() == 2)
    {
      editButton.doClick(); //emulate button click
    }
  }//GEN-LAST:event_rulesListMouseClicked

  private void deleteSelectedButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_deleteSelectedButtonActionPerformed
  {//GEN-HEADEREND:event_deleteSelectedButtonActionPerformed
    Object[] selected = rulesList.getSelectedValues();
    for (int i = 0; i < selected.length; i++)
    {
      content.removeElement(selected[i]);
    }
  }//GEN-LAST:event_deleteSelectedButtonActionPerformed

  private void addButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addButtonActionPerformed
  {//GEN-HEADEREND:event_addButtonActionPerformed
    String result = JOptionPane.showInputDialog(this, "New Contract Rule:", "");
    if (result != null)
    {
      result = result.trim();
      if (result.length() != 0)
      {
        content.addElement(new ContractRuleHolder(new ContractRule(result, contract)));
      }
      else
      {
        JOptionPane.showMessageDialog(this, "Empty Rule Name", "Error", JOptionPane.ERROR_MESSAGE);
      }
    }
  }//GEN-LAST:event_addButtonActionPerformed

  private void editButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_editButtonActionPerformed
  {//GEN-HEADEREND:event_editButtonActionPerformed
    ContractRuleHolder contractRuleHolder = (ContractRuleHolder)rulesList.getSelectedValue();
    if (contractRuleHolder == null)
    {
      JOptionPane.showMessageDialog(this, "No Contract Rule selected", "Warning", JOptionPane.WARNING_MESSAGE);
      return;
    }
    
    ContractRuleEditDialog.showDialog(this, contractRuleHolder.contractRule, "Contract Rule Editor");
  }//GEN-LAST:event_editButtonActionPerformed

  private void defaultLicenceTextCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultLicenceTextCheckBoxActionPerformed
  {//GEN-HEADEREND:event_defaultLicenceTextCheckBoxActionPerformed
    defaultLicenceTextButton.setEnabled(defaultLicenceTextCheckBox.isSelected());
  }//GEN-LAST:event_defaultLicenceTextCheckBoxActionPerformed

  private void defaultLicenceTextButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultLicenceTextButtonActionPerformed
  {//GEN-HEADEREND:event_defaultLicenceTextButtonActionPerformed
    String result = MemoEditDialog.showDialog(this, defaultLicenceText, "Default Licence Text", "Default Licence Text");
    if (result != null) defaultLicenceText = result;
  }//GEN-LAST:event_defaultLicenceTextButtonActionPerformed

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    Date validFrom = null;
    Date validTo = null;
    try
    {
      DateFormat dateFormat = DateFormat.getInstance();
      String validFromText = validFromTextField.getText().trim().toLowerCase();
      if (validFromText.length() != 0 && validFromText.compareTo("no limitation") != 0) validFrom = dateFormat.parse(validFromText);

      String validToText = validToTextField.getText().trim().toLowerCase();
      if (validToText.length() != 0 && validToText.compareTo("no limitation") != 0) validTo = dateFormat.parse(validToText);
    }
    catch (ParseException e)
    {
      JOptionPane.showMessageDialog(this, "Error occured while parsing 'Valid From' or 'Valid To':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    
    contract.setDescription(descriptionTextField.getText());
    

    if (!defaultPushDestinationNodesCheckBox.isSelected()) defaultPushDestinationNodes = null;
    if (!defaultPushNodeFilterCheckBox.isSelected()) defaultPushNodeFilter = null;
    if (!defaultPullNodeFilterCheckBox.isSelected()) defaultPullNodeFilter = null;
    if (!defaultLicenceTextCheckBox.isSelected()) defaultLicenceText = "";
    
    contract.setDefaultPushDestinationNodes(defaultPushDestinationNodes);
    contract.setDefaultPushNodeFilter(defaultPushNodeFilter);
    contract.setDefaultPullNodeFilter(defaultPullNodeFilter);
    contract.setDefaultLicenceText(defaultLicenceText);
    contract.setValidFrom(validFrom);
    contract.setValidTo(validTo);
    contract.setAutoDelete(autodeleteCheckBox.isSelected());
    
    Map rules = contract.getRules();
    rules.clear();
    Enumeration en = content.elements();
    while (en.hasMoreElements())
    {
      ContractRule contractRule = ((ContractRuleHolder)en.nextElement()).contractRule;
      rules.put(contractRule.getName(), contractRule);
    }
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void defaultPullNodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPullNodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_defaultPullNodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, defaultPullNodeFilter.getPatterns(), "Node Name Pattern", "Default Pull Filter Editor");
  }//GEN-LAST:event_defaultPullNodeFilterButtonActionPerformed

  private void defaultPullNodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPullNodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_defaultPullNodeFilterCheckBoxActionPerformed
    defaultPullNodeFilterButton.setEnabled(defaultPullNodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_defaultPullNodeFilterCheckBoxActionPerformed

  private void defaultPushDestinationNodesButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPushDestinationNodesButtonActionPerformed
  {//GEN-HEADEREND:event_defaultPushDestinationNodesButtonActionPerformed
    NodeNameLicListEditDialog.showDialog(this, defaultPushDestinationNodes, "Default Push Destination Nodes Editor");
  }//GEN-LAST:event_defaultPushDestinationNodesButtonActionPerformed

  private void defaultPushDestinationNodesCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_defaultPushDestinationNodesCheckBoxActionPerformed
  {//GEN-HEADEREND:event_defaultPushDestinationNodesCheckBoxActionPerformed
    defaultPushDestinationNodesButton.setEnabled(defaultPushDestinationNodesCheckBox.isSelected());
  }//GEN-LAST:event_defaultPushDestinationNodesCheckBoxActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JButton addButton;
  private javax.swing.JCheckBox autodeleteCheckBox;
  private javax.swing.JButton cancelButton;
  private javax.swing.JButton defaultLicenceTextButton;
  private javax.swing.JCheckBox defaultLicenceTextCheckBox;
  private javax.swing.JPanel defaultLicenceTextPanel;
  private javax.swing.JButton defaultPullNodeFilterButton;
  private javax.swing.JCheckBox defaultPullNodeFilterCheckBox;
  private javax.swing.JPanel defaultPullNodeFilterPanel;
  private javax.swing.JButton defaultPushDestinationNodesButton;
  private javax.swing.JCheckBox defaultPushDestinationNodesCheckBox;
  private javax.swing.JPanel defaultPushDestinationNodesPanel;
  private javax.swing.JButton defaultPushNodeFilterButton;
  private javax.swing.JCheckBox defaultPushNodeFilterCheckBox;
  private javax.swing.JPanel defaultPushNodeFilterPanel;
  private javax.swing.JPanel defaultsPanel;
  private javax.swing.JButton deleteSelectedButton;
  private javax.swing.JLabel descriptionLabel;
  private javax.swing.JTextField descriptionTextField;
  private javax.swing.JButton editButton;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel4;
  private javax.swing.JPanel jPanel6;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JScrollPane jScrollPane1;
  private javax.swing.JPanel nameAndDescriptionPanel;
  private javax.swing.JLabel nameLabel;
  private javax.swing.JTextField nameTextField;
  private javax.swing.JPanel parametersPanel;
  private javax.swing.JList rulesList;
  private javax.swing.JLabel validFromLabel;
  private javax.swing.JPanel validFromPanel;
  private javax.swing.JTextField validFromTextField;
  private javax.swing.JLabel validToLabel;
  private javax.swing.JPanel validToPanel;
  private javax.swing.JTextField validToTextField;
  // End of variables declaration//GEN-END:variables
  
}
