/*
 * InputTriggerEditDialog.java
 *
 * Created on 17. kvten 2004, 22:47
 */

package SOFA.SOFAnet.Browser;

import SOFA.SOFAnet.Repository.InputTrigger;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.BundleNameFilter;
import SOFA.SOFAnet.Repository.NodeNameList;
import SOFA.SOFAnet.Repository.OutputTrigger;
import SOFA.SOFAnet.Repository.NodeNameLicList;
import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import javax.swing.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class InputTriggerEditDialog extends javax.swing.JDialog
{
  private InputTrigger inputTrigger;
  private BundleNameFilter bundleFilter;
  private NodeNameFilter nodeFilter;
  private OutputTrigger alpOutputTrigger;
  private NodeNameLicList apNodeNameLicList;
  private InputTriggerShareActionPropertiesEditDialog.ActionShareParameters asParameters;  
  private NodeNameList apoNodeNameList;
  private boolean isOK;
  
  public static boolean showDialog(Component frameComp, InputTrigger inputTrigger, String title)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    InputTriggerEditDialog dialog = new InputTriggerEditDialog(frame, true, inputTrigger, title);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form InputTriggerEditDialog */
  public InputTriggerEditDialog(java.awt.Frame parent, boolean modal, InputTrigger inputTrigger, String title)
  {
    super(parent, modal);
    this.inputTrigger = inputTrigger;
    bundleFilter = inputTrigger.getBundleFilter();
    nodeFilter = inputTrigger.getNodeFilter();
    alpOutputTrigger = inputTrigger.getAlpOutputTrigger();
    apNodeNameLicList = inputTrigger.getApNodeNameLicList();
    apoNodeNameList = inputTrigger.getApoNodeNameList();
    asParameters = new InputTriggerShareActionPropertiesEditDialog.ActionShareParameters(inputTrigger);
    isOK = false;
    
    initComponents();
    setTitle(title);
    
    nameTextField.setText(inputTrigger.getName());
    descriptionTextField.setText(inputTrigger.getDescription());
    pushCheckBox.setSelected(inputTrigger.isPushSource());
    pullCheckBox.setSelected(inputTrigger.isPullSource());
    madeCheckBox.setSelected(inputTrigger.isMadeSource());
    switch (inputTrigger.getBundleType())
    {
    default:
    case InputTrigger.BUNDLETYPE_FULL:
      bundleCheckBox.setSelected(true);
      bundleOfferCheckBox.setSelected(false);
      break;
    case InputTrigger.BUNDLETYPE_OFFER:
      bundleCheckBox.setSelected(false);
      bundleOfferCheckBox.setSelected(true);
      break;
    case InputTrigger.BUNDLETYPE_ALL:
      bundleCheckBox.setSelected(true);
      bundleOfferCheckBox.setSelected(true);
      break;
    }

    if (bundleFilter != null)
    {
      bundleFilterCheckBox.setSelected(true);
      bundleFilterButton.setEnabled(true);
    }
    else
    {
      bundleFilterCheckBox.setSelected(false);
      bundleFilterButton.setEnabled(false);
    }
    
    if (nodeFilter != null)
    {
      nodeFilterCheckBox.setSelected(true);
      nodeFilterButton.setEnabled(true);
    }
    else
    {
      nodeFilterCheckBox.setSelected(false);
      nodeFilterButton.setEnabled(false);
    }

    if (bundleFilter != null) bundleFilter = (BundleNameFilter)bundleFilter.clone();
    else bundleFilter = new BundleNameFilter();
    if (nodeFilter != null) nodeFilter = (NodeNameFilter)nodeFilter.clone();
    else nodeFilter = new NodeNameFilter();
    if (apoNodeNameList != null) apoNodeNameList = (NodeNameList)apoNodeNameList.clone();
    else apoNodeNameList = new NodeNameList();
    if (alpOutputTrigger != null) alpOutputTrigger = (OutputTrigger)alpOutputTrigger.clone();
    else alpOutputTrigger = new OutputTrigger();
    if (apNodeNameLicList != null) apNodeNameLicList = (NodeNameLicList)apNodeNameLicList.clone();
    else apNodeNameLicList = new NodeNameLicList();
    
    Date validFrom = inputTrigger.getValidFrom();
    Date validTo = inputTrigger.getValidTo();
    DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);

    if (validFrom != null) validFromTextField.setText(dateFormat.format(validFrom));
    else validFromTextField.setText("");

    if (validTo != null) validToTextField.setText(dateFormat.format(validTo));
    else validToTextField.setText("");
    
    autodeleteCheckBox.setSelected(inputTrigger.getAutoDelete());
    
    if (inputTrigger.isContract())
    {
      contractIDTextField.setText(inputTrigger.getContractID());
      contractRuleTextField.setText(inputTrigger.getContractRule());
    }
    
    int actions = inputTrigger.getActions();

    if ((actions & InputTrigger.ACTION_INSTALL) != 0)
    {
      actionInstallCheckBox.setSelected(true);
    }
    else
    {
      actionInstallCheckBox.setSelected(false);
    }
    
    if ((actions & InputTrigger.ACTION_SHARE) != 0)
    {
      actionShareCheckBox.setSelected(true);
      editShareParametersButton.setEnabled(true);
    }
    else
    {
      actionShareCheckBox.setSelected(false);
      editShareParametersButton.setEnabled(false);
    }
    
    if ((actions & InputTrigger.ACTION_ALLOW_PULL) != 0)
    {
      actionAllowPullCheckBox.setSelected(true);
      alpOutputTriggerButton.setEnabled(true);
    }
    else
    {
      actionAllowPullCheckBox.setSelected(false);
      alpOutputTriggerButton.setEnabled(false);
    }
    
    if ((actions & InputTrigger.ACTION_PUSH) != 0)
    {
      actionPushCheckBox.setSelected(true);
      apNodeNameLicListButton.setEnabled(true);
    }
    else
    {
      actionPushCheckBox.setSelected(false);
      apNodeNameLicListButton.setEnabled(false);
    }
    
    if ((actions & InputTrigger.ACTION_PUSH_OFFER) != 0)
    {
      actionPushOfferCheckBox.setSelected(true);
      apoNodeNameListButton.setEnabled(true);
    }
    else
    {
      actionPushOfferCheckBox.setSelected(false);
      apoNodeNameListButton.setEnabled(false);
    }
    
    if ((actions & InputTrigger.ACTION_DELETE) != 0)
    {
      actionDeleteCheckBox.setSelected(true);
    }
    else
    {
      actionDeleteCheckBox.setSelected(false);
    }
    
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    nameAndDescriptionPanel = new javax.swing.JPanel();
    nameLabel = new javax.swing.JLabel();
    nameTextField = new javax.swing.JTextField();
    descriptionLabel = new javax.swing.JLabel();
    descriptionTextField = new javax.swing.JTextField();
    jPanel7 = new javax.swing.JPanel();
    jPanel1 = new javax.swing.JPanel();
    sourcePanel = new javax.swing.JPanel();
    pushCheckBox = new javax.swing.JCheckBox();
    pullCheckBox = new javax.swing.JCheckBox();
    madeCheckBox = new javax.swing.JCheckBox();
    bundleTypePanel = new javax.swing.JPanel();
    bundleCheckBox = new javax.swing.JCheckBox();
    bundleOfferCheckBox = new javax.swing.JCheckBox();
    jPanel2 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    parametersPanel = new javax.swing.JPanel();
    bundleFilterPanel = new javax.swing.JPanel();
    bundleFilterCheckBox = new javax.swing.JCheckBox();
    bundleFilterButton = new javax.swing.JButton();
    nodeFilterPanel = new javax.swing.JPanel();
    nodeFilterCheckBox = new javax.swing.JCheckBox();
    nodeFilterButton = new javax.swing.JButton();
    validFromPanel = new javax.swing.JPanel();
    validFromLabel = new javax.swing.JLabel();
    validFromTextField = new javax.swing.JTextField();
    validToPanel = new javax.swing.JPanel();
    validToLabel = new javax.swing.JLabel();
    validToTextField = new javax.swing.JTextField();
    autodeleteCheckBox = new javax.swing.JCheckBox();
    contractPanel = new javax.swing.JPanel();
    jPanel5 = new javax.swing.JPanel();
    contractIDLabel = new javax.swing.JLabel();
    contractIDTextField = new javax.swing.JTextField();
    contractRuleLabel = new javax.swing.JLabel();
    contractRuleTextField = new javax.swing.JTextField();
    jPanel4 = new javax.swing.JPanel();
    actionsPanel = new javax.swing.JPanel();
    actionInstallPanel = new javax.swing.JPanel();
    actionInstallCheckBox = new javax.swing.JCheckBox();
    actionSharePanel = new javax.swing.JPanel();
    actionShareCheckBox = new javax.swing.JCheckBox();
    editShareParametersButton = new javax.swing.JButton();
    actionAllowPullPanel = new javax.swing.JPanel();
    actionAllowPullCheckBox = new javax.swing.JCheckBox();
    alpOutputTriggerButton = new javax.swing.JButton();
    actionPushPanel = new javax.swing.JPanel();
    actionPushCheckBox = new javax.swing.JCheckBox();
    apNodeNameLicListButton = new javax.swing.JButton();
    actionPushOfferPanel = new javax.swing.JPanel();
    actionPushOfferCheckBox = new javax.swing.JCheckBox();
    apoNodeNameListButton = new javax.swing.JButton();
    actionDeleteCheckBox = new javax.swing.JCheckBox();
    jPanel6 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    nameAndDescriptionPanel.setLayout(new java.awt.GridLayout(4, 0));

    nameAndDescriptionPanel.setBorder(new javax.swing.border.TitledBorder("Name and Description of Input Trigger"));
    nameLabel.setText("Name:");
    nameAndDescriptionPanel.add(nameLabel);

    nameTextField.setEditable(false);
    nameAndDescriptionPanel.add(nameTextField);

    descriptionLabel.setText("Description:");
    nameAndDescriptionPanel.add(descriptionLabel);

    nameAndDescriptionPanel.add(descriptionTextField);

    getContentPane().add(nameAndDescriptionPanel, java.awt.BorderLayout.NORTH);

    jPanel7.setLayout(new java.awt.BorderLayout());

    jPanel1.setLayout(new java.awt.GridLayout(1, 0));

    sourcePanel.setBorder(new javax.swing.border.TitledBorder("Source"));
    pushCheckBox.setText("Push");
    sourcePanel.add(pushCheckBox);

    pullCheckBox.setText("Pull");
    sourcePanel.add(pullCheckBox);

    madeCheckBox.setText("Made");
    sourcePanel.add(madeCheckBox);

    jPanel1.add(sourcePanel);

    bundleTypePanel.setBorder(new javax.swing.border.TitledBorder("Bundle Type"));
    bundleCheckBox.setText("Bundle");
    bundleTypePanel.add(bundleCheckBox);

    bundleOfferCheckBox.setText("Bundle Offer");
    bundleTypePanel.add(bundleOfferCheckBox);

    jPanel1.add(bundleTypePanel);

    jPanel7.add(jPanel1, java.awt.BorderLayout.NORTH);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.GridLayout(1, 0));

    parametersPanel.setLayout(new java.awt.GridLayout(5, 0));

    parametersPanel.setBorder(new javax.swing.border.TitledBorder("Parameters"));
    bundleFilterPanel.setLayout(new javax.swing.BoxLayout(bundleFilterPanel, javax.swing.BoxLayout.X_AXIS));

    bundleFilterCheckBox.setText("Bundle Filter");
    bundleFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        bundleFilterCheckBoxActionPerformed(evt);
      }
    });

    bundleFilterPanel.add(bundleFilterCheckBox);

    bundleFilterButton.setText("Edit Bundle Filter");
    bundleFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        bundleFilterButtonActionPerformed(evt);
      }
    });

    bundleFilterPanel.add(bundleFilterButton);

    parametersPanel.add(bundleFilterPanel);

    nodeFilterPanel.setLayout(new javax.swing.BoxLayout(nodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    nodeFilterCheckBox.setText("Node Filter");
    nodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterCheckBoxActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterCheckBox);

    nodeFilterButton.setText("Edit Node Filter");
    nodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterButtonActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterButton);

    parametersPanel.add(nodeFilterPanel);

    validFromPanel.setLayout(new javax.swing.BoxLayout(validFromPanel, javax.swing.BoxLayout.X_AXIS));

    validFromLabel.setText("Valid From:");
    validFromPanel.add(validFromLabel);

    validFromPanel.add(validFromTextField);

    parametersPanel.add(validFromPanel);

    validToPanel.setLayout(new javax.swing.BoxLayout(validToPanel, javax.swing.BoxLayout.X_AXIS));

    validToLabel.setText("Valid To:");
    validToPanel.add(validToLabel);

    validToPanel.add(validToTextField);

    parametersPanel.add(validToPanel);

    autodeleteCheckBox.setText("Automatically Delete When Expired");
    parametersPanel.add(autodeleteCheckBox);

    jPanel3.add(parametersPanel);

    contractPanel.setLayout(new java.awt.BorderLayout());

    contractPanel.setBorder(new javax.swing.border.TitledBorder("Contract Connection"));
    contractPanel.setMinimumSize(new java.awt.Dimension(200, 111));
    contractPanel.setPreferredSize(new java.awt.Dimension(200, 111));
    jPanel5.setLayout(new java.awt.GridLayout(4, 0));

    contractIDLabel.setText("Contract ID:");
    jPanel5.add(contractIDLabel);

    jPanel5.add(contractIDTextField);

    contractRuleLabel.setText("Contract Rule:");
    jPanel5.add(contractRuleLabel);

    jPanel5.add(contractRuleTextField);

    contractPanel.add(jPanel5, java.awt.BorderLayout.NORTH);

    jPanel3.add(contractPanel);

    jPanel2.add(jPanel3, java.awt.BorderLayout.NORTH);

    jPanel4.setLayout(new java.awt.BorderLayout());

    actionsPanel.setLayout(new java.awt.GridLayout(6, 0));

    actionsPanel.setBorder(new javax.swing.border.TitledBorder("Actions"));
    actionInstallPanel.setLayout(new javax.swing.BoxLayout(actionInstallPanel, javax.swing.BoxLayout.X_AXIS));

    actionInstallCheckBox.setText("Install Bundle");
    actionInstallPanel.add(actionInstallCheckBox);

    actionsPanel.add(actionInstallPanel);

    actionSharePanel.setLayout(new javax.swing.BoxLayout(actionSharePanel, javax.swing.BoxLayout.X_AXIS));

    actionShareCheckBox.setText("Share Bundle");
    actionShareCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        actionShareCheckBoxActionPerformed(evt);
      }
    });

    actionSharePanel.add(actionShareCheckBox);

    editShareParametersButton.setText("Edit Share Parameters");
    editShareParametersButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        editShareParametersButtonActionPerformed(evt);
      }
    });

    actionSharePanel.add(editShareParametersButton);

    actionsPanel.add(actionSharePanel);

    actionAllowPullPanel.setLayout(new javax.swing.BoxLayout(actionAllowPullPanel, javax.swing.BoxLayout.X_AXIS));

    actionAllowPullCheckBox.setText("Allow Pull");
    actionAllowPullCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        actionAllowPullCheckBoxActionPerformed(evt);
      }
    });

    actionAllowPullPanel.add(actionAllowPullCheckBox);

    alpOutputTriggerButton.setText("Edit Output Trigger");
    alpOutputTriggerButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        alpOutputTriggerButtonActionPerformed(evt);
      }
    });

    actionAllowPullPanel.add(alpOutputTriggerButton);

    actionsPanel.add(actionAllowPullPanel);

    actionPushPanel.setLayout(new javax.swing.BoxLayout(actionPushPanel, javax.swing.BoxLayout.X_AXIS));

    actionPushCheckBox.setText("Push Bundle");
    actionPushCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        actionPushCheckBoxActionPerformed(evt);
      }
    });

    actionPushPanel.add(actionPushCheckBox);

    apNodeNameLicListButton.setText("Edit Nodes & Licence Numbers");
    apNodeNameLicListButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        apNodeNameLicListButtonActionPerformed(evt);
      }
    });

    actionPushPanel.add(apNodeNameLicListButton);

    actionsPanel.add(actionPushPanel);

    actionPushOfferPanel.setLayout(new javax.swing.BoxLayout(actionPushOfferPanel, javax.swing.BoxLayout.X_AXIS));

    actionPushOfferCheckBox.setText("Push Bundle Offer");
    actionPushOfferCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        actionPushOfferCheckBoxActionPerformed(evt);
      }
    });

    actionPushOfferPanel.add(actionPushOfferCheckBox);

    apoNodeNameListButton.setText("Edit Nodes");
    apoNodeNameListButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        apoNodeNameListButtonActionPerformed(evt);
      }
    });

    actionPushOfferPanel.add(apoNodeNameListButton);

    actionsPanel.add(actionPushOfferPanel);

    actionDeleteCheckBox.setText("Delete Bundle");
    actionsPanel.add(actionDeleteCheckBox);

    jPanel4.add(actionsPanel, java.awt.BorderLayout.NORTH);

    jPanel2.add(jPanel4, java.awt.BorderLayout.CENTER);

    jPanel7.add(jPanel2, java.awt.BorderLayout.CENTER);

    getContentPane().add(jPanel7, java.awt.BorderLayout.CENTER);

    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel6.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel6.add(cancelButton);

    getContentPane().add(jPanel6, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    if (!bundleCheckBox.isSelected() && !bundleOfferCheckBox.isSelected())
    {
      JOptionPane.showMessageDialog(this, "Invalid Bundle Type - at least one option must be selected", "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    Date validFrom = null;
    Date validTo = null;
    try
    {
      DateFormat dateFormat = DateFormat.getInstance();
      String validFromText = validFromTextField.getText().trim().toLowerCase();
      if (validFromText.length() != 0 && validFromText.compareTo("no limitation") != 0) validFrom = dateFormat.parse(validFromText);

      String validToText = validToTextField.getText().trim().toLowerCase();
      if (validToText.length() != 0 && validToText.compareTo("no limitation") != 0) validTo = dateFormat.parse(validToText);
    }
    catch (ParseException e)
    {
      JOptionPane.showMessageDialog(this, "Error occured while parsing 'Valid From' or 'Valid To':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    String contractID = contractIDTextField.getText().trim();
    String contractRule = contractRuleTextField.getText().trim();
    if (contractID.length() != 0 && contractRule.length() == 0)
    {
      JOptionPane.showMessageDialog(this, "Missing Contract Rule", "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    inputTrigger.setDescription(descriptionTextField.getText());
    
    inputTrigger.setPushSource(pushCheckBox.isSelected());
    inputTrigger.setPullSource(pullCheckBox.isSelected());
    inputTrigger.setMadeSource(madeCheckBox.isSelected());
    if (bundleCheckBox.isSelected())
    {
      if (bundleOfferCheckBox.isSelected()) inputTrigger.setBundleType(InputTrigger.BUNDLETYPE_ALL);
      else inputTrigger.setBundleType(InputTrigger.BUNDLETYPE_FULL);
    }
    else
    {
      if (bundleOfferCheckBox.isSelected()) inputTrigger.setBundleType(InputTrigger.BUNDLETYPE_OFFER);
      else inputTrigger.setBundleType(InputTrigger.BUNDLETYPE_FULL);
    }

    if (!bundleFilterCheckBox.isSelected()) bundleFilter = null;
    if (!nodeFilterCheckBox.isSelected()) nodeFilter = null;
    inputTrigger.setBundleFilter(bundleFilter);
    inputTrigger.setNodeFilter(nodeFilter);
    inputTrigger.setValidFrom(validFrom);
    inputTrigger.setValidTo(validTo);
    inputTrigger.setAutoDelete(autodeleteCheckBox.isSelected());
    if (contractID.length() != 0) inputTrigger.setContract(contractID, contractRule);
    else inputTrigger.setContract(null, null);
    
    int actions = 0;
    if (actionInstallCheckBox.isSelected()) actions |= InputTrigger.ACTION_INSTALL;
    if (actionShareCheckBox.isSelected()) actions |= InputTrigger.ACTION_SHARE;
    else asParameters = null;
    if (actionAllowPullCheckBox.isSelected()) actions |= InputTrigger.ACTION_ALLOW_PULL;
    else alpOutputTrigger = null;
    if (actionPushCheckBox.isSelected()) actions |= InputTrigger.ACTION_PUSH;
    else apNodeNameLicList = null;
    if (actionPushOfferCheckBox.isSelected()) actions |= InputTrigger.ACTION_PUSH_OFFER;
    else apoNodeNameList = null;
    if (actionDeleteCheckBox.isSelected()) actions |= InputTrigger.ACTION_DELETE;
    
    inputTrigger.setActions(actions);
    
    if (asParameters != null) asParameters.save(inputTrigger);
    inputTrigger.setAlpOutputTrigger(alpOutputTrigger);
    inputTrigger.setApNodeNameLicList(apNodeNameLicList);
    inputTrigger.setApoNodeNameList(apoNodeNameList);
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void apoNodeNameListButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_apoNodeNameListButtonActionPerformed
  {//GEN-HEADEREND:event_apoNodeNameListButtonActionPerformed
    StringListEditDialog.showDialog(this, apoNodeNameList.getList(), "Node Name", "Node Name List");
  }//GEN-LAST:event_apoNodeNameListButtonActionPerformed

  private void actionPushOfferCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_actionPushOfferCheckBoxActionPerformed
  {//GEN-HEADEREND:event_actionPushOfferCheckBoxActionPerformed
    apoNodeNameListButton.setEnabled(actionPushOfferCheckBox.isSelected());
  }//GEN-LAST:event_actionPushOfferCheckBoxActionPerformed

  private void apNodeNameLicListButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_apNodeNameLicListButtonActionPerformed
  {//GEN-HEADEREND:event_apNodeNameLicListButtonActionPerformed
    NodeNameLicListEditDialog.showDialog(this, apNodeNameLicList, "Action Push Node Name-Licence List");
  }//GEN-LAST:event_apNodeNameLicListButtonActionPerformed

  private void actionPushCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_actionPushCheckBoxActionPerformed
  {//GEN-HEADEREND:event_actionPushCheckBoxActionPerformed
    apNodeNameLicListButton.setEnabled(actionPushCheckBox.isSelected());
  }//GEN-LAST:event_actionPushCheckBoxActionPerformed

  private void alpOutputTriggerButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_alpOutputTriggerButtonActionPerformed
  {//GEN-HEADEREND:event_alpOutputTriggerButtonActionPerformed
    OutputTriggerEditDialog.showDialog(this, alpOutputTrigger, "Allow Pull Action Output Trigger Editor");
  }//GEN-LAST:event_alpOutputTriggerButtonActionPerformed

  private void actionAllowPullCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_actionAllowPullCheckBoxActionPerformed
  {//GEN-HEADEREND:event_actionAllowPullCheckBoxActionPerformed
    alpOutputTriggerButton.setEnabled(actionAllowPullCheckBox.isSelected());
  }//GEN-LAST:event_actionAllowPullCheckBoxActionPerformed

  private void editShareParametersButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_editShareParametersButtonActionPerformed
  {//GEN-HEADEREND:event_editShareParametersButtonActionPerformed
    InputTriggerShareActionPropertiesEditDialog.showDialog(this, asParameters);
  }//GEN-LAST:event_editShareParametersButtonActionPerformed

  private void actionShareCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_actionShareCheckBoxActionPerformed
  {//GEN-HEADEREND:event_actionShareCheckBoxActionPerformed
    editShareParametersButton.setEnabled(actionShareCheckBox.isSelected());
  }//GEN-LAST:event_actionShareCheckBoxActionPerformed

  private void nodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_nodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, nodeFilter.getPatterns(), "Node Name Pattern", "Node Filter Editor");
  }//GEN-LAST:event_nodeFilterButtonActionPerformed

  private void nodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_nodeFilterCheckBoxActionPerformed
    nodeFilterButton.setEnabled(nodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_nodeFilterCheckBoxActionPerformed

  private void bundleFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bundleFilterButtonActionPerformed
  {//GEN-HEADEREND:event_bundleFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, bundleFilter.getPatterns(), "Bundle Name Pattern", "Bundle Filter Editor");
  }//GEN-LAST:event_bundleFilterButtonActionPerformed

  private void bundleFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bundleFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_bundleFilterCheckBoxActionPerformed
    bundleFilterButton.setEnabled(bundleFilterCheckBox.isSelected());
  }//GEN-LAST:event_bundleFilterCheckBoxActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JCheckBox actionAllowPullCheckBox;
  private javax.swing.JPanel actionAllowPullPanel;
  private javax.swing.JCheckBox actionDeleteCheckBox;
  private javax.swing.JCheckBox actionInstallCheckBox;
  private javax.swing.JPanel actionInstallPanel;
  private javax.swing.JCheckBox actionPushCheckBox;
  private javax.swing.JCheckBox actionPushOfferCheckBox;
  private javax.swing.JPanel actionPushOfferPanel;
  private javax.swing.JPanel actionPushPanel;
  private javax.swing.JCheckBox actionShareCheckBox;
  private javax.swing.JPanel actionSharePanel;
  private javax.swing.JPanel actionsPanel;
  private javax.swing.JButton alpOutputTriggerButton;
  private javax.swing.JButton apNodeNameLicListButton;
  private javax.swing.JButton apoNodeNameListButton;
  private javax.swing.JCheckBox autodeleteCheckBox;
  private javax.swing.JCheckBox bundleCheckBox;
  private javax.swing.JButton bundleFilterButton;
  private javax.swing.JCheckBox bundleFilterCheckBox;
  private javax.swing.JPanel bundleFilterPanel;
  private javax.swing.JCheckBox bundleOfferCheckBox;
  private javax.swing.JPanel bundleTypePanel;
  private javax.swing.JButton cancelButton;
  private javax.swing.JLabel contractIDLabel;
  private javax.swing.JTextField contractIDTextField;
  private javax.swing.JPanel contractPanel;
  private javax.swing.JLabel contractRuleLabel;
  private javax.swing.JTextField contractRuleTextField;
  private javax.swing.JLabel descriptionLabel;
  private javax.swing.JTextField descriptionTextField;
  private javax.swing.JButton editShareParametersButton;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel4;
  private javax.swing.JPanel jPanel5;
  private javax.swing.JPanel jPanel6;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JCheckBox madeCheckBox;
  private javax.swing.JPanel nameAndDescriptionPanel;
  private javax.swing.JLabel nameLabel;
  private javax.swing.JTextField nameTextField;
  private javax.swing.JButton nodeFilterButton;
  private javax.swing.JCheckBox nodeFilterCheckBox;
  private javax.swing.JPanel nodeFilterPanel;
  private javax.swing.JPanel parametersPanel;
  private javax.swing.JCheckBox pullCheckBox;
  private javax.swing.JCheckBox pushCheckBox;
  private javax.swing.JPanel sourcePanel;
  private javax.swing.JLabel validFromLabel;
  private javax.swing.JPanel validFromPanel;
  private javax.swing.JTextField validFromTextField;
  private javax.swing.JLabel validToLabel;
  private javax.swing.JPanel validToPanel;
  private javax.swing.JTextField validToTextField;
  // End of variables declaration//GEN-END:variables
  
}
