/*
 * LocalInfoDialog.java
 *
 * Created on 17. kvten 2004, 15:19
 */

package SOFA.SOFAnet.Browser;

import javax.swing.*;
import SOFA.SOFAnet.Repository.LocalInfo;
import SOFA.SOFAnet.Repository.BundleContent;
import SOFA.SOFAnet.Repository.Licence;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.ShareGroups;
import SOFA.SOFAnet.Repository.NodeNameList;
import java.util.List;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class LocalInfoDialog extends javax.swing.JDialog
{
  private LocalInfo localInfo;
  private BundleContent bundleContent;
  
  public static void showDialog(Component frameComp, LocalInfo localInfo, BundleContent bundleContent)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    LocalInfoDialog dialog = new LocalInfoDialog(frame, true, localInfo, bundleContent);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
  }
  
  /** Creates new form LocalInfoDialog */
  public LocalInfoDialog(java.awt.Frame parent, boolean modal, LocalInfo localInfo, BundleContent bundleContent)
  {
    super(parent, modal);
    this.localInfo = localInfo;
    this.bundleContent = bundleContent;
    initComponents();
    
    Licence licence = localInfo.getLicence();
    
    if (localInfo.isInstalled())
    {
      installedPanel.setEnabled(true);
      inMemoryCheckBox.setEnabled(true);
      inMemoryCounterLabel.setEnabled(true);
      inMemoryCounterTextField.setEnabled(true);
      
      installedCheckBox.setSelected(true);
      inMemoryCheckBox.setSelected(localInfo.isInMemory());
      inMemoryCounterTextField.setText(Integer.toString(localInfo.getInMemoryCounter()));
    }
    else
    {
      installedPanel.setEnabled(false);
      inMemoryCheckBox.setEnabled(false);
      inMemoryCounterLabel.setEnabled(false);
      inMemoryCounterTextField.setEnabled(false);
      
      installedCheckBox.setSelected(false);
      inMemoryCheckBox.setSelected(false);
      inMemoryCounterTextField.setText("");
    }
    
    if (localInfo.isShareManager())
    {
      shareManagerPanel.setEnabled(true);
      nodeFilterCheckBox.setEnabled(true);
      nodeFilterButton.setEnabled(localInfo.getSMNodeFilter() != null);
      shareGroupsCheckBox.setEnabled(true);
      shareGroupsButton.setEnabled(localInfo.getSMShareGroups() != null);
      clientsCheckBox.setEnabled(true);
      clientsButton.setEnabled(licence.withCopies());
      equalityCheckBox.setEnabled(true);
      
      shareManagerCheckBox.setSelected(true);
      nodeFilterCheckBox.setSelected(localInfo.getSMNodeFilter() != null);
      shareGroupsCheckBox.setSelected(localInfo.getSMShareGroups() != null);
      clientsCheckBox.setSelected(licence.withCopies());
      equalityCheckBox.setSelected(localInfo.isSMEquality());
    }
    else
    {
      shareManagerPanel.setEnabled(false);
      nodeFilterCheckBox.setEnabled(false);
      nodeFilterButton.setEnabled(false);
      shareGroupsCheckBox.setEnabled(false);
      shareGroupsButton.setEnabled(false);
      clientsCheckBox.setEnabled(false);
      clientsButton.setEnabled(false);
      equalityCheckBox.setEnabled(false);
      
      shareManagerCheckBox.setSelected(false);
      nodeFilterCheckBox.setSelected(false);
      shareGroupsCheckBox.setSelected(false);
      clientsCheckBox.setSelected(false);
      equalityCheckBox.setSelected(false);
    }
    
    if (localInfo.isShareClient())
    {
      shareClientPanel.setEnabled(true);
      managerLabel.setEnabled(true);
      managerTextField.setEnabled(true);
      
      shareClientCheckBox.setSelected(true);
      managerTextField.setText(localInfo.getSCManager());
    }
    else
    {
      shareClientPanel.setEnabled(false);
      managerLabel.setEnabled(false);
      managerTextField.setEnabled(false);
      
      shareClientCheckBox.setSelected(false);
      managerTextField.setText("");
    }
    
    if (bundleContent == null) showBundleContentButton.setVisible(false);
    
    setTitle("Local Info of '" + localInfo.getName() + "'");
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    jPanel12 = new javax.swing.JPanel();
    installedCheckBox = new javax.swing.JCheckBox();
    installedPanel = new javax.swing.JPanel();
    inMemoryCheckBox = new javax.swing.JCheckBox();
    inMemoryCounterLabel = new javax.swing.JLabel();
    inMemoryCounterTextField = new javax.swing.JTextField();
    shareManagerCheckBox = new javax.swing.JCheckBox();
    shareManagerPanel = new javax.swing.JPanel();
    jPanel6 = new javax.swing.JPanel();
    nodeFilterCheckBox = new javax.swing.JCheckBox();
    nodeFilterButton = new javax.swing.JButton();
    jPanel7 = new javax.swing.JPanel();
    shareGroupsCheckBox = new javax.swing.JCheckBox();
    shareGroupsButton = new javax.swing.JButton();
    jPanel8 = new javax.swing.JPanel();
    clientsCheckBox = new javax.swing.JCheckBox();
    clientsButton = new javax.swing.JButton();
    equalityCheckBox = new javax.swing.JCheckBox();
    shareClientCheckBox = new javax.swing.JCheckBox();
    shareClientPanel = new javax.swing.JPanel();
    managerLabel = new javax.swing.JLabel();
    managerTextField = new javax.swing.JTextField();
    licencePanel = new javax.swing.JPanel();
    licenceButton = new javax.swing.JButton();
    showBundleContentButton = new javax.swing.JButton();
    jPanel5 = new javax.swing.JPanel();
    closeButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    jPanel12.setLayout(new java.awt.GridLayout(4, 0, 0, 5));

    jPanel12.setBorder(new javax.swing.border.EtchedBorder());
    installedCheckBox.setText("Installed");
    installedCheckBox.setBorderPaintedFlat(true);
    installedCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel12.add(installedCheckBox);

    installedPanel.setLayout(new java.awt.GridLayout(4, 0));

    installedPanel.setBorder(new javax.swing.border.EtchedBorder());
    inMemoryCheckBox.setText("In Memory");
    inMemoryCheckBox.setBorderPaintedFlat(true);
    inMemoryCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    installedPanel.add(inMemoryCheckBox);

    inMemoryCounterLabel.setText(" In Memory Counter:");
    installedPanel.add(inMemoryCounterLabel);

    inMemoryCounterTextField.setEditable(false);
    installedPanel.add(inMemoryCounterTextField);

    jPanel12.add(installedPanel);

    shareManagerCheckBox.setText("Share Manager");
    shareManagerCheckBox.setBorderPaintedFlat(true);
    shareManagerCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel12.add(shareManagerCheckBox);

    shareManagerPanel.setLayout(new java.awt.GridLayout(4, 0));

    shareManagerPanel.setBorder(new javax.swing.border.EtchedBorder());
    jPanel6.setLayout(new javax.swing.BoxLayout(jPanel6, javax.swing.BoxLayout.X_AXIS));

    nodeFilterCheckBox.setText("Node Filter");
    nodeFilterCheckBox.setBorderPaintedFlat(true);
    nodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel6.add(nodeFilterCheckBox);

    nodeFilterButton.setText("Show Node Filter");
    nodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterButtonActionPerformed(evt);
      }
    });

    jPanel6.add(nodeFilterButton);

    shareManagerPanel.add(jPanel6);

    jPanel7.setLayout(new javax.swing.BoxLayout(jPanel7, javax.swing.BoxLayout.X_AXIS));

    shareGroupsCheckBox.setText("Share Groups");
    shareGroupsCheckBox.setBorderPaintedFlat(true);
    shareGroupsCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel7.add(shareGroupsCheckBox);

    shareGroupsButton.setText("Show Share Groups");
    shareGroupsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        shareGroupsButtonActionPerformed(evt);
      }
    });

    jPanel7.add(shareGroupsButton);

    shareManagerPanel.add(jPanel7);

    jPanel8.setLayout(new javax.swing.BoxLayout(jPanel8, javax.swing.BoxLayout.X_AXIS));

    clientsCheckBox.setText("Clients");
    clientsCheckBox.setBorderPaintedFlat(true);
    clientsCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel8.add(clientsCheckBox);

    clientsButton.setText("Show Clients");
    clientsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        clientsButtonActionPerformed(evt);
      }
    });

    jPanel8.add(clientsButton);

    shareManagerPanel.add(jPanel8);

    equalityCheckBox.setText("Equality");
    equalityCheckBox.setBorderPaintedFlat(true);
    equalityCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    shareManagerPanel.add(equalityCheckBox);

    jPanel12.add(shareManagerPanel);

    shareClientCheckBox.setText("Share Client");
    shareClientCheckBox.setBorderPaintedFlat(true);
    shareClientCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        CheckBoxNoChangeAction(evt);
      }
    });

    jPanel12.add(shareClientCheckBox);

    shareClientPanel.setLayout(new java.awt.GridLayout(4, 0));

    shareClientPanel.setBorder(new javax.swing.border.EtchedBorder());
    managerLabel.setText(" Manager:");
    shareClientPanel.add(managerLabel);

    managerTextField.setEditable(false);
    shareClientPanel.add(managerTextField);

    jPanel12.add(shareClientPanel);

    licenceButton.setText("Show Local Licence");
    licenceButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        licenceButtonActionPerformed(evt);
      }
    });

    licencePanel.add(licenceButton);

    showBundleContentButton.setText("Show Bundle Content");
    showBundleContentButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        showBundleContentButtonActionPerformed(evt);
      }
    });

    licencePanel.add(showBundleContentButton);

    jPanel12.add(licencePanel);

    getContentPane().add(jPanel12, java.awt.BorderLayout.NORTH);

    closeButton.setText("Close");
    closeButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        closeButtonActionPerformed(evt);
      }
    });

    jPanel5.add(closeButton);

    getContentPane().add(jPanel5, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void showBundleContentButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_showBundleContentButtonActionPerformed
  {//GEN-HEADEREND:event_showBundleContentButtonActionPerformed
    List compList = bundleContent.getComponents();
    Object[] components = compList.toArray();
    ListDialog.showDialog(this, this, "Components:", "Content of bundle '" + localInfo.getName() + "'", components, null, null, false, false);
  }//GEN-LAST:event_showBundleContentButtonActionPerformed

  private void closeButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeButtonActionPerformed
  {//GEN-HEADEREND:event_closeButtonActionPerformed
    dispose();
  }//GEN-LAST:event_closeButtonActionPerformed

  private void licenceButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_licenceButtonActionPerformed
  {//GEN-HEADEREND:event_licenceButtonActionPerformed
    LicenceEditDialog.showDialog(this, localInfo.getLicence(), false, "Local Licence of '" + localInfo.getName() + "'");
  }//GEN-LAST:event_licenceButtonActionPerformed

  private void clientsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_clientsButtonActionPerformed
  {//GEN-HEADEREND:event_clientsButtonActionPerformed
    List clients = localInfo.getSMClients().getList();
    
    ListDialog.showDialog(this, this, "Client nodes:", "Share Manager's Clients of '" + localInfo.getName() + "'", (String[])clients.toArray(new String[clients.size()]), null, null, false, false);
  }//GEN-LAST:event_clientsButtonActionPerformed

  private void shareGroupsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_shareGroupsButtonActionPerformed
  {//GEN-HEADEREND:event_shareGroupsButtonActionPerformed
    List shareGroups = localInfo.getSMShareGroups().getList();
    
    ListDialog.showDialog(this, this, "Share Groups:", "Share Manager's Share Groups of '" + localInfo.getName() + "'", (String[])shareGroups.toArray(new String[shareGroups.size()]), null, null, false, false);
  }//GEN-LAST:event_shareGroupsButtonActionPerformed

  private void nodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_nodeFilterButtonActionPerformed
    List patterns = localInfo.getSMNodeFilter().getPatterns();
    
    ListDialog.showDialog(this, this, "Node Name Patterns:", "Share Manager's Node Filter of '" + localInfo.getName() + "'", (String[])patterns.toArray(new String[patterns.size()]), null, null, false, false);
  }//GEN-LAST:event_nodeFilterButtonActionPerformed

  private void CheckBoxNoChangeAction(java.awt.event.ActionEvent evt)//GEN-FIRST:event_CheckBoxNoChangeAction
  {//GEN-HEADEREND:event_CheckBoxNoChangeAction
    JCheckBox checkBox = (JCheckBox)evt.getSource();
    checkBox.setSelected(!checkBox.isSelected());
  }//GEN-LAST:event_CheckBoxNoChangeAction
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton clientsButton;
  private javax.swing.JCheckBox clientsCheckBox;
  private javax.swing.JButton closeButton;
  private javax.swing.JCheckBox equalityCheckBox;
  private javax.swing.JCheckBox inMemoryCheckBox;
  private javax.swing.JLabel inMemoryCounterLabel;
  private javax.swing.JTextField inMemoryCounterTextField;
  private javax.swing.JCheckBox installedCheckBox;
  private javax.swing.JPanel installedPanel;
  private javax.swing.JPanel jPanel12;
  private javax.swing.JPanel jPanel5;
  private javax.swing.JPanel jPanel6;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JPanel jPanel8;
  private javax.swing.JButton licenceButton;
  private javax.swing.JPanel licencePanel;
  private javax.swing.JLabel managerLabel;
  private javax.swing.JTextField managerTextField;
  private javax.swing.JButton nodeFilterButton;
  private javax.swing.JCheckBox nodeFilterCheckBox;
  private javax.swing.JCheckBox shareClientCheckBox;
  private javax.swing.JPanel shareClientPanel;
  private javax.swing.JButton shareGroupsButton;
  private javax.swing.JCheckBox shareGroupsCheckBox;
  private javax.swing.JCheckBox shareManagerCheckBox;
  private javax.swing.JPanel shareManagerPanel;
  private javax.swing.JButton showBundleContentButton;
  // End of variables declaration//GEN-END:variables
  
}
