/*
 * OutputTriggerEditDialog.java
 *
 * Created on 17. kvten 2004, 22:47
 */

package SOFA.SOFAnet.Browser;

import SOFA.SOFAnet.Repository.OutputTrigger;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Repository.BundleNameFilter;
import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import javax.swing.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class OutputTriggerEditDialog extends javax.swing.JDialog
{
  private OutputTrigger outputTrigger;
  private BundleNameFilter bundleFilter;
  private NodeNameFilter nodeFilter;
  private boolean isOK;
  
  public static boolean showDialog(Component frameComp, OutputTrigger outputTrigger, String title)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    OutputTriggerEditDialog dialog = new OutputTriggerEditDialog(frame, true, outputTrigger, title);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form OutputTriggerEditDialog */
  public OutputTriggerEditDialog(java.awt.Frame parent, boolean modal, OutputTrigger outputTrigger, String title)
  {
    super(parent, modal);
    this.outputTrigger = outputTrigger;
    bundleFilter = outputTrigger.getBundleFilter();
    nodeFilter = outputTrigger.getNodeFilter();
    isOK = false;
    
    initComponents();
    setTitle(title);
    
    nameTextField.setText(outputTrigger.getName());
    descriptionTextField.setText(outputTrigger.getDescription());

    if (bundleFilter != null)
    {
      bundleFilterCheckBox.setSelected(true);
      bundleFilterButton.setEnabled(true);
    }
    else
    {
      bundleFilterCheckBox.setSelected(false);
      bundleFilterButton.setEnabled(false);
    }
    
    if (nodeFilter != null)
    {
      nodeFilterCheckBox.setSelected(true);
      nodeFilterButton.setEnabled(true);
    }
    else
    {
      nodeFilterCheckBox.setSelected(false);
      nodeFilterButton.setEnabled(false);
    }

    if (bundleFilter != null) bundleFilter = (BundleNameFilter)bundleFilter.clone();
    else bundleFilter = new BundleNameFilter();
    if (nodeFilter != null) nodeFilter = (NodeNameFilter)nodeFilter.clone();
    else nodeFilter = new NodeNameFilter();
    
    Date validFrom = outputTrigger.getValidFrom();
    Date validTo = outputTrigger.getValidTo();
    DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);

    if (validFrom != null) validFromTextField.setText(dateFormat.format(validFrom));
    else validFromTextField.setText("");

    if (validTo != null) validToTextField.setText(dateFormat.format(validTo));
    else validToTextField.setText("");
    
    autodeleteCheckBox.setSelected(outputTrigger.getAutoDelete());
    
    if (outputTrigger.isContract())
    {
      contractIDTextField.setText(outputTrigger.getContractID());
      contractRuleTextField.setText(outputTrigger.getContractRule());
    }
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    nameAndDescriptionPanel = new javax.swing.JPanel();
    nameLabel = new javax.swing.JLabel();
    nameTextField = new javax.swing.JTextField();
    descriptionLabel = new javax.swing.JLabel();
    descriptionTextField = new javax.swing.JTextField();
    jPanel7 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    parametersPanel = new javax.swing.JPanel();
    bundleFilterPanel = new javax.swing.JPanel();
    bundleFilterCheckBox = new javax.swing.JCheckBox();
    bundleFilterButton = new javax.swing.JButton();
    nodeFilterPanel = new javax.swing.JPanel();
    nodeFilterCheckBox = new javax.swing.JCheckBox();
    nodeFilterButton = new javax.swing.JButton();
    validFromPanel = new javax.swing.JPanel();
    validFromLabel = new javax.swing.JLabel();
    validFromTextField = new javax.swing.JTextField();
    validToPanel = new javax.swing.JPanel();
    validToLabel = new javax.swing.JLabel();
    validToTextField = new javax.swing.JTextField();
    autodeleteCheckBox = new javax.swing.JCheckBox();
    contractPanel = new javax.swing.JPanel();
    jPanel5 = new javax.swing.JPanel();
    contractIDLabel = new javax.swing.JLabel();
    contractIDTextField = new javax.swing.JTextField();
    contractRuleLabel = new javax.swing.JLabel();
    contractRuleTextField = new javax.swing.JTextField();
    jPanel6 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    nameAndDescriptionPanel.setLayout(new java.awt.GridLayout(4, 0));

    nameAndDescriptionPanel.setBorder(new javax.swing.border.TitledBorder("Name and Description of Output Trigger"));
    nameLabel.setText("Name:");
    nameAndDescriptionPanel.add(nameLabel);

    nameTextField.setEditable(false);
    nameAndDescriptionPanel.add(nameTextField);

    descriptionLabel.setText("Description:");
    nameAndDescriptionPanel.add(descriptionLabel);

    nameAndDescriptionPanel.add(descriptionTextField);

    getContentPane().add(nameAndDescriptionPanel, java.awt.BorderLayout.NORTH);

    jPanel7.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.GridLayout(1, 0));

    parametersPanel.setLayout(new java.awt.GridLayout(5, 0));

    parametersPanel.setBorder(new javax.swing.border.TitledBorder("Parameters"));
    bundleFilterPanel.setLayout(new javax.swing.BoxLayout(bundleFilterPanel, javax.swing.BoxLayout.X_AXIS));

    bundleFilterCheckBox.setText("Bundle Filter");
    bundleFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        bundleFilterCheckBoxActionPerformed(evt);
      }
    });

    bundleFilterPanel.add(bundleFilterCheckBox);

    bundleFilterButton.setText("Edit Bundle Filter");
    bundleFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        bundleFilterButtonActionPerformed(evt);
      }
    });

    bundleFilterPanel.add(bundleFilterButton);

    parametersPanel.add(bundleFilterPanel);

    nodeFilterPanel.setLayout(new javax.swing.BoxLayout(nodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    nodeFilterCheckBox.setText("Node Filter");
    nodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterCheckBoxActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterCheckBox);

    nodeFilterButton.setText("Edit Node Filter");
    nodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterButtonActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterButton);

    parametersPanel.add(nodeFilterPanel);

    validFromPanel.setLayout(new javax.swing.BoxLayout(validFromPanel, javax.swing.BoxLayout.X_AXIS));

    validFromLabel.setText("Valid From:");
    validFromPanel.add(validFromLabel);

    validFromPanel.add(validFromTextField);

    parametersPanel.add(validFromPanel);

    validToPanel.setLayout(new javax.swing.BoxLayout(validToPanel, javax.swing.BoxLayout.X_AXIS));

    validToLabel.setText("Valid To:");
    validToPanel.add(validToLabel);

    validToPanel.add(validToTextField);

    parametersPanel.add(validToPanel);

    autodeleteCheckBox.setText("Automatically Delete When Expired");
    parametersPanel.add(autodeleteCheckBox);

    jPanel3.add(parametersPanel);

    contractPanel.setLayout(new java.awt.BorderLayout());

    contractPanel.setBorder(new javax.swing.border.TitledBorder("Contract Connection"));
    contractPanel.setMinimumSize(new java.awt.Dimension(200, 111));
    contractPanel.setPreferredSize(new java.awt.Dimension(200, 111));
    jPanel5.setLayout(new java.awt.GridLayout(4, 0));

    contractIDLabel.setText("Contract ID:");
    jPanel5.add(contractIDLabel);

    jPanel5.add(contractIDTextField);

    contractRuleLabel.setText("Contract Rule:");
    jPanel5.add(contractRuleLabel);

    jPanel5.add(contractRuleTextField);

    contractPanel.add(jPanel5, java.awt.BorderLayout.NORTH);

    jPanel3.add(contractPanel);

    jPanel7.add(jPanel3, java.awt.BorderLayout.NORTH);

    getContentPane().add(jPanel7, java.awt.BorderLayout.CENTER);

    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel6.add(OKButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel6.add(cancelButton);

    getContentPane().add(jPanel6, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    Date validFrom = null;
    Date validTo = null;
    try
    {
      DateFormat dateFormat = DateFormat.getInstance();
      String validFromText = validFromTextField.getText().trim().toLowerCase();
      if (validFromText.length() != 0 && validFromText.compareTo("no limitation") != 0) validFrom = dateFormat.parse(validFromText);

      String validToText = validToTextField.getText().trim().toLowerCase();
      if (validToText.length() != 0 && validToText.compareTo("no limitation") != 0) validTo = dateFormat.parse(validToText);
    }
    catch (ParseException e)
    {
      JOptionPane.showMessageDialog(this, "Error occured while parsing 'Valid From' or 'Valid To':\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    String contractID = contractIDTextField.getText().trim();
    String contractRule = contractRuleTextField.getText().trim();
    if (contractID.length() != 0 && contractRule.length() == 0)
    {
      JOptionPane.showMessageDialog(this, "Missing Contract Rule", "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    outputTrigger.setDescription(descriptionTextField.getText());
    

    if (!bundleFilterCheckBox.isSelected()) bundleFilter = null;
    if (!nodeFilterCheckBox.isSelected()) nodeFilter = null;
    outputTrigger.setBundleFilter(bundleFilter);
    outputTrigger.setNodeFilter(nodeFilter);
    outputTrigger.setValidFrom(validFrom);
    outputTrigger.setValidTo(validTo);
    outputTrigger.setAutoDelete(autodeleteCheckBox.isSelected());
    if (contractID.length() != 0) outputTrigger.setContract(contractID, contractRule);
    else outputTrigger.setContract(null, null);
    
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void nodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_nodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, nodeFilter.getPatterns(), "Node Name Pattern", "Node Filter Editor");
  }//GEN-LAST:event_nodeFilterButtonActionPerformed

  private void nodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_nodeFilterCheckBoxActionPerformed
    nodeFilterButton.setEnabled(nodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_nodeFilterCheckBoxActionPerformed

  private void bundleFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bundleFilterButtonActionPerformed
  {//GEN-HEADEREND:event_bundleFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, bundleFilter.getPatterns(), "Bundle Name Pattern", "Bundle Filter Editor");
  }//GEN-LAST:event_bundleFilterButtonActionPerformed

  private void bundleFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bundleFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_bundleFilterCheckBoxActionPerformed
    bundleFilterButton.setEnabled(bundleFilterCheckBox.isSelected());
  }//GEN-LAST:event_bundleFilterCheckBoxActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKButton;
  private javax.swing.JCheckBox autodeleteCheckBox;
  private javax.swing.JButton bundleFilterButton;
  private javax.swing.JCheckBox bundleFilterCheckBox;
  private javax.swing.JPanel bundleFilterPanel;
  private javax.swing.JButton cancelButton;
  private javax.swing.JLabel contractIDLabel;
  private javax.swing.JTextField contractIDTextField;
  private javax.swing.JPanel contractPanel;
  private javax.swing.JLabel contractRuleLabel;
  private javax.swing.JTextField contractRuleTextField;
  private javax.swing.JLabel descriptionLabel;
  private javax.swing.JTextField descriptionTextField;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel5;
  private javax.swing.JPanel jPanel6;
  private javax.swing.JPanel jPanel7;
  private javax.swing.JPanel nameAndDescriptionPanel;
  private javax.swing.JLabel nameLabel;
  private javax.swing.JTextField nameTextField;
  private javax.swing.JButton nodeFilterButton;
  private javax.swing.JCheckBox nodeFilterCheckBox;
  private javax.swing.JPanel nodeFilterPanel;
  private javax.swing.JPanel parametersPanel;
  private javax.swing.JLabel validFromLabel;
  private javax.swing.JPanel validFromPanel;
  private javax.swing.JTextField validFromTextField;
  private javax.swing.JLabel validToLabel;
  private javax.swing.JPanel validToPanel;
  private javax.swing.JTextField validToTextField;
  // End of variables declaration//GEN-END:variables
  
}
