/*
 * RMISearchConfigurationEditDialog.java
 *
 * Created on 21. kvten 2004, 14:22
 */

package SOFA.SOFAnet.Browser;

import javax.swing.*;
import SOFA.SOFAnet.Repository.NodeNameFilter;
import SOFA.SOFAnet.Search.RMI.RMISearchConfiguration;
import java.util.*;
import java.awt.Component;
import java.awt.Frame;

/**
 *
 * @author  Ladislav Sobr
 */
public class RMISearchConfigurationEditDialog extends javax.swing.JDialog
{
  private RMISearchConfiguration configuration;
  private DefaultListModel content;
  private NodeNameFilter nodeFilter;
  private boolean isOK;
  private boolean apply;

  public static boolean showDialog(Component frameComp, RMISearchConfiguration configuration)
  {
    Frame frame = JOptionPane.getFrameForComponent(frameComp);
    RMISearchConfigurationEditDialog dialog = new RMISearchConfigurationEditDialog(frame, true, configuration);
    dialog.setLocationRelativeTo(frameComp);
    dialog.setVisible(true);
    return dialog.isOK();
  }
  
  /** Creates new form RMISearchConfigurationEditDialog */
  public RMISearchConfigurationEditDialog(java.awt.Frame parent, boolean modal, RMISearchConfiguration configuration)
  {
    super(parent, modal);
    this.configuration = configuration;
    content = new DefaultListModel();
    isOK = false;
    apply = false;
    nodeFilter = configuration.getNodeFilter();
    
    initComponents();
    
    if (nodeFilter != null)
    {
      nodeFilterCheckBox.setSelected(true);
      nodeFilterButton.setEnabled(true);
    }
    else
    {
      nodeFilterCheckBox.setSelected(false);
      nodeFilterButton.setEnabled(false);
    }

    if (nodeFilter != null) nodeFilter = (NodeNameFilter)nodeFilter.clone();
    else nodeFilter = new NodeNameFilter();

    List connections = configuration.getConnections();
    synchronized (connections)
    {
      Iterator it = connections.iterator();
      while (it.hasNext())
      {
        RMISearchConfiguration.Connection connection = (RMISearchConfiguration.Connection)it.next();
        content.addElement(connection.clone());
      }
    }
  }
  
  public boolean isOK()
  {
    return isOK;
  }
  
  public boolean apply()
  {
    return apply;
  }
  
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents()//GEN-BEGIN:initComponents
  {
    jPanel1 = new javax.swing.JPanel();
    nodeFilterPanel = new javax.swing.JPanel();
    nodeFilterCheckBox = new javax.swing.JCheckBox();
    nodeFilterButton = new javax.swing.JButton();
    jLabel1 = new javax.swing.JLabel();
    jScrollPane1 = new javax.swing.JScrollPane();
    connectionsList = new javax.swing.JList();
    jPanel2 = new javax.swing.JPanel();
    jPanel4 = new javax.swing.JPanel();
    editButton = new javax.swing.JButton();
    addButton = new javax.swing.JButton();
    deleteSelectedButton = new javax.swing.JButton();
    jPanel3 = new javax.swing.JPanel();
    OKButton = new javax.swing.JButton();
    OKAndApplyButton = new javax.swing.JButton();
    cancelButton = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("RMI Search Subsystem Configuration");
    jPanel1.setLayout(new java.awt.GridLayout(2, 0));

    jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 1, 1, 1)));
    nodeFilterPanel.setLayout(new javax.swing.BoxLayout(nodeFilterPanel, javax.swing.BoxLayout.X_AXIS));

    nodeFilterCheckBox.setText("Node Filter");
    nodeFilterCheckBox.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterCheckBoxActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterCheckBox);

    nodeFilterButton.setText("Edit Node Filter");
    nodeFilterButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        nodeFilterButtonActionPerformed(evt);
      }
    });

    nodeFilterPanel.add(nodeFilterButton);

    jPanel1.add(nodeFilterPanel);

    jLabel1.setText("Connections:");
    jPanel1.add(jLabel1);

    getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

    jScrollPane1.setMinimumSize(new java.awt.Dimension(260, 132));
    connectionsList.setModel(content);
    connectionsList.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(java.awt.event.MouseEvent evt)
      {
        connectionsListMouseClicked(evt);
      }
    });

    jScrollPane1.setViewportView(connectionsList);

    getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel4.setLayout(new java.awt.GridLayout(3, 0));

    jPanel4.setBorder(new javax.swing.border.TitledBorder("Search Connections"));
    editButton.setText("Edit Connection");
    editButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        editButtonActionPerformed(evt);
      }
    });

    jPanel4.add(editButton);

    addButton.setText("Add New Connection");
    addButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        addButtonActionPerformed(evt);
      }
    });

    jPanel4.add(addButton);

    deleteSelectedButton.setText("Delete Selected");
    deleteSelectedButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        deleteSelectedButtonActionPerformed(evt);
      }
    });

    jPanel4.add(deleteSelectedButton);

    jPanel2.add(jPanel4, java.awt.BorderLayout.NORTH);

    getContentPane().add(jPanel2, java.awt.BorderLayout.EAST);

    OKButton.setText("OK");
    OKButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKButtonActionPerformed(evt);
      }
    });

    jPanel3.add(OKButton);

    OKAndApplyButton.setText("Save and Apply Configuration");
    OKAndApplyButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        OKAndApplyButtonActionPerformed(evt);
      }
    });

    jPanel3.add(OKAndApplyButton);

    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(java.awt.event.ActionEvent evt)
      {
        cancelButtonActionPerformed(evt);
      }
    });

    jPanel3.add(cancelButton);

    getContentPane().add(jPanel3, java.awt.BorderLayout.SOUTH);

    pack();
  }//GEN-END:initComponents

  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
  {//GEN-HEADEREND:event_cancelButtonActionPerformed
    dispose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void saveResults()
  {
    if (!nodeFilterCheckBox.isSelected()) nodeFilter = null;
    
    configuration.setNodeFilter(nodeFilter);
    
    List connections = configuration.getConnections();
    connections.clear();
    Enumeration en = content.elements();
    while (en.hasMoreElements()) connections.add((RMISearchConfiguration.Connection)en.nextElement());
  }
  
  private void OKAndApplyButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKAndApplyButtonActionPerformed
  {//GEN-HEADEREND:event_OKAndApplyButtonActionPerformed
    saveResults();
    isOK = true;
    apply = true;
    dispose();
  }//GEN-LAST:event_OKAndApplyButtonActionPerformed

  private void OKButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_OKButtonActionPerformed
  {//GEN-HEADEREND:event_OKButtonActionPerformed
    saveResults();
    isOK = true;
    dispose();
  }//GEN-LAST:event_OKButtonActionPerformed

  private void connectionsListMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_connectionsListMouseClicked
  {//GEN-HEADEREND:event_connectionsListMouseClicked
    if (evt.getClickCount() == 2)
    {
      editButton.doClick(); //emulate button click
    }
  }//GEN-LAST:event_connectionsListMouseClicked

  private void deleteSelectedButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_deleteSelectedButtonActionPerformed
  {//GEN-HEADEREND:event_deleteSelectedButtonActionPerformed
    Object[] selected = connectionsList.getSelectedValues();
    for (int i = 0; i < selected.length; i++) content.removeElement(selected[i]);
  }//GEN-LAST:event_deleteSelectedButtonActionPerformed

  private void addButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addButtonActionPerformed
  {//GEN-HEADEREND:event_addButtonActionPerformed
    RMISearchConfiguration.Connection connection = new RMISearchConfiguration.Connection("", null);
    if (RMISearchConfigurationConnectionEditDialog.showDialog(this, connection))
    {
      content.addElement(connection);
    }
  }//GEN-LAST:event_addButtonActionPerformed

  private void editButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_editButtonActionPerformed
  {//GEN-HEADEREND:event_editButtonActionPerformed
    RMISearchConfiguration.Connection connection = (RMISearchConfiguration.Connection)connectionsList.getSelectedValue();
    
    if (connection == null)
    {
      JOptionPane.showMessageDialog(this, "No Connection selected", "Warning", JOptionPane.WARNING_MESSAGE);
      return;
    }
    
    if (RMISearchConfigurationConnectionEditDialog.showDialog(this, connection))
    {
    }
  }//GEN-LAST:event_editButtonActionPerformed

  private void nodeFilterButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterButtonActionPerformed
  {//GEN-HEADEREND:event_nodeFilterButtonActionPerformed
    StringListEditDialog.showDialog(this, nodeFilter.getPatterns(), "Node Name Pattern", "Node Filter Editor");
  }//GEN-LAST:event_nodeFilterButtonActionPerformed

  private void nodeFilterCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_nodeFilterCheckBoxActionPerformed
  {//GEN-HEADEREND:event_nodeFilterCheckBoxActionPerformed
    nodeFilterButton.setEnabled(nodeFilterCheckBox.isSelected());
  }//GEN-LAST:event_nodeFilterCheckBoxActionPerformed
  
 
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JButton OKAndApplyButton;
  private javax.swing.JButton OKButton;
  private javax.swing.JButton addButton;
  private javax.swing.JButton cancelButton;
  private javax.swing.JList connectionsList;
  private javax.swing.JButton deleteSelectedButton;
  private javax.swing.JButton editButton;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JPanel jPanel1;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private javax.swing.JPanel jPanel4;
  private javax.swing.JScrollPane jScrollPane1;
  private javax.swing.JButton nodeFilterButton;
  private javax.swing.JCheckBox nodeFilterCheckBox;
  private javax.swing.JPanel nodeFilterPanel;
  // End of variables declaration//GEN-END:variables
  
}
