/*
 * BinBundles.java
 *
 * Created on 22. listopad 2003, 20:51
 */

package SOFA.SOFAnet.Repository;

import java.io.*;

/**
 * Storage of binary bundles.
 *
 * @author  Ladislav Sobr
 */
public class BinBundles extends StorageBase
{
  
  /** Creates a new instance of BinBundles */
  public BinBundles(File baseDir)
  {
    super(baseDir, "jar", false);
    init();
  }
  
  protected StorageItem newItem(String name, File file)
  {
    BundleInfo bundleInfo = new BundleInfo();
    try
    {
      bundleInfo.fromFilename(name);
    }
    catch (BundleInfo.InvalidBundleNameException e)
    {
      return null;
    }
    bundleInfo.setFile(file);
    return bundleInfo;
  }

  /**
   * Returns requested memory structure of binary bundle
   *
   * @return memory structure representing binary bundle; null if not present
   */
  public BundleInfo getBinBundle(String bundleName)
  {
    return (BundleInfo)map.get(bundleName);
  }
  
  /**
   * Deletes binary bundle (including file).
   *
   * @param bundleName name of bundle
   * @return true, if binary bundle was found (and deleted)
   */
  public boolean deleteBinBundle(String bundleName)
  {
    synchronized (map)
    {
      BundleInfo bundleInfo = (BundleInfo)map.remove(bundleName);
      if (bundleInfo != null)
      {
        try
        {
          bundleInfo.getFile().delete();
        }
        catch (Exception e)
        {
        }
        return true;
      }
      else return false;
    }
  }
  
  /**
   * Adds (creates) new binary bundle (only the memory structure, not the file itself)
   *
   * @param bundleName name of bundle
   * @return BundleInfo, if binary bundle was added (bundleName was correct and there was not already the one with the same name), null otherwise
   */
  public BundleInfo addBinBundle(String bundleName)
  {
    synchronized (map)
    {
      if (map.get(bundleName) != null) return null;

      BundleInfo bundleInfo = new BundleInfo();
      try
      {
        bundleInfo.fromBundleName(bundleName);
      }
      catch (BundleInfo.InvalidBundleNameException e)
      {
        return null;
      }
      
      bundleInfo.setFile(new File(baseDir, bundleInfo.getFileName() + ".jar"));
      map.put(bundleName, bundleInfo);
      return bundleInfo;
    }
  }
}
