/*
 * CompBundleMap.java
 *
 * Created on 23. listopad 2003, 19:44
 */

package SOFA.SOFAnet.Repository;

import java.util.*;
import SOFA.SOFAnode.TR.Impl.ComponentInfoImpl;

/**
 * Map between names of components and names of bundles.
 * It maintain the information which component belongs to which bundle.
 * It is used to quick look up of the bundle (or bundle offer) that contains concrete component
 *
 * @author  Ladislav Sobr
 */
public class CompBundleMap
{
  public Map map;
  
  /** Creates a new instance of CompBundleMap */
  public CompBundleMap()
  {
    map = Collections.synchronizedMap(new HashMap());
  }
  
  /**
   * Returns name of bundle that contains specified component
   * @param componentFullName full name of component in format name[version]
   * @return name of bundle or null if no bundle found
   */
  public String findBundle(String componentFullName)
  {
    return (String)map.get(componentFullName);
  }
  
  /**
   * Adds records to the map - components of one bundle/
   * @param components list of components (ComponentInfoImpl class)
   * @param bundleName name of bundle
   */
  public void addComponents(List components, String bundleName)
  {
    synchronized (map)
    {
      synchronized (components)
      {
        Iterator it = components.iterator();
        while (it.hasNext())
        {
          String componentName = ((ComponentInfoImpl)it.next()).toString();
          map.put(componentName, bundleName);
        }
      }
    }
  }

  /**
   * Adds records to the map - they are taken from bundle content
   * @param bundleContent content of the bundle to add
   */
  public void addBundleContent(BundleContent bundleContent)
  {
    List components = bundleContent.getComponents();
    String bundleName = bundleContent.getName();
    addComponents(components, bundleName);
  }
  
  /**
   * Adds records to the map - they are taken from bundle offer
   * @param bundleOffer bundle offer to add
   */
  public void addBundleOffer(BundleOffer bundleOffer)
  {
    List components = bundleOffer.getComponents();
    String bundleName = bundleOffer.getName();
    addComponents(components, bundleName);
  }
  
  /**
   * Removes all records from the map that are connected with this name of bundle.
   * In fact it removes content of such bundle
   * @param bundleName name of bundle to delete
   */
  public void deleteBundle(String bundleName)
  {
    synchronized (map)
    {
      Iterator it = map.values().iterator();
      while (it.hasNext())
      {
        String name = (String)it.next();
        if (name.compareTo(bundleName) == 0) it.remove();
      }
    }
  }
  
}
