/*
 * NodeList.java
 *
 * Created on 3. prosinec 2003, 17:53
 */

package SOFA.SOFAnet.Repository;

import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.Serializable;

/**
 * Implementation of list of node names.
 *
 * @author  Ladislav Sobr
 */
public class NodeNameList implements Cloneable, Serializable
{
  private List list;
  
  /** Creates a new instance of NodeList */
  public NodeNameList()
  {
    list = Collections.synchronizedList(new LinkedList());
  }

  public Object clone()
  {
    NodeNameList clone = null;
    try
    { 
      clone = (NodeNameList)super.clone();
    }
    catch (CloneNotSupportedException e)
    { 
      throw new InternalError();
    }
    clone.list = Collections.synchronizedList(new LinkedList());
    clone.list.addAll(list);
    return clone;
  }
  
  public List getList()
  {
    return list;
  }

  public void add(String nodeName)
  {
    list.add(nodeName);
  }

  public void add(NodeNameList nodeNameList)
  {
    if (nodeNameList != null)
    {
      List li = nodeNameList.getList();
      synchronized (li)
      {
        Iterator it = li.iterator();
        while (it.hasNext())
        {
          String nodeName = (String)it.next();
          list.add(nodeName);
        }
      }
    }
  }
  
  public boolean contains(String nodeName)
  {
    return list.contains(nodeName);
  }
  
  public void remove(String nodeName)
  {
    list.remove(nodeName);
  }
  
  /**
   * XML format:
   * <p>
   * <pre>
   *  *&lt;node name="string"/&gt;
   * </pre
   */
  public void loadFromXML(Element element) throws SAXException
  {
    synchronized (list)
    {
      NodeList nl = element.getChildNodes();
      for (int i = 0; i < nl.getLength(); i++)
      {
        Node node = nl.item(i);
        if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().compareTo("node") == 0)
        {
          Element el = (Element)node;
          String name = el.getAttribute("name");
          list.add(name);
        }
      }
    }
  }
  
  public void saveToXML(Element element)
  {
    synchronized (list)
    {
      Document doc = element.getOwnerDocument();

      Iterator it = list.iterator();
      while (it.hasNext())
      {
        String name = (String)it.next();

        Element el = doc.createElement("node");
        el.setAttribute("name", name);
        element.appendChild(el);
      }
    }
  }
  
}
