/* $Id: CompReffer.java,v 1.2 2004/05/20 14:23:51 bures Exp $ */
package SOFA.SOFAnode.Made.CDL;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.ArrayDef;
import SOFA.SOFAnode.Made.TIR.AttributeDef;
import SOFA.SOFAnode.Made.TIR.CDLRepository;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.ConstantDef;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ExceptionDef;
import SOFA.SOFAnode.Made.TIR.ExprOperDef;
import SOFA.SOFAnode.Made.TIR.FixedDef;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.InterfaceDef;
import SOFA.SOFAnode.Made.TIR.OperationDef;
import SOFA.SOFAnode.Made.TIR.ParamDescr;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.PrimitiveKind;
import SOFA.SOFAnode.Made.TIR.SequenceDef;
import SOFA.SOFAnode.Made.TIR.StringDef;
import SOFA.SOFAnode.Made.TIR.StructMember;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;
import SOFA.SOFAnode.Made.TIR.TypedefDef;
import SOFA.SOFAnode.Made.TIR.UnionDef;
import SOFA.SOFAnode.Made.TIR.UnionMember;
import SOFA.SOFAnode.Made.TIR.WstringDef;

class CompReffer extends CompObjectIm implements CompType {
  public int what;  /* what is reffered */
  public CompPrimitive primKind;
  public FullID ref;
  public boolean isNew;  // false -- object is in repository
  public int line;
  public int basereffer;   // for typedefs (the base type of typedef)
  public int baserefferprim;  // set if basereffer is Primitive
  public boolean isSystem;    // if architecture or frame is system

  public CompReffer() {
    super(ObjectsKind.o_none);
    line = 0;
    basereffer = 0;
    isSystem = false;
  }

  public String toString() {
    if (ref!=null)
      return ref.toString();
    return "";
  }

  public Object getRefferedObject(CompRepository rep) throws CDLExceptLock, CDLExceptRemote {
    int i,j;

    if (isNew) {
      CompContainer con = rep;
      ref.sn.names.toFirst();
      for(i=0;i<ref.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) ref.sn.names.aktual());
        con = (CompContainer) a[0];
        ref.sn.names.toNext();
      }  
      return  con.lookup(ref.name, ref.version);
    } else {
      try {
        Container con = rep.cdlRepository;
        ref.sn.names.toFirst();
        for(i=0;i<ref.sn.names.size();i++) {
          Contained[] a = con.lookup_name((String) ref.sn.names.aktual());
          if (a.length==1) {
            con = (Container) a[0];
          } else {
            for(j=0;j<a.length;j++) {
              if (a[j].get_identification().version().compareTo(ref.version)==0) {
                con = (Container) a[j];
                break;
              }
            }
          }
          ref.sn.names.toNext();
        }
        return con.lookup(ref.name, ref.version);
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      }
    }
  }

  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    int i,j;
    if (isNew) {
      CompContainer con = rep;
      ref.sn.names.toFirst();
      for(i=0;i<ref.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) ref.sn.names.aktual());
        con = (CompContainer) a[0];
        ref.sn.names.toNext();
      }  
      CompContained sec = con.lookup(ref.name, ref.version);
      sec.checkConsist(props, rep);
    } else {
      try {
        Container con = rep.cdlRepository;
        ref.sn.names.toFirst();
        for(i=0;i<ref.sn.names.size();i++) {
          Contained[] a = con.lookup_name((String) ref.sn.names.aktual());
          if (a.length==1) {
            con = (Container) a[0];
          } else {
            for(j=0;j<a.length;j++) {
              if (a[j].get_identification().version().compareTo(ref.version)==0) {
                con = (Container) a[j];
                break;
              }
            }
          }
          ref.sn.names.toNext();
        }
        Contained sec = con.lookup(ref.name, ref.version);
        checkNormalConsist(sec,props, rep.cdlRepository);
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      }
    }
  }

  private void checkNormalConsist(TIRObject obj, EnumList props, CDLRepository rep) throws CDLExceptCheck, TIRExceptLock, RemoteException, CDLExceptRemote {
    int i;
    if (obj instanceof Container) {
      Contained[] con = ((Container) obj).contents(null);
      for (i=0;i<con.length;i++) {
        checkNormalConsist(con[i], props, rep);
      }
      switch (obj.get_def_kind().value()) {
      case DefinitionKind.dk_Interface:
        InterfaceDef[] base = ((InterfaceDef) obj).base_interfaces();
        for (i=0;i<base.length;i++) {
          checkNormalConsist(base[i], props, rep);
        }
        break;
      case DefinitionKind.dk_Union:
        checkNormalConsist(((UnionDef) obj).switch_type(), props, rep);
        break;
      }
    } else { // non Container
      switch (obj.get_def_kind().value()) {
      case DefinitionKind.dk_Typedef:
        checkNormalConsist(((TypedefDef) obj).original_type(), props, rep);
        break;
      case DefinitionKind.dk_Array:
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((ArrayDef) obj).length())))
          throw new CDLExceptCheck("Unknown property");
        checkNormalConsist(((ArrayDef) obj).element_type(), props, rep);
        break;
      case DefinitionKind.dk_Sequence:
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((SequenceDef) obj).bound())))
          throw new CDLExceptCheck("Unknown property");
        checkNormalConsist(((SequenceDef) obj).element_type(), props, rep);
        break;
      case DefinitionKind.dk_String:
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((StringDef) obj).bound())))
          throw new CDLExceptCheck("Unknown property");
        break;
      case DefinitionKind.dk_Wstring:
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((WstringDef) obj).bound())))
          throw new CDLExceptCheck("Unknown property");
        break;
      case DefinitionKind.dk_Fixed:
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((FixedDef) obj).digits())))
          throw new CDLExceptCheck("Unknown property");
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((FixedDef) obj).scale())))
          throw new CDLExceptCheck("Unknown property");
        break;
      case DefinitionKind.dk_StructMember:
        checkNormalConsist(((StructMember) obj).type(), props, rep);
        break;
      case DefinitionKind.dk_UnionMember:
        checkNormalConsist(((UnionMember) obj).type(), props, rep);
        ExprOperDef[] labs = ((UnionMember) obj).label();
        for(i=0;i<labs.length;i++) {
          if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(labs[i])))
            throw new CDLExceptCheck("Unknown property");
        }
        break;
      case DefinitionKind.dk_Attribute:
        checkNormalConsist(((AttributeDef) obj).type(), props, rep);
        break;
      case DefinitionKind.dk_Operation:
        checkNormalConsist(((OperationDef) obj).result(), props, rep);
        ExceptionDef[] exs = ((OperationDef) obj).exceptions();
        for (i=0;i<exs.length;i++) {
          checkNormalConsist(exs[i], props, rep);
        }
        ParamDescr[] pars = ((OperationDef) obj).params();
        for (i=0;i<exs.length;i++) {
          checkNormalConsist(pars[i].type(), props, rep);
        }
        break;
      case DefinitionKind.dk_Constant:
        checkNormalConsist(((ConstantDef) obj).type(), props, rep);
        if (!CompExprOper.testPropInExpr(props, CompExprOper.scanNormalExpr(((ConstantDef) obj).value())))
          throw new CDLExceptCheck("Unknown property");
        break;
      // others are OK 
      }
    }
  }

  public CDLType toNormal(CDLRepository newRep) throws CDLExceptRemote, CDLExceptToNormal, CDLExceptLock {
    try {
      int i,j;
      Container con = newRep;
      ref.sn.names.toFirst();
      for(i=0;i<ref.sn.names.size();i++) {
        Contained[] a = con.lookup_name((String) ref.sn.names.aktual());
        if (a==null || a.length==0)
          throw new CDLExceptToNormal("Can't find type "+ref);
        if (a.length==1) {
          if (!(a[0] instanceof Container))
            throw new CDLExceptToNormal("Can't find type "+ref);
          con = (Container) a[0];
        } else {
          boolean found = false;
          for(j=0;j<a.length;j++) {
            if (a[j].get_identification().version().compareTo(ref.version)==0) {
              if (!(a[j] instanceof Container))
                throw new CDLExceptToNormal("Can't find type "+ref);
              con = (Container) a[j];
              found = true;
              break;
            }
          }
          if (!found)
            throw new CDLExceptToNormal("Can't find type "+ref);
        }
        ref.sn.names.toNext();
      }
      Contained sec = con.lookup(ref.name, ref.version);
      if (sec==null)
        throw new CDLExceptToNormal("Can't find type "+ref);
      return (CDLType) sec;
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    }
  }

  /** if object's kind is type */
  public static boolean isType(int kind) {
    switch (kind) {
      case ObjectsKind.o_Typedef:
      case ObjectsKind.o_Struct:
      case ObjectsKind.o_Union:
      case ObjectsKind.o_Enum:
      case ObjectsKind.o_Primitive:
      case ObjectsKind.o_String:
      case ObjectsKind.o_Sequence:
      case ObjectsKind.o_Array:
      case ObjectsKind.o_Wstring:
      case ObjectsKind.o_Fixed:
      case ObjectsKind.o_Interface:
      case ObjectsKind.o_Frame:
        return true;
    }
    return false;
  }

  /** if object's kind is const_type */
  public static boolean isConstType(int kind) {
    switch (kind) {
      case ObjectsKind.o_Primitive:
      case ObjectsKind.o_String:
      case ObjectsKind.o_Wstring:
      case ObjectsKind.o_Fixed:
      case ObjectsKind.o_Enum:
        return true;
    }
    return false;
  }

  /** if object's kind is param_type */
  public static boolean isParamType(int kind) {
    switch (kind) {
      case ObjectsKind.o_Primitive:
      case ObjectsKind.o_String:
      case ObjectsKind.o_Wstring:
      case ObjectsKind.o_Struct:
      case ObjectsKind.o_Union:
      case ObjectsKind.o_Enum:
      case ObjectsKind.o_Interface:
        return true;
    }
    return false;
  }

  /** if CompReffer typedef, scan through it (what must be type) */
  public static boolean isConstType(CompReffer what, CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
   boolean ok = false;
   int kind = what.what;
   CompReffer r = what;
   while (!ok) {
     if (!CompReffer.isConstType(kind)) {
       if (kind == ObjectsKind.o_Typedef) {
         int i,j;
         FullID id = r.ref;
         if (r.isNew) {  // in actually added objects
           CompContainer con = rep;
           id.sn.names.toFirst();
           for(i=0;i<id.sn.names.size();i++) {
             CompContained[] a = con.lookup((String) id.sn.names.aktual());
             if (a==null || a.length==0) 
               return false;
             if (a.length==1) {
               if (a[0].isContainer()) con = (CompContainer) a[0];
               else return false;
             } else {
               return false;
             }
             id.sn.names.toNext();
           }
           CompContained sec = con.lookup(id.name, id.version);
           if (sec!=null) {
             CompType sectype = ((CompTypedef) sec).type;
             if (!isConstType(sectype.objectKind())) {
               if (sectype.objectKind()==ObjectsKind.o_none) { // is CompReffer
                 r = (CompReffer) sectype;
                 kind = r.what;
               } else  
                 return false;
             } else {
               what.basereffer = sectype.objectKind();
               if (sectype.objectKind()==ObjectsKind.o_Primitive) 
                 what.baserefferprim = ((CompPrimitive)sectype).kind;
               ok = true;
             }
           } else
             return false;
         } else {  // in normal repository
           try {
             Container con = rep.cdlRepository;
             id.sn.names.toFirst();
             for(i=0;i<id.sn.names.size();i++) {
               Contained[] a = con.lookup_name((String) id.sn.names.aktual());
               if (a==null || a.length==0) 
                 return false;
               if (a.length==1) {
                 if (a[0] instanceof Container) con = (Container) a[0];
                 else return false;
               } else {
                 boolean found = false;
                 for(j=0;j<a.length;j++) {
                   if (a[j].get_identification().version().compareTo(id.version)==0) {
                     if ( !(a[j] instanceof Container)) return false;
                     found = true;
                     con = (Container) a[j];
                     break;
                   }
                 }
                 if (!found) return false;
               }
               id.sn.names.toNext();
             }
             Contained sec = con.lookup(id.name, id.version);
             if (sec!=null) {
               CDLType sectype = ((TypedefDef) sec).original_type();
               if (!isConstType(sectype.get_def_kind().value())) {
                 if (sectype.get_def_kind().value() == DefinitionKind.dk_Typedef) {
                   CompReffer ret = new CompReffer();
                   ret.what = sectype.get_def_kind().value();
                   ret.ref = new FullID(true);
                   Identification idl;
                   ret.ref.name = new String((idl = ((TypedefDef)sectype).get_identification()).name());
                   ret.ref.version = new String(idl.version());
                   ret.ref.isin = IDKind.version;
                   ret.isNew = false;
                   AbsoluteName abs = idl.absolute_name();
                   for(j=0;j<abs.size();j++) {
                     ret.ref.sn.addScope(new String(abs.elementAt(j)));
                   }
                   r = ret;
                   kind = r.what;
                 } else
                   return false;
               } else {
                 what.basereffer = sectype.get_def_kind().value();
                 if (sectype.get_def_kind().value()==DefinitionKind.dk_Primitive) 
                   what.baserefferprim = ((PrimitiveDef)sectype).kind().value();
                 ok = true;
               }
             } else {
               return false;
             }
           } catch (TIRExceptLock e) {
             throw new CDLExceptLock("Repository is locked.");
           } catch (RemoteException e) {
             throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
           }
         }
       } else {
         return false;
       }
     } else {
       what.basereffer = kind;
       ok = true;
     }
   } 
   return true;
  }

  /** if object's kind is switch_type */
  public static boolean isSwitchType(int a) {
    switch (a) {
      case ObjectsKind.o_Primitive:
      case ObjectsKind.o_Enum:
        return true;
    }
    return false;
  }

  public static boolean isSwitchType(CompReffer what, CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
   boolean ok = false;
   int kind = what.what;
   CompReffer r = what;
   while (!ok) {
     if (!CompReffer.isConstType(kind)) {
       if (kind == ObjectsKind.o_Typedef) {
         int i,j;
         FullID id = r.ref;
         if (r.isNew) {  // in actually added objects
           CompContainer con = rep;
           id.sn.names.toFirst();
           for(i=0;i<id.sn.names.size();i++) {
             CompContained[] a = con.lookup((String) id.sn.names.aktual());
             if (a==null || a.length==0) 
               return false;
             if (a.length==1) {
               if (a[0].isContainer()) con = (CompContainer) a[0];
               else return false;
             } else {
               return false;
             }
             id.sn.names.toNext();
           }
           CompContained sec = con.lookup(id.name, id.version);
           if (sec!=null) {
             CompType sectype = ((CompTypedef) sec).type;
             if (!isConstType(sectype.objectKind())) {
               if (sectype.objectKind()==ObjectsKind.o_none) { // is CompReffer
                 r = (CompReffer) sectype;
                 kind = r.what;
               } else  
                 return false;
             } else {
               what.basereffer = sectype.objectKind();
               if (sectype.objectKind()==ObjectsKind.o_Primitive) {
                 switch (((CompPrimitive)sectype).kind) {
                   case CompPrimKind.p_short:
                   case CompPrimKind.p_long:
                   case CompPrimKind.p_longlong:
                   case CompPrimKind.p_ushort:
                   case CompPrimKind.p_ulong:
                   case CompPrimKind.p_ulonglong:
                   case CompPrimKind.p_char:
                   case CompPrimKind.p_boolean:
                     ok = true;
                     what.baserefferprim = ((CompPrimitive)sectype).kind;
                     break;
                   default: return false;
                 }
               } else 
                 ok = true;
             }  
           } else
             return false;
         } else {  // in normal repository
           try {
             Container con = rep.cdlRepository;
             id.sn.names.toFirst();
             for(i=0;i<id.sn.names.size();i++) {
               Contained[] a = con.lookup_name((String) id.sn.names.aktual());
               if (a==null || a.length==0) 
                 return false;
               if (a.length==1) {
                 if (a[0] instanceof Container) con = (Container) a[0];
                 else return false;
               } else {
                 boolean found = false;
                 for(j=0;j<a.length;j++) {
                   if (a[j].get_identification().version().compareTo(id.version)==0) {
                     if ( !(a[j] instanceof Container)) return false;
                     found = true;
                     con = (Container) a[j];
                     break;
                   }
                 }
                 if (!found) return false;
               }
               id.sn.names.toNext();
             }
             Contained sec = con.lookup(id.name, id.version);
             if (sec!=null) {
               CDLType sectype = ((TypedefDef) sec).original_type();
               if (!isConstType(sectype.get_def_kind().value())) {
                 if (sectype.get_def_kind().value() == DefinitionKind.dk_Typedef) {
                   CompReffer ret = new CompReffer();
                   ret.what = sectype.get_def_kind().value();
                   ret.ref = new FullID(true);
                   Identification idl;
                   ret.ref.name = new String((idl = ((TypedefDef)sectype).get_identification()).name());
                   ret.ref.version = new String(idl.version());
                   ret.ref.isin = IDKind.version;
                   ret.isNew = false;
                   AbsoluteName abs = idl.absolute_name();
                   for(j=0;j<abs.size();j++) {
                     ret.ref.sn.addScope(new String(abs.elementAt(j)));
                   }
                   r = ret;
                   kind = r.what;
                 } else
                   return false;
               } else {
                 what.basereffer = sectype.get_def_kind().value();
                 if (sectype.get_def_kind().value()==DefinitionKind.dk_Primitive) {
                   switch (((PrimitiveDef)sectype).kind().value()) {
                     case PrimitiveKind.pk_short:
                     case PrimitiveKind.pk_long:
                     case PrimitiveKind.pk_longlong:
                     case PrimitiveKind.pk_ushort:
                     case PrimitiveKind.pk_ulong:
                     case PrimitiveKind.pk_ulonglong:
                     case PrimitiveKind.pk_char:
                     case PrimitiveKind.pk_boolean:
                       what.baserefferprim = ((PrimitiveDef)sectype).kind().value();
                       ok = true;
                       break;
                     default: return false;
                   }
                 } else
                   ok = true;
               }
             } else {
               return false;
             }
           } catch (TIRExceptLock e) {
             throw new CDLExceptLock("Repository is locked.");
           } catch (RemoteException e) {
             throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
           }
         }
       } else {
         return false;
       }
     } else {
       what.basereffer = kind;
       ok = true;
     }
   } 
   return true;
  }

  private static CompType goThroughCompArrSeq(CompType arseq) {
    while (arseq instanceof CompArray || arseq instanceof CompSequence) {
      if (arseq instanceof CompArray) {
        arseq = ((CompArray) arseq).type;
      } else {
        arseq = ((CompSequence) arseq).type;
      }
    }
    return arseq;
  }

  private static CDLType goThroughCDLArrSeq(CDLType arseq) throws RemoteException {
    while (arseq instanceof ArrayDef || arseq instanceof SequenceDef) {
      if (arseq instanceof ArrayDef) {
        arseq = ((ArrayDef) arseq).element_type();
      } else {
        arseq = ((SequenceDef) arseq).element_type();
      }
    }
    return arseq;
  }

  /** if CompReffer is typedef, scan through it */
  public static boolean isParamType(CompReffer what, CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
   boolean ok = false;
   int kind = what.what;
   CompReffer r = what;
   while (!ok) {
     if (!CompReffer.isParamType(kind)) {
       if (kind == ObjectsKind.o_Typedef) {
         int i,j;
         FullID id = r.ref;
         if (r.isNew) {  // in actually added objects
           CompContainer con = rep;
           id.sn.names.toFirst();
           for(i=0;i<id.sn.names.size();i++) {
             CompContained[] a = con.lookup((String) id.sn.names.aktual());
             if (a==null || a.length==0) 
               return false;
             if (a.length==1) {
               if (a[0].isContainer()) con = (CompContainer) a[0];
               else return false;
             } else {
               return false;
             }
             id.sn.names.toNext();
           }
           CompContained sec = con.lookup(id.name, id.version);
           if (sec!=null) {
             CompType sectype = ((CompTypedef) sec).type;
             if (!isParamType(sectype.objectKind())) {
               if (sectype.objectKind()==ObjectsKind.o_none) { // is CompReffer
                 r = (CompReffer) sectype;
                 kind = r.what;
               } else {
                 if (sectype.objectKind()==ObjectsKind.o_Array ||
                      sectype.objectKind()==ObjectsKind.o_Sequence) {
                   sectype = goThroughCompArrSeq(sectype);
                   if (!isParamType(sectype.objectKind())) {
                     if (sectype.objectKind()==ObjectsKind.o_none) { // is CompReffer
                       r = (CompReffer) sectype;
                       kind = r.what;
                     }
                   } else {
                     what.basereffer = sectype.objectKind();
                     if (sectype.objectKind()==ObjectsKind.o_Primitive) 
                       what.baserefferprim = ((CompPrimitive)sectype).kind;
                     ok = true;
                   }
                   
                 } else
                   return false;
               }
             } else {
               what.basereffer = sectype.objectKind();
               if (sectype.objectKind()==ObjectsKind.o_Primitive) 
                 what.baserefferprim = ((CompPrimitive)sectype).kind;
               ok = true;
             }
           } else
             return false;
         } else {  // in normal repository
           try {
             Container con = rep.cdlRepository;
             id.sn.names.toFirst();
             for(i=0;i<id.sn.names.size();i++) {
               Contained[] a = con.lookup_name((String) id.sn.names.aktual());
               if (a==null || a.length==0) 
                 return false;
               if (a.length==1) {
                 if (a[0] instanceof Container) con = (Container) a[0];
                 else return false;
               } else {
                 boolean found = false;
                 for(j=0;j<a.length;j++) {
                   if (a[j].get_identification().version().compareTo(id.version)==0) {
                     if ( !(a[j] instanceof Container)) return false;
                     found = true;
                     con = (Container) a[j];
                     break;
                   }
                 }
                 if (!found) return false;
               }
               id.sn.names.toNext();
             }
             Contained sec = con.lookup(id.name, id.version);
             if (sec!=null) {
               CDLType sectype = ((TypedefDef) sec).original_type();
               if (!isConstType(sectype.get_def_kind().value())) {
                 int defk = sectype.get_def_kind().value();
                 if (defk == DefinitionKind.dk_Typedef) {
                   CompReffer ret = new CompReffer();
                   ret.what = sectype.get_def_kind().value();
                   ret.ref = new FullID(true);
                   Identification idl;
                   ret.ref.name = new String((idl = ((TypedefDef)sectype).get_identification()).name());
                   ret.ref.version = new String(idl.version());
                   ret.ref.isin = IDKind.version;
                   ret.isNew = false;
                   AbsoluteName abs = idl.absolute_name();
                   for(j=0;j<abs.size();j++) {
                     ret.ref.sn.addScope(new String(abs.elementAt(j)));
                   }
                   r = ret;
                   kind = r.what;
                 } else {

                   if (defk == DefinitionKind.dk_Array ||
                       defk == DefinitionKind.dk_Sequence) {
                     sectype = goThroughCDLArrSeq(sectype);

                     if (!isConstType(sectype.get_def_kind().value())) {
                       if (sectype.get_def_kind().value()== DefinitionKind.dk_Typedef) {
                         CompReffer ret = new CompReffer();
                         ret.what = sectype.get_def_kind().value();
                         ret.ref = new FullID(true);
                         Identification idl;
                         ret.ref.name = new String((idl = ((TypedefDef)sectype).get_identification()).name());
                         ret.ref.version = new String(idl.version());
                         ret.ref.isin = IDKind.version;
                         ret.isNew = false;
                         AbsoluteName abs = idl.absolute_name();
                         for(j=0;j<abs.size();j++) {
                           ret.ref.sn.addScope(new String(abs.elementAt(j)));
                         }
                         r = ret;
                         kind = r.what;
                       } else {
                         return false;
                       }
                     } else {
                       what.basereffer = sectype.get_def_kind().value();
                       if (sectype.get_def_kind().value()==DefinitionKind.dk_Primitive) 
                         what.baserefferprim = ((PrimitiveDef)sectype).kind().value();
                       ok = true;
                     }

                   } else {
                     return false;
                   }
                 }
               } else {
                 what.basereffer = sectype.get_def_kind().value();
                 if (sectype.get_def_kind().value()==DefinitionKind.dk_Primitive) 
                   what.baserefferprim = ((PrimitiveDef)sectype).kind().value();
                 ok = true;
               }
             } else {
               return false;
             }
           } catch (TIRExceptLock e) {
             throw new CDLExceptLock("Repository is locked.");
           } catch (RemoteException e) {
             throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
           }
         }
       } else {
         return false;
       }
     } else {
       what.basereffer = kind;
       ok = true;
     }
   } 
   return true;
  }

  /** if CompReffer is typedef, scan through it */
  public static boolean isInterface(CompReffer what, CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
   boolean ok = false;
   int kind = what.what;
   CompReffer r = what;
   while (!ok) {
     if (kind != ObjectsKind.o_Interface) {
       if (kind == ObjectsKind.o_Typedef) {
         int i,j;
         FullID id = r.ref;
         if (r.isNew) {  // in actually added objects
           CompContainer con = rep;
           id.sn.names.toFirst();
           for(i=0;i<id.sn.names.size();i++) {
             CompContained[] a = con.lookup((String) id.sn.names.aktual());
             if (a==null || a.length==0) 
               return false;
             if (a.length==1) {
               if (a[0].isContainer()) con = (CompContainer) a[0];
               else return false;
             } else {
               return false;
             }
             id.sn.names.toNext();
           }
           CompContained sec = con.lookup(id.name, id.version);
           if (sec!=null) {
             CompType sectype = ((CompTypedef) sec).type;
             if (sectype.objectKind() != ObjectsKind.o_Interface) {
               if (sectype.objectKind()==ObjectsKind.o_none) { // is CompReffer
                 r = (CompReffer) sectype;
                 kind = r.what;
               } else  
                 return false;
             } else {
               what.basereffer = sectype.objectKind();
               what.baserefferprim = -1;
               ok = true;
             }
           } else
             return false;
         } else {  // in normal repository
           try {
             Container con = rep.cdlRepository;
             id.sn.names.toFirst();
             for(i=0;i<id.sn.names.size();i++) {
               Contained[] a = con.lookup_name((String) id.sn.names.aktual());
               if (a==null || a.length==0) 
                 return false;
               if (a.length==1) {
                 if (a[0] instanceof Container) con = (Container) a[0];
                 else return false;
               } else {
                 boolean found = false;
                 for(j=0;j<a.length;j++) {
                   if (a[j].get_identification().version().compareTo(id.version)==0) {
                     if ( !(a[j] instanceof Container)) return false;
                     found = true;
                     con = (Container) a[j];
                     break;
                   }
                 }
                 if (!found) return false;
               }
               id.sn.names.toNext();
             }
             Contained sec = con.lookup(id.name, id.version);
             if (sec!=null) {
               CDLType sectype = ((TypedefDef) sec).original_type();
               if (sectype.get_def_kind().value() != DefinitionKind.dk_Interface) {
                 if (sectype.get_def_kind().value() == DefinitionKind.dk_Typedef) {
                   CompReffer ret = new CompReffer();
                   ret.what = sectype.get_def_kind().value();
                   ret.ref = new FullID(true);
                   Identification idl;
                   ret.ref.name = new String((idl = ((TypedefDef)sectype).get_identification()).name());
                   ret.ref.version = new String(idl.version());
                   ret.ref.isin = IDKind.version;
                   ret.isNew = false;
                   AbsoluteName abs = idl.absolute_name();
                   for(j=0;j<abs.size();j++) {
                     ret.ref.sn.addScope(new String(abs.elementAt(j)));
                   }
                   r = ret;
                   kind = r.what;
                 } else
                   return false;
               } else {
                 what.basereffer = sectype.get_def_kind().value();
                 what.baserefferprim = -1;
                 ok = true;
               }
             } else {
               return false;
             }
           } catch (TIRExceptLock e) {
             throw new CDLExceptLock("Repository is locked.");
           } catch (RemoteException e) {
             throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
           }
         }
       } else {
         return false;
       }
     } else {
       ok = true;
     }
   } 
   return true;
  }
}
