/* $Id: GeneratedObject.java,v 1.2 2004/05/20 14:23:51 bures Exp $ */
package SOFA.SOFAnode.Made.CodeGen;

import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.PrimitiveKind;
import SOFA.SOFAnode.Made.TIR.TIRObject;

/** Superclass for representation of generated objects.
  * 
  * @author Petr Hnetynka
  */
abstract public class GeneratedObject {
  /** Package with standard holders, helpers,... */
  public static final String stdPackage = "cz.cuni.sofa.lib.";
  /** Default name of ostream in helpers */
  public static final String defaultOstream = "ostream";
  /** Default name of istream in helpers */
  public static final String defaultIstream = "istream";

  /** In reference kind */
  public static final int INREF = 1;
  /** Out (or inout) reference kind */
  public static final int OUTREF = 2;

  /** Key of object. It is string representation of id of objects and it is used in for searching correct type */
  public String key;  


  /** Compares key of object with given key.
    * 
    * @param key   key
    * @return <tt>true</tt> if keys are same
    */
  public boolean compareKey(String key) {
    return this.key.compareTo(key) == 0;
  }

  /** Returns generated reference to object.
    *
    * @param kind Kind of reference - <tt>INREF</tt> or <tt>OUTREF</tt>
    */
  public abstract String genReference(int kind);
  
  /** Writes reference to the <tt>out</tt>.
    * 
    * @param kind Kind of reference - <tt>INREF</tt> or <tt>OUTREF</tt>
    * @param out  output writer
    * @exception IOException  if an I/O error occurs
    */
  public abstract void genReference(CGFileWriter out, int kind) throws IOException;

  /** Returns special generated reference to object - only for some types.
    *
    * @param kind Kind of reference - <tt>INREF</tt> or <tt>OUTREF</tt>
    * @param specType type of reference
    */
  public String genSpecReference(int kind, int specType) { return ""; }
  
  /** Writes special reference to the <tt>out</tt> - only for some types.
    * 
    * @param kind Kind of reference - <tt>INREF</tt> or <tt>OUTREF</tt>
    * @param out  output writer
    * @param specType type of reference
    * @exception IOException  if an I/O error occurs
    */
  public void genSpecReference(CGFileWriter out, int kind, int specType) throws IOException {;}
  

  /** Writes default value of the type to the <tt>out</tt>.
    * 
    * @param out  output writer
    * @exception IOException  if an I/O error occurs
    */
  public abstract void defaultValue(CGFileWriter out) throws IOException;

  /** Returns default value of the type.
    * 
    * @param out  output writer
    * @exception IOException  if an I/O error occurs
    */
  public abstract String defaultValue() throws IOException;

  /** Writes code for the reading of the value with this type.
    * 
    * @param out  output writer
    * @param varName "written variable"
    * @exception IOException  if an I/O error occurs
    */
  public abstract void genReadType(CGFileWriter out, String varName) throws IOException;

  /** Returns code for the reading of the value with this type.
    * 
    * @param varName "written variable"
    * @exception IOException  if an I/O error occurs
    */
  public abstract String genReadType(String varName) throws IOException;

  /** Writes code for the writing of the value with this type.
    * 
    * @param out  output writer
    * @param varName "read variable"
    * @exception IOException  if an I/O error occurs
    */
  public abstract void genWriteType(CGFileWriter out, String varName) throws IOException;

  /** Returns code for the writing of the value with this type.
    * 
    * @param varName "read variable"
    * @exception IOException  if an I/O error occurs
    */
  public abstract String genWriteType(String varName) throws IOException;

  /** Returns key created from given repository object.
    *
    * @param obj   repository object
    * @exception RemoteException  remote exception
    */
  public static String createKey(TIRObject obj) throws RemoteException {
    if (obj instanceof Contained) {
      Identification idl = ((Contained) obj).get_identification();
      return idl.absolute_name().name()+"?"+idl.version();
    }
    if (obj instanceof PrimitiveDef) {
      switch (((PrimitiveDef) obj).kind().value()) {
       case PrimitiveKind.pk_void:
         return "void";
       case PrimitiveKind.pk_ushort:
         return "ushort";
       case PrimitiveKind.pk_short:
         return "short";
       case PrimitiveKind.pk_ulong:
         return "ulong";
       case PrimitiveKind.pk_long:
         return "long";
       case PrimitiveKind.pk_ulonglong:
         return "ulonglong";
       case PrimitiveKind.pk_longlong:
         return "longlong";
       case PrimitiveKind.pk_float:
         return "float";
       case PrimitiveKind.pk_double:
         return "double";
       case PrimitiveKind.pk_longdouble:
         return null;  // !!!! no mapping
       case PrimitiveKind.pk_boolean:
         return "boolean";
       case PrimitiveKind.pk_wchar:
         return "wchar";
       case PrimitiveKind.pk_char:
         return "char";
       case PrimitiveKind.pk_octet:
         return "octet";
       case PrimitiveKind.pk_wstring:
         return "wstring";
       case PrimitiveKind.pk_string:
         return "string";
       case PrimitiveKind.pk_any:
         return "any";
       case PrimitiveKind.pk_object:
         return "object";
      }
    }
    return null;
  }

  
  static String[] reservedWords = { 
        "abstract", "default", "if", "private", "throw",
        "boolean", "do", "implements", "protected", "throws",
        "break", "double", "import", "public", "transient",
        "byte", "else", "instanceof", "return", "try",
        "case", "extends", "int", "short", "void",
        "catch", "final", "interface", "static", "volatile",
        "char", "finally", "long", "super", "while",
        "class", "float", "native", "switch",
        "const", "for", "new", "synchronized",
        "continue", "goto", "package", "this",
        "true", "false", "null",
        "clone", "equals", "finalize", "getClass", "hashCode",
        "notify", "notifyAll", "toString", "wait",
        };


  /** Test, if it is reserved name (then underscore will be prepended)
    * @param name  tested name
    */
  public static boolean isReservedName(String name) {
    for (int i=0;i<reservedWords.length;i++) {
      if (reservedWords[i].compareToIgnoreCase(name)==0)
        return true;
    }
    String lname = name.toLowerCase();
    if (lname.endsWith("helper"))
      return true;
    if (lname.endsWith("holder"))
      return true;
    if (lname.endsWith("operations"))
      return true;
    if (lname.endsWith("package"))
      return true;
    return false;
  }
}
