/* $Id: ConstantDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.ArrayDef;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.ConstantDef;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ExprOperDef;
import SOFA.SOFAnode.Made.TIR.FixedDef;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.SequenceDef;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.StringDef;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.WstringDef;

public class ConstantDefImpl extends ContainedImpl implements ConstantDef {
  CDLType typ;
  ExprOperDef val;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  ConstantDefImpl normal;
  long workId;

  public ConstantDefImpl(Identification ident, Container in, Repository inrep, CDLType t, ExprOperDef v, long workId) throws RemoteException {
    super(ident,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Constant);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    typ = t;
    val = v;
    normal = null;
    this.workId = workId;
  }
 
  public ConstantDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Constant);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    typ = null;
    val = null;
    normal = null;
    this.workId = -1L;
  }

  public ConstantDefImpl(ConstantDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    super(a.id,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Constant);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    typ = a.typ;
    val = a.val;
    normal = a;
    this.workId = workId;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    super.tag(t);
  }

  public CDLType type() throws RemoteException {
    return typ;
  }
  
  public ExprOperDef value() throws RemoteException {
    return val;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      boolean closeFile = false;
      if (st.curOutFile==null) {
        st.curOutFile = new DataOutputStream(new FileOutputStream(st.currentFile));
	closeFile = true;
      } else {
        ;
      }
      ((IdentificationImpl)id).save(st.curOutFile);   // saving identification
      Storage.writeCDLType(st, typ);
      Storage.writeExprOperDef(st, val);
      if (closeFile) {
        st.curOutFile.close();
	st.curOutFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      boolean closeFile = false;
      if (st.curInFile==null) {  // saving in module
        st.curInFile = new DataInputStream(new FileInputStream(st.currentFile));
	closeFile = true;
      } else {
        ;
      }
      id = new IdentificationImpl();
      ((IdentificationImpl)id).load(st.curInFile);
      typ = Storage.readCDLType(st);
      val = Storage.readExprOperDef(st);
      if (closeFile) {
        st.curInFile.close();
	st.curInFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
    if (typ.get_def_kind().value() == DefinitionKind.dk_none) {   // is typ NotYetInRep?
      if (((NotYetInRep) typ).defKind == DefinitionKind.dk_Primitive)
        typ = r.get_prim(((NotYetInRep) typ).primKind);
      else {     // in tp there is reference to some type
        AbsoluteName nm = ((NotYetInRep) typ).nm;
	Container con = r;  // begin search from repository
	Contained[] rt;
	Contained ret;
	IdentificationImpl id;
        for (int i=0;i<nm.size()-1;i++) {  // last element is name of object
	  id = new IdentificationImpl(nm.elementAt(0),"::"+nm.elementAt(i),"");
          rt = ((SContainer) con).slookup_name(id);
	  if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
          if (rt.length==1) {
            con = (Container) rt[0];
	  } else {  // type in struct (etc.) - more versions of this struct
	    boolean found = false;
            id.version(((NotYetInRep) typ).id.version());  // substruct version is inherited from parent
            int j = 0;
	    for (j=0;j<rt.length;j++) {
	      if (id.is_short_equal(rt[j].get_identification())) {
	        found = true;
		break;  // -- break for
	      }
	    } // -- end of for
	    if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
	    con = (Container) rt[j];
	  }
	}
	if ((ret = ((SContainer) con).slookup(((NotYetInRep) typ).id))==null)
	  throw new TIRExceptStorage("Can't find type " + nm.name());
	typ = (CDLType) ret;
	if (typ.get_def_kind().value()==DefinitionKind.dk_Array ||
            typ.get_def_kind().value()==DefinitionKind.dk_Sequence ||
            typ.get_def_kind().value()==DefinitionKind.dk_String ||
            typ.get_def_kind().value()==DefinitionKind.dk_Wstring)
	  ((TIRImplObject) typ).postLoad(r);
      }
    } else {
      if (typ.get_def_kind().value()==DefinitionKind.dk_Array ||
          typ.get_def_kind().value()==DefinitionKind.dk_Sequence ||
          typ.get_def_kind().value()==DefinitionKind.dk_String ||
          typ.get_def_kind().value()==DefinitionKind.dk_Wstring)
        ((TIRImplObject) typ).postLoad(r);
    }
    ((TIRImplObject) val).postLoad(r);

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      switch (typ.get_def_kind().value()) {
        case DefinitionKind.dk_Primitive:
          typ = ((SRepository)rep).get_prim(((PrimitiveDef)typ).kind().value());
          break;
        case DefinitionKind.dk_Array:
          ArrayDef nt = new ArrayDefImpl(ExprHelp.toNormal(((ArrayDef) typ).length()), ((ArrayDef) typ).element_type());
          typ = nt;
          ((TIRImplObject)typ).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Sequence:
          SequenceDef st = new SequenceDefImpl(ExprHelp.toNormal(((SequenceDef) typ).bound()), ((SequenceDef) typ).element_type());
          typ = st;
          ((TIRImplObject)typ).doCommit(in,rep);
          break;
        case DefinitionKind.dk_String:
          StringDef str = new StringDefImpl(ExprHelp.toNormal(((StringDef) typ).bound()));
          typ = str;
          ((TIRImplObject)typ).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Wstring:
          WstringDef wstr = new WstringDefImpl(ExprHelp.toNormal(((WstringDef) typ).bound()));
          typ = wstr;
          ((TIRImplObject)typ).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Fixed:
          FixedDef ft = new FixedDefImpl(ExprHelp.toNormal(((FixedDef) typ).digits()), ExprHelp.toNormal(((FixedDef) typ).scale()));
          typ = ft;
          ((TIRImplObject)typ).doCommit(in,rep);
          break;
        default:
          Identification idl = ((Contained)typ).get_identification();
          typ = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
          break;
      }
      val = ExprHelp.toNormal(val);
      ((TIRImplObject)val).doCommit(in,rep);
    } else {
      normal=null;
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
