/* $Id: ExceptionDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.EnumDef;
import SOFA.SOFAnode.Made.TIR.ExceptionDef;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.StructDef;
import SOFA.SOFAnode.Made.TIR.StructMember;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;
import SOFA.SOFAnode.Made.TIR.UnionDef;

public class ExceptionDefImpl extends ContainerImpl implements ExceptionDef, TIRImplObject, SContained {
  protected Identification id;
  protected Container parent;
  protected Repository rep;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  ExceptionDefImpl normal;
  long workId;

  public ExceptionDefImpl(Identification ident, Container in, Repository inrep, long workId) throws RemoteException {
    id = ident;
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Exception);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    normal = null;
    this.workId = workId;
  }

  public ExceptionDefImpl(Container in, Repository inrep) throws RemoteException {
    id = null;
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Exception);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    normal = null;
    this.workId = -1L;
  }

  public ExceptionDefImpl(ExceptionDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    id = a.id;
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Exception);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    normal = a;
    this.workId = workId;
  }

  /* form interface Contained */
  public Identification get_identification() throws RemoteException {
    return id;
  }

  /* form interface Contained */
  public Container get_defined_in() throws RemoteException {
    return parent;
  }

  /* form interface Contained */
  public Container get_containing_repository() throws RemoteException {
    return rep;
  }

  /* form interface Contained */
  public AbsoluteName get_absolute_name() throws RemoteException {
    AbsoluteNameImpl absName;
    if (rep == parent) {  // object is contained in repository
      absName = new AbsoluteNameImpl("::"+id.name());
    } else {
      absName = new AbsoluteNameImpl(((SContained)parent).get_absolute_name().name()+"::"+id.name());
    }
    return (AbsoluteName) absName;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    String ebt = id.branchTag();
    if (t.compareTo(ebt)==0)
      throw new TIRExceptCreate("This tag exists");
    String[] et = id.tag();
    int i,j;
    for (i=0;i<et.length;i++) {
      if (t.compareTo(et[i])==0)
       throw new TIRExceptCreate("This tag exists");       
    }
    Contained[] sibl = parent.lookup_name(id);
    for (j=0;j<sibl.length;j++) {
      Identification idl = sibl[j].get_identification();
      
      ebt = idl.branchTag();
      if (t.compareTo(ebt)==0)
        throw new TIRExceptCreate("This tag exists");
      et = idl.tag();
      for (i=0;i<et.length;i++) {
        if (t.compareTo(et[i])==0)
          throw new TIRExceptCreate("This tag exists");       
      } 
    }
    ((SIdentification) id).tag(t);
  }

  private void fromNormObj() throws RemoteException, TIRExceptLock {
    if (normal==null)
      return;
    if (((WorkRepositoryImpl)rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    akt = normal.firstChild;
    for(i=0;i<normal.numOfItems;i++) {
      addListItem(WorkRepositoryImpl.newWorkFromNormal(akt.obj,rep,this,workId));
      akt = akt.next;
    }
    hasCont = true;
  }

  public StructMember[] members() throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
    } else {
      if (!hasCont) 
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
    }
    StructMember[] all = new StructMember [numOfItems];
    LiItem akt = firstChild;
    int i,j = 0;
    for(i=0;i<numOfItems;i++) {
      if (akt.obj.get_def_kind().value()==DefinitionKind.dk_StructMember) {
        all[j] = (StructMember) akt.obj;
	j++;
      }
      akt = akt.next;
    }
    StructMember[] ret = new StructMember [j];
    for(i=0;i<j;i++) {
      ret[i] = all[i];
    }
    return ret;   
  }

  public Contained[] contents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);     
    }
  }    

  public Contained lookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    }
  }

  public Contained lookup(String name, String ver) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, ver);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, ver);
    }
  }

  public Contained[] lookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    }
  }

  public Contained[] lookup_name(String name) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(name);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(name);
    }
  }

  public Contained lookup_tag(String name, String tag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    }
  }

  public Contained[] lookup_branchtag(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    }
  }

  public Contained lookup_lastinbranch(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    }
  }
  
  public Contained lookup_lastfromversion(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    }
  }

  public Contained[] scontents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.scontents(type);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.scontents(type);
    }
  }

  public Contained slookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(id);
    }
  }

  public Contained slookup(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(name, version);
    }
  }

  public Contained[] slookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(id);
    }
  }

  public Contained[] slookup_name(String name) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.slookup_name(name);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(name);
    }
  }

  public StructMember create_member(String name, CDLType type) throws RemoteException, TIRExceptCreate, TIRExceptLock  {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (name!=null && name.compareTo("")!=0 && type!=null) {
      if (type.get_state().value()==StateKind.sk_work ) {
	StructMember[] a = members();
	boolean found = false;
	for (int i=0;i<a.length;i++) {
          if (a[i].name().compareTo(name)==0) {
	    found = true;
	    break;
	  }
	}
        if (found)
	  throw new TIRExceptCreate("Member with this name exists in exception.");
	StructMember m = new StructMemberImpl(new IdentificationImpl(((SIdentification)this.id).language(), this.id.absolute_name().name()+"::"+name,this.id.version()), this, rep, type);
	addListItem(m);
	return m;
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    } else
      return null;
  }

  public StructDef create_struct(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (id!=null) {
      if (!hasCont)
        fromNormObj();
      // set version of substructure same as this version  
      id = new IdentificationImpl(((SIdentification)id).language(), id.absolute_name().name(),this.id.version());
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Struct)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      StructDef ret = new StructDefImpl(id, this, rep,workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }
  
  public UnionDef create_union(Identification id, CDLType switch_type) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (id==null || switch_type==null)
      return null;
    else {
      if (switch_type.get_state().value()==StateKind.sk_work ) { 
        if (!hasCont)
          fromNormObj();
        // set version of substructure same as this version  
        id = new IdentificationImpl(((SIdentification)id).language(), id.absolute_name().name(),this.id.version());
        Contained[] a = lookup_name(id.name());
        if (a.length != 0) {
          if (a[0].get_def_kind().value() != DefinitionKind.dk_Union)
	    throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	  boolean found = false;  
	  for (int i=0;i<a.length;i++) {
	    if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	      found = true;
	      break;
	    }
	  }
	  if (found) 
	    throw new TIRExceptCreate("Object with same identification exists in repository.");
        }
        // creating new object
        UnionDef ret = new UnionDefImpl(id, this, rep, switch_type,workId);
        addListItem(ret); // adding object to list
        return ret;       // return object
      } else {
        throw new TIRExceptCreate("Given object isn't work.");
      }
    }
  }
  
  public EnumDef create_enum(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (id!=null) {
      if (!hasCont)
        fromNormObj();
      // set version of substructure same as this version  
      id = new IdentificationImpl(((SIdentification)id).language(), id.absolute_name().name(),this.id.version());
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Enum)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      EnumDef ret = new EnumDefImpl(id, this, rep, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }

  /*from interface TIRImplObject */
  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      boolean closeFile = false;
      if (st.curOutFile==null) {  // saving in module
        st.curOutFile = new DataOutputStream(new FileOutputStream(st.currentFile));
	closeFile = true;
      } else {   // saving in other structure (or union)
        ;
      }
      ((IdentificationImpl)id).save(st.curOutFile);   // saving identification of struct
      st.curOutFile.writeInt(numOfItems);
      LiItem akt = firstChild;
      for(int i=0;i<numOfItems;i++) {  // saving all childs
        st.curOutFile.writeInt(akt.obj.get_def_kind().value());
        ((TIRImplObject)(akt.obj)).save(st);
	akt = akt.next;
      }
      if (closeFile)  { // close file 
        st.curOutFile.close();
	st.curOutFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  }

  /*from interface TIRImplObject */
  public void load(Storage st) throws RemoteException, TIRExceptStorage{
    try {
      boolean closeFile = false;
      if (st.curInFile==null) {
        st.curInFile = new DataInputStream(new FileInputStream(st.currentFile));
        closeFile = true;
      } else {
        ;
      }
      id = new IdentificationImpl();
      ((IdentificationImpl)id).load(st.curInFile);
      int num = st.curInFile.readInt();
      int dkind;
      TIRImplObject obj;
      for (int i=0;i<num;i++) {
        dkind = st.curInFile.readInt();
	switch (dkind) {
	  case DefinitionKind.dk_StructMember:
	    obj = new StructMemberImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
          case DefinitionKind.dk_Struct:
	    obj = new StructDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
	  case DefinitionKind.dk_Enum:
	    obj = new EnumDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
	  case DefinitionKind.dk_Union:
	    obj = new UnionDefImpl(this, rep);
	    obj.load(st);
	    addListItem((TIRObject)obj);
	    break;
	  default:
            throw new TIRExceptStorage("Unexpected kind of object in \""
	                              +st.currentFile+"\".");
	} // -- end of switch
      }
      if (closeFile)  { // close file 
        st.curInFile.close();
	st.curInFile = null;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      ((TIRImplObject) akt.obj).postLoad(r);
      akt = akt.next;
    }
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  } 

  public void canCommit() throws RemoteException, TIRExceptCommit {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    LiItem akt;
    int i;
    akt = firstChild;
    for (i=0;i<numOfItems;i++) {
      ((TIRImplObject)akt.obj).canCommit();
      akt = akt.next;
    }
  }

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      LiItem akt = firstChild;
      for (int i=0;i<numOfItems;i++) {
        ((TIRImplObject)akt.obj).doCommit(this,rep);
        akt = akt.next;     
      }
      parent = in;
      this.rep = rep;
    } else {
      normal = null;
    }
  }

  public void doAbort(long workId) throws RemoteException {
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      if (((TIRImplObject)akt.obj).isNew()) 
        ((TIRImplObject)akt.obj).doAbort(workId);
      akt = akt.next;
    }  
  }
}
