/* $Id: ManageProfilesImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ManageProfiles;
import SOFA.SOFAnode.Made.TIR.ProfileDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;

public class ManageProfilesImpl extends UnicastRemoteObject implements ManageProfiles, TIRImplObject {
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  LiItem firstProf;
  LiItem lastProf;
  int numOfProfs;
  Repository rep;
  boolean hasCont;
  ManageProfilesImpl normal;

  public ManageProfilesImpl(Repository rep) throws RemoteException {
    firstProf = lastProf = null;
    numOfProfs = 0;
    this.rep = rep;
    hasCont = true;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ManageProfiles);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    normal = null;
  }

  public ManageProfilesImpl(ManageProfilesImpl a, Repository rep) throws RemoteException {
    firstProf = lastProf = null;
    numOfProfs = 0;
    this.rep = rep;
    hasCont = false;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ManageProfiles);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    normal = a;
  }

  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }
  
  public ProfileDef[] all_profiles() throws RemoteException, TIRExceptLock {
    int i;
    if (stKindImpl.value() == StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      ProfileDef[] ret = new ProfileDef [numOfProfs];
      LiItem akt = firstProf;
      for(i=0;i<numOfProfs;i++) {
        ret[i] = (ProfileDef) akt.obj;
        akt = akt.next;
      }
      return ret;
    } else {
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      if (!hasCont) {
        if (normal!=null) {
          LiItem akt = normal.firstProf;
          for(i=0;i<normal.numOfProfs;i++) {
            LiItem item = new LiItem(new ProfileDefImpl((ProfileDefImpl) akt.obj, rep));
            if (firstProf == null) 
              firstProf = lastProf = item;
            else {
              lastProf.next = item;
              item.prev = lastProf;
              lastProf = item;
            }
            numOfProfs++;
            akt = akt.next;
          }
        }
        hasCont=true;
      }
      ProfileDef[] ret = new ProfileDef [numOfProfs];
      LiItem akt = firstProf;
      for(i=0;i<numOfProfs;i++) {
        ret[i] = (ProfileDef) akt.obj;
        akt = akt.next;
      }
      return ret;
    }
  }
  
  public ProfileDef lookup(String name) throws RemoteException, TIRExceptLock {
    int i;
    if (stKindImpl.value() == StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      LiItem akt = firstProf;
      for(i=0;i<numOfProfs;i++) {
        if ((name.compareTo(((ProfileDef) akt.obj).name()))==0)
          return (ProfileDef) akt.obj;
        akt = akt.next;
      }
      return null;
    } else {
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      if (!hasCont) {
        if (normal!=null) {
          LiItem akt = normal.firstProf;
          for(i=0;i<normal.numOfProfs;i++) {
            LiItem item = new LiItem(new ProfileDefImpl((ProfileDefImpl) akt.obj, rep));
            if (firstProf == null) 
              firstProf = lastProf = item;
            else {
              lastProf.next = item;
              item.prev = lastProf;
              lastProf = item;
            }
            numOfProfs++;
            akt = akt.next;
          }
        }
        hasCont = true;
      }
      LiItem akt = firstProf;
      for(i=0;i<numOfProfs;i++) {
        if ((name.compareTo(((ProfileDef) akt.obj).name()))==0)
          return (ProfileDef) akt.obj;
        akt = akt.next;
      }
      return null;
    }
  }
  
  synchronized public ProfileDef create_profile(String name) throws RemoteException, TIRExceptLock, TIRExceptCreate {
    if (stKindImpl.value() == StateKind.sk_normal)
      throw new TIRExceptCreate("You can call create only on work object");

    if (((WorkRepositoryImpl) rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    if (!hasCont) {
      if (normal!=null) {
        LiItem akt = normal.firstProf;
        for(int i=0;i<normal.numOfProfs;i++) {
          LiItem item = new LiItem(new ProfileDefImpl((ProfileDefImpl) akt.obj, rep));
          if (firstProf == null) 
            firstProf = lastProf = item;
          else {
            lastProf.next = item;
            item.prev = lastProf;
            lastProf = item;
          }
          numOfProfs++;
          akt = akt.next;
        }
      }
      hasCont = true;
    }

    LiItem akt = firstProf;
    for(int i=0;i<numOfProfs;i++) {
      if ((name.compareTo(((ProfileDef) akt.obj).name()))==0)
        throw new TIRExceptCreate("Profile with this name exists");
      akt = akt.next;
    }
    
    ProfileDef ret = new ProfileDefImpl(new String(name), rep, StateKind.sk_work);
    LiItem item = new LiItem(ret);
    if (firstProf == null) {
       firstProf = lastProf = item;
    } else {
      lastProf.next = item;
      item.prev = lastProf;
      lastProf = item;
    }
    numOfProfs++;
    return ret;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      String[][] childs = null;
      try {
        childs = st.listOfChilds();
      } catch (TIRExceptStorage e) { // new dir without index.tir
        ;
      }
      File f = new File(st.current,Storage.indexFile);  // name of index file
      // ((IdentificationImpl)id).save(f);   //saving identification
      LiItem akt = firstProf;
      File old = st.current;
      BufferedWriter index = new BufferedWriter(new FileWriter(f.toString())); 
      index.write("CHILDS",0,6); index.newLine();
      for(int i=0;i<numOfProfs;i++) {
        String[] achild;    // actual child info
        IdentificationImpl idl;
	File nfile;               // path to child's current path
        if ((achild=st.existsChild(childs,idl = new IdentificationImpl("","::"+((ProfileDef)akt.obj).name(),"")))!=null) { // file (dir) exists
	  // saving info to index file
          index.write(achild[0],0,achild[0].length()); index.write("?",0,1);
          index.write(achild[1],0,achild[1].length()); index.write("?",0,1);
          index.write(achild[2],0,achild[2].length()); index.write("?",0,1);
          index.write(achild[3],0,achild[3].length()); index.newLine();
	  nfile = new File(old,achild[0]);
	  
        } else {  //file (dir) doesn't exist
	  String hlp;
          nfile = File.createTempFile("prf","",old);
          nfile.delete();
	  // saving info to index file
	  index.write(hlp = nfile.getName(),0,hlp.length()); index.write("?",0,1);
	  index.write(hlp=Integer.toString(akt.obj.get_def_kind().value()),0,hlp.length());
	  index.write("?",0,1);
	  index.write(hlp = ((ProfileDef)akt.obj).name(),0,hlp.length()); index.write("?",0,1);
          index.write("",0,0); index.newLine();
        }
	st.currentFile = nfile;
	st.curOutFile = null;
        
	((TIRImplObject) akt.obj).save(st);   // save akt object
        akt = akt.next;                       // go to the next object
      } // -- end of for
      index.close();
      st.current = old;
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage{
    String[][] childs = st.listOfChilds();
    File old = st.current;
    TIRImplObject m;
    for(int i=0;i<childs.length;i++) {
       m = new ProfileDefImpl(rep);
       st.currentFile = new File(old,childs[i][0]);
       m.load(st);
       LiItem item = new LiItem((TIRObject) m);
       if (firstProf == null) 
         firstProf = lastProf = item;
       else {
         lastProf.next = item;
         item.prev = lastProf;
         lastProf = item;
       }
       numOfProfs++;
       st.currentFile = null;
    }
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {
    LiItem akt = firstProf;
    for (int i=0;i<numOfProfs; i++) {
      if (((TIRImplObject) akt.obj).isNew()) {
        LiItem nak = normal.firstProf;
        for (int j=0;j<normal.numOfProfs;j++) {
          if (((ProfileDef) akt.obj).name().compareTo(((ProfileDef) nak.obj).name())==0)
            throw new TIRExceptCommit("Profile with name " + ((ProfileDef) akt.obj).name() + " exists");
          nak = nak.next;
        }
      }
      ((TIRImplObject) akt.obj).canCommit();
      akt = akt.next;
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {}
  public boolean isNew() { return false; }

  public void doCommit(Container in, Repository rep) throws RemoteException {
    LiItem akt = firstProf;
    for (int i=0;i<numOfProfs;i++) {
      if (((TIRImplObject)akt.obj).isNew()) {
        ((TIRImplObject) akt.obj).doCommit(in, rep);
        LiItem item = new LiItem(akt.obj);
        if (normal.numOfProfs==0) {
          normal.firstProf = normal.lastProf = item;
        } else {
          normal.lastProf.next = item;
          item.prev = normal.lastProf;
          normal.lastProf = item;
        }
        normal.numOfProfs++;
      } else {  // profile exists
        if (((ProfileDefImpl) akt.obj).changed) {
          ProfileDefImpl work = (ProfileDefImpl) akt.obj;
          ProfileDefImpl norm = searchNormProfile(((ProfileDefImpl)akt.obj).name());
          mergeProfiles(norm,work);
        }
        if (((ProfileDefImpl) akt.obj).isLocked()) {
          if (((ProfileDefImpl) akt.obj).lockForMe) {
            ((ProfileDefImpl) akt.obj).unlock();
          }
        }
      }

      akt = akt.next;
    }
  }

  private ProfileDefImpl searchNormProfile(String name) throws RemoteException {
    LiItem akt = normal.firstProf;
    for (int i=0;i<normal.numOfProfs;i++) {
      if (name.compareTo(((ProfileDef)akt.obj).name())==0)
        return (ProfileDefImpl) akt.obj;
      akt = akt.next;
    }
    return null;
  }

  private void mergeProfiles(ProfileDefImpl normal, ProfileDefImpl work) throws RemoteException {
    normal.firstId = normal.lastId = null;
    normal.numOfIds = 0;
    LiIdItem akt = work.firstId;
    for (int i=0;i<work.numOfIds;i++) {
      LiIdItem item = new LiIdItem(akt.obj);
      if (normal.firstId == null) {
        normal.firstId = normal.lastId = item;
      } else {
        normal.lastId.next = item;
        item.prev = normal.lastId;
        normal.lastId = item;
      }
      normal.numOfIds++;
      akt = akt.next;
    }
  }
  
  public void doAbort(long workId) throws RemoteException {
    LiItem akt = firstProf;
    for (int i=0;i<numOfProfs;i++) {
      if (!((TIRImplObject)akt.obj).isNew()) {
        if (((ProfileDefImpl) akt.obj).isLocked()) {
          if (((ProfileDefImpl) akt.obj).lockForMe) {
            ((ProfileDefImpl) akt.obj).unlock();
          }
        }
      }
      akt = akt.next;
    }
  }
}
