/* $Id: OperationDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.ArrayDef;
import SOFA.SOFAnode.Made.TIR.CDLType;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.ExceptionDef;
import SOFA.SOFAnode.Made.TIR.FixedDef;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.OperationDef;
import SOFA.SOFAnode.Made.TIR.ParamDescr;
import SOFA.SOFAnode.Made.TIR.PrimitiveDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.SequenceDef;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.StringDef;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;
import SOFA.SOFAnode.Made.TIR.WstringDef;

public class OperationDefImpl extends ContainedImpl implements OperationDef {
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  CDLType res;
  int numOfParams;
  LiItem firstPar;
  LiItem lastPar;
  ExceptionDef[] excepts;
  CDLType[] loadEx;

  OperationDefImpl normal;
  boolean hasCont;
  long workId;

  public OperationDefImpl(Identification ident, Container in, Repository inrep, CDLType r, ExceptionDef[] excps, long workId) throws RemoteException {
    super(ident,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Operation);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    res = r;
    if (excps==null)
      excepts = new ExceptionDef [0];
    else   
      excepts = excps;
    numOfParams = 0;
    firstPar = lastPar = null;
    normal = null;
    hasCont = false;
    this.workId = workId;
  }

  public OperationDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Operation);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    res = null;
    excepts = null;
    numOfParams = 0;
    firstPar = lastPar = null;
    normal = null;
    hasCont = false;
    this.workId = -1L;
  }

  public OperationDefImpl(OperationDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    super(a.id,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Operation);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    res = a.res;
    excepts = a.excepts;
    numOfParams = 0;
    firstPar = lastPar = null;
    normal = a;
    hasCont = false;
    this.workId = workId;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    super.tag(t);
  }

  public CDLType result() throws RemoteException {
    return res;
  }

  private void fromNormObj() throws RemoteException, TIRExceptLock {
    if (normal==null)
      return;
    if (((WorkRepositoryImpl)rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    akt = normal.firstPar;
    for(i=0;i<normal.numOfParams;i++) {
      addListItem(new ParamDescrImpl((ParamDescrImpl)akt.obj));
      akt = akt.next;
    }
    hasCont = true;
  }

  private void addListItem(TIRObject what) {
    LiItem item = new LiItem(what);
    if (firstPar == null) {
      firstPar = lastPar = item;
    } else {
      LiItem akt = firstPar;
      while (akt.next != null) akt = akt.next;
      akt.next = item;
      item.prev = akt;
      lastPar = item;
    }
    numOfParams++;
  }

  public ParamDescr[] params() throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal && !hasCont)
      fromNormObj();
    ParamDescr[] ret = new ParamDescr [numOfParams];
    LiItem akt = firstPar;
    for(int i=0;i<numOfParams;i++) {
      ret[i] = (ParamDescr) akt.obj;
      akt = akt.next;
    }
    return ret;
  }

  public ExceptionDef[] exceptions() throws RemoteException {
    return excepts;
  }

  public ParamDescr create_parametr(String name, CDLType type, int paramMode) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (!isNew())
      throw new TIRExceptCreate("you can call create method on work object");
    if (type==null || name==null || name.compareTo("")==0) 
      return null;
    if (type.get_state().value()==StateKind.sk_work ) {
      ParamDescr[] a = params();
      boolean found = false;
      for (int i=0;i<a.length;i++) {
        if (a[i].name().compareTo(name)==0) {
          found = true;
          break;
        }
      }
      if (found)
        throw new TIRExceptCreate("Parametr with this name exists in operation.");
      ParamDescr ret = new ParamDescrImpl(new String(name), type, new ParamModeImpl(paramMode));
      addListItem(ret);
      return ret;
    } else 
      throw new TIRExceptCreate("Given object isn't work.");
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i;
      ((IdentificationImpl) id).save(st.curOutFile);
      Storage.writeCDLType(st, res);
      st.curOutFile.writeInt(numOfParams);
      LiItem akt = firstPar;
      for(i=0;i<numOfParams;i++) {
        ((TIRImplObject)(akt.obj)).save(st);
	akt = akt.next;
      }
      st.curOutFile.writeInt(excepts.length);
      for(i=0;i<excepts.length;i++) {
        Storage.writeCDLType(st, excepts[i]);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

   public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i, num;
      id = new IdentificationImpl();
      ((IdentificationImpl) id).load(st.curInFile);
      res = Storage.readCDLType(st);
      num = st.curInFile.readInt();
      ParamDescrImpl par;
      for(i=0;i<num;i++) {
        par = new ParamDescrImpl();
        par.load(st);
	addListItem(par);
      }
      num = st.curInFile.readInt();
      loadEx = new CDLType [num];
      for(i=0;i<num;i++) {
        loadEx[i] = Storage.readCDLType(st);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  } 

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
    int i,k;
    Container con;
    Contained[] rt;
    Contained ret;
    IdentificationImpl id;
    AbsoluteName nm;
    boolean found;
    if (res.get_def_kind().value() == DefinitionKind.dk_none) {   // is tp NotYetInRep?
      if (((NotYetInRep) res).defKind == DefinitionKind.dk_Primitive)
        res = r.get_prim(((NotYetInRep) res).primKind);
      else {     // in tp there is reference to some type
        nm = ((NotYetInRep) res).nm;
	con = r;  // begin search from repository
        for (i=0;i<nm.size()-1;i++) {  // last element is name of object
	  id = new IdentificationImpl(nm.elementAt(0), "::"+nm.elementAt(i),"");
          rt = ((SContainer)con).slookup_name(id);
	  if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
          if (rt.length==1) {
            con = (Container) rt[0];
	  } else {  // type in struct (etc.) - more versions of this struct
	    found = false;
            id.version(((NotYetInRep) res).id.version());  // substruct version is inherited from parent
	    for (k=0;k<rt.length;k++) {
	      if (id.is_short_equal(rt[k].get_identification())) {
	        found = true;
		break;  // -- break for
	      }
	    } // -- end of for
	    if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
	    con = (Container) rt[k];
	  }
	}
	if ((ret = ((SContainer)con).slookup(((NotYetInRep) res).id))==null)
	  throw new TIRExceptStorage("Can't find type " + nm.name());
	res = (CDLType) ret;
	if (res.get_def_kind().value()==DefinitionKind.dk_Array ||
            res.get_def_kind().value()==DefinitionKind.dk_Sequence ||
            res.get_def_kind().value()==DefinitionKind.dk_String ||
            res.get_def_kind().value()==DefinitionKind.dk_Wstring)
	  ((TIRImplObject) res).postLoad(r);
      }
    } else {
      if (res.get_def_kind().value()==DefinitionKind.dk_Array ||
          res.get_def_kind().value()==DefinitionKind.dk_Sequence ||
          res.get_def_kind().value()==DefinitionKind.dk_String ||
          res.get_def_kind().value()==DefinitionKind.dk_Wstring)
        ((TIRImplObject) res).postLoad(r);
    }
    // postLoad excepts
    excepts = new ExceptionDef [loadEx.length];
    for (int j=0;j<loadEx.length;j++) {
      if (loadEx[j].get_def_kind().value() == DefinitionKind.dk_none) {   // is tp NotYetInRep?
        nm = ((NotYetInRep) loadEx[j]).nm;
  	con = r;  // begin search from repository
        for (i=0;i<nm.size()-1;i++) {  // last element is name of object
  	  id = new IdentificationImpl(nm.elementAt(0), "::"+nm.elementAt(i),"");
          rt = ((SContainer)con).slookup_name(id);
  	  if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
            if (rt.length==1) {
              con = (Container) rt[0];
  	  } else {  // type in struct (etc.) - more versions of this struct
  	    found = false;
            id.version(((NotYetInRep) loadEx[j]).id.version());  // substruct version is inherited from parent
  	    for (k=0;k<rt.length;k++) {
  	      if (id.is_short_equal(rt[k].get_identification())) {
  	        found = true;
  		break;  // -- break for
  	      }
  	    } // -- end of for
  	    if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
  	    con = (Container) rt[k];
  	  }
  	}
  	if ((ret = ((SContainer)con).slookup(((NotYetInRep) loadEx[j]).id))==null)
  	  throw new TIRExceptStorage("Can't find type " + nm.name());
  	excepts[j] = (ExceptionDef) ret;
	if (excepts[j].get_def_kind().value()==DefinitionKind.dk_Array ||
            excepts[j].get_def_kind().value()==DefinitionKind.dk_Sequence)
	  ((TIRImplObject) excepts[j]).postLoad(r);
      }
    }

    LiItem akt = firstPar;
    for(i=0;i<numOfParams;i++) {
      ((TIRImplObject) akt.obj).postLoad(r);
      akt = akt.next;
    }

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      switch (res.get_def_kind().value()) {
        case DefinitionKind.dk_Primitive:
          res = ((SRepository)rep).get_prim(((PrimitiveDef)res).kind().value());
          break;
        case DefinitionKind.dk_Array:
          ArrayDef nt = new ArrayDefImpl(ExprHelp.toNormal(((ArrayDef) res).length()), ((ArrayDef) res).element_type());
          res = nt;
          ((TIRImplObject) res).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Sequence:
          SequenceDef st = new SequenceDefImpl(ExprHelp.toNormal(((SequenceDef) res).bound()), ((SequenceDef) res).element_type());
          res = st;
          ((TIRImplObject)res).doCommit(in,rep);
          break;
        case DefinitionKind.dk_String:
          StringDef str = new StringDefImpl(ExprHelp.toNormal(((StringDef) res).bound()));
          res = str;
          ((TIRImplObject)res).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Wstring:
          WstringDef wstr = new WstringDefImpl(ExprHelp.toNormal(((WstringDef) res).bound()));
          res = wstr;
          ((TIRImplObject) res).doCommit(in,rep);
          break;
        case DefinitionKind.dk_Fixed:
          FixedDef ft = new FixedDefImpl(ExprHelp.toNormal(((FixedDef) res).digits()), ExprHelp.toNormal(((FixedDef) res).scale()));
          res = ft;
          ((TIRImplObject)res).doCommit(in,rep);
          break;
        default:
          Identification idl = ((Contained)res).get_identification();
          res = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
          break;
      }
      LiItem akt = firstPar;
      int i;
      for (i=0;i<numOfParams;i++) {
        ((TIRImplObject)akt.obj).doCommit(in,rep);
        akt = akt.next;     
      }
      loadEx = new CDLType [excepts.length];
      for (i=0;i<excepts.length;i++) {
        Identification idl = ((Contained)excepts[i]).get_identification();
        loadEx[i] = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
      }
      excepts = null;
      parent = in;
      this.rep = rep;
    } else {
      normal = null;
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
