/* $Id: ProviderDefImpl.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.rmi.RemoteException;

import SOFA.SOFAnode.Made.TIR.AModuleDef;
import SOFA.SOFAnode.Made.TIR.AbsoluteName;
import SOFA.SOFAnode.Made.TIR.ArchitectureDef;
import SOFA.SOFAnode.Made.TIR.Contained;
import SOFA.SOFAnode.Made.TIR.Container;
import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import SOFA.SOFAnode.Made.TIR.FrameDef;
import SOFA.SOFAnode.Made.TIR.Identification;
import SOFA.SOFAnode.Made.TIR.ProviderDef;
import SOFA.SOFAnode.Made.TIR.Repository;
import SOFA.SOFAnode.Made.TIR.StateKind;
import SOFA.SOFAnode.Made.TIR.TIRExceptCommit;
import SOFA.SOFAnode.Made.TIR.TIRExceptCreate;
import SOFA.SOFAnode.Made.TIR.TIRExceptLock;
import SOFA.SOFAnode.Made.TIR.TIRObject;

public class ProviderDefImpl extends ContainerImpl implements ProviderDef, SContained {
  protected Identification id;
  protected Container parent;
  protected Repository rep;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  ProviderDefImpl normal;

  Lock lockCreate;
  boolean lockForMe;   // used in work object
  public long lockFor; // used in normal object
  public long workId;  // id of "transaction"

  public ProviderDefImpl(Identification ident, Container in, Repository inrep, int state, long workId) throws RemoteException {
    id = ident;
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provider);
    stKindImpl = new StateKindImpl(state);
    normal = null;
    lockCreate = new Lock(false);
    lockForMe = false;
    lockFor = -1L;
    this.workId = workId;
  }

  public ProviderDefImpl(Container in, Repository inrep) throws RemoteException {
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provider);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    normal = null;
    lockCreate = new Lock(false);
    lockForMe = false;
    workId = -1L;
    lockFor = -1L;
  }

  public ProviderDefImpl(ProviderDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    parent = in;
    rep = inrep;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Provider);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    id = a.id;
    normal = a;
    lockCreate = a.lockCreate;
    lockForMe = false;
    this.workId = workId;
    lockFor = -1L;
  }

  /* form interface Contained */
  public Identification get_identification() throws RemoteException {
    return id;
  }

  /* form interface Contained */
  public Container get_defined_in() throws RemoteException {
    return parent;
  }

  /* form interface Contained */
  public Container get_containing_repository() throws RemoteException {
    return rep;
  }

  /* form interface Contained */
  public AbsoluteName get_absolute_name() throws RemoteException {
    AbsoluteNameImpl absName;
    if (rep == parent) {  // object is contained in repository
      absName = new AbsoluteNameImpl("::"+id.name());
    } else {
      absName = new AbsoluteNameImpl(((SContained)parent).get_absolute_name().name()+"::"+id.name());
    }
    return (AbsoluteName) absName;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    String ebt = id.branchTag();
    if (t.compareTo(ebt)==0)
      throw new TIRExceptCreate("This tag exists");
    String[] et = id.tag();
    int i,j;
    for (i=0;i<et.length;i++) {
      if (t.compareTo(et[i])==0)
       throw new TIRExceptCreate("This tag exists");       
    }
    Contained[] sibl = parent.lookup_name(id);
    for (j=0;j<sibl.length;j++) {
      Identification idl = sibl[j].get_identification();
      
      ebt = idl.branchTag();
      if (t.compareTo(ebt)==0)
        throw new TIRExceptCreate("This tag exists");
      et = idl.tag();
      for (i=0;i<et.length;i++) {
        if (t.compareTo(et[i])==0)
          throw new TIRExceptCreate("This tag exists");       
      } 
    }
    ((SIdentification) id).tag(t);
  }

  private void fromNormObj() throws RemoteException, TIRExceptLock {
    if (normal==null)
      return;
    if (((WorkRepositoryImpl)rep).lock.isLocked())
      throw new TIRExceptLock("Repository is locked");
    int i;
    LiItem akt;
    akt = normal.firstChild;
    for(i=0;i<normal.numOfItems;i++) {
      addListItem(WorkRepositoryImpl.newWorkFromNormal(akt.obj,rep,this,workId));
      akt = akt.next;
    }
    hasCont = true;
  }

  public Contained[] contents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);
    } else {
      if (!hasCont)
        fromNormObj();
       if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.contents(type);     
    }
  }    

  public Contained lookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(id);
    }
  }

  public Contained lookup(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup(name, version);
    }
  }

  public Contained[] lookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(id);
    }
  }

  public Contained[] lookup_name(String name) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(name);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_name(name);
    }
  }

  public Contained lookup_tag(String name, String tag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_tag(name, tag);
    }
  }

  public Contained[] lookup_branchtag(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_branchtag(name, brtag);
    }
  }

   public Contained lookup_lastinbranch(String name, String brtag) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastinbranch(name, brtag);
    }
  }
  
  public Contained lookup_lastfromversion(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      if (((RepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      if (((WorkRepositoryImpl) rep).lock.isLocked())
        throw new TIRExceptLock("Repository is locked");
      else 
        return super.lookup_lastfromversion(name, version);
    }
  } 

  public Contained[] scontents(DefinitionKind type) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.scontents(type);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.scontents(type);
    }
  }

  public Contained slookup(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(id);
    }
  }

  public Contained slookup(String name, String version) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup(name, version);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup(name, version);
    }
  }

  public Contained[] slookup_name(Identification id) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
        return super.slookup_name(id);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(id);
    }
  }

  public Contained[] slookup_name(String name) throws RemoteException, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal) {
      return super.slookup_name(name);
    } else {
      if (!hasCont)
        fromNormObj();
      return super.slookup_name(name);
    }
  }

  /*from interface TIRImplObject */
  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      String[][] childs = null;
      try {
        childs = st.listOfChilds();
      } catch (TIRExceptStorage e) { // new dir without index.tir
        ;
      }
      File f = new File(st.current,Storage.indexFile);  // name of index file
      // ((IdentificationImpl)id).save(f);   //saving identification
      LiItem akt = firstChild;
      File old = st.current;
      BufferedWriter index = new BufferedWriter(new FileWriter(f.toString()));
      index.write("CHILDS",0,6); index.newLine();
      for(int i=0;i<numOfItems;i++) {
        String[] achild;    // actual child info
        IdentificationImpl idl;   // actual child id
	File nfile;               // path to child's current path
        if ((achild=st.existsChild(childs,idl = (IdentificationImpl)((Contained)akt.obj).get_identification()))!=null) { // file (dir) exists
	  // saving info to index file
          index.write(achild[0],0,achild[0].length()); index.write("?",0,1);
          index.write(achild[1],0,achild[1].length()); index.write("?",0,1);
          index.write(achild[2],0,achild[2].length()); index.write("?",0,1);
          index.write(achild[3],0,achild[3].length()); index.newLine();
	  nfile = new File(old,achild[0]);
	  
        } else {  //file (dir) doesn't exist
	  String hlp;
	  //if (idl.name().length() <= Storage.maxLenFileName)
	  //  nfile = new File(old,idl.name());    // file (dir) name same as id
	  //else   
            nfile = File.createTempFile("tir","",old);
          nfile.delete();
	  // saving info to index file
	  index.write(hlp = nfile.getName(),0,hlp.length()); index.write("?",0,1);
	  index.write(hlp=Integer.toString(akt.obj.get_def_kind().value()),0,hlp.length());
	  index.write("?",0,1);
	  index.write(hlp = idl.name(),0,hlp.length()); index.write("?",0,1);
          index.write(hlp = idl.version(),0,hlp.length()); index.newLine();
        }
        if (akt.obj.get_def_kind().value()==DefinitionKind.dk_AModule) {
	  st.current = nfile;   // directory for module
	  if (!st.current.exists())  //create only if doesn't exist
            if (!st.current.mkdir())
	      throw new TIRExceptStorage("Can't create directory "+st.current+".");
	} else {
	  st.currentFile = nfile;  // file for all except Amodule
          st.curOutFile = null;
        }
        
	((TIRImplObject) akt.obj).save(st);   // save akt object
        akt = akt.next;                       // go to the next object
        st.current = old;
        st.currentFile = null;
      } // -- end of for
      index.close();
      st.current = old;
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in "+st.current+".");
    }
  } // -- end of save()

  /*from interface TIRImplObject */
  public void load(Storage st) throws RemoteException, TIRExceptStorage{
    String[][] childs = st.listOfChilds();
    File old = st.current;
    TIRImplObject m;
    IdentificationImpl idl;
    for(int i=0;i<childs.length;i++) {
      switch (Integer.parseInt(childs[i][1])) {
        case DefinitionKind.dk_AModule: 
	  idl = new IdentificationImpl(((SIdentification) this.id).language(), this.id.absolute_name().name()+"::"+childs[i][2],"");
	  m = new AModuleDefImpl(idl,this, rep,StateKind.sk_normal,-1L);
      	  st.current = new File(old,childs[i][0]);
      	  m.load(st);
	  addListItem((TIRObject) m);
	  st.current = old;
	  break;
	case DefinitionKind.dk_Architecture: 
	  st.currentFile = new File(old,childs[i][0]);
	  st.curInFile = null;
	  m = new ArchitectureDefImpl(this, rep);
      	  m.load(st);
	  addListItem((TIRObject) m);
	  st.currentFile = null;
	  break;  
	default: 
	  throw new TIRExceptStorage("Unexpected kind of object in \""
	                              +st.current+File.separator+childs[i]+"\".");
      }
    }
  }
  
  public AModuleDef create_amodule(Identification id) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (id!=null) {
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormObj();
      id = new IdentificationImpl(((SIdentification)id).language(), id.absolute_name().name(),"");
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_AModule)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      AModuleDef ret = new AModuleDefImpl(id, this, rep, StateKind.sk_work, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }
  
  public ArchitectureDef create_architecture(Identification id, FrameDef frame, boolean system) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()==StateKind.sk_normal)
      throw new TIRExceptCreate("you can call create method on work object");
    if (id!=null && frame!=null) {
      if (!getCreateLock())
        throw new TIRExceptLock("This part of repository is locked for creating.");
      if (!hasCont)
        fromNormObj();
      id = new IdentificationImpl(id);  
      Contained[] a = lookup_name(id.name());
      if (a.length != 0) {
        if (a[0].get_def_kind().value() != DefinitionKind.dk_Architecture)
	  throw new TIRExceptCreate("Name \""+id.name()+"\" exists in repository.");
	boolean found = false;  
	for (int i=0;i<a.length;i++) {
	  if (((SIdentification) a[i].get_identification()).is_short_equal(id)) {
	    found = true;
	    break;
	  }
	}
	if (found) 
	  throw new TIRExceptCreate("Object with same identification exists in repository.");
      }
      // creating new object
      ArchitectureDef ret = new ArchitectureDefImpl(id, this, rep, frame, StateKind.sk_work, system, workId);
      addListItem(ret); // adding object to list
      return ret;       // return object
    } else
      return null;
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      ((TIRImplObject) akt.obj).postLoad(r);
      akt = akt.next;
    }
  }

 public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    LiItem akt;
    int i,j;
    akt = firstChild;
    for (i=0;i<numOfItems;i++) {
      if (((TIRImplObject)akt.obj).isNew() && !isNew()) {
        LiItem nak = normal.firstChild;
        for (j=0;j<normal.numOfItems;j++) {
          if (((SIdentification) ((Contained)nak.obj).get_identification()).is_short_name_equal(((Contained)akt.obj).get_identification())) {
            if (akt.obj.get_def_kind().value()!=nak.obj.get_def_kind().value())
              throw new TIRExceptCommit("Object with name \""+((SContained)nak.obj).get_absolute_name().name()+"\"exists.");
            if (((SIdentification) ((Contained)nak.obj).get_identification()).is_short_equal(((Contained)akt.obj).get_identification()))  
              throw new TIRExceptCommit("Object with identification \""+((Contained)nak.obj).get_identification()+"\"in container \""+get_absolute_name().name()+"\" exists.");
          }
          nak = nak.next;
        }
      }
      ((TIRImplObject)akt.obj).canCommit();
      akt = akt.next;
    }
  }

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    LiItem akt = firstChild;
    boolean nn = false;
    int i;
    if (isNew()) {
      stKindImpl.toNormal();
      for(i=0;i<numOfItems;i++) {
        ((TIRImplObject)akt.obj).doCommit(this,rep);
        akt = akt.next;
      }
      parent = in;
      this.rep = rep;
    } else {
      for(i=0;i<numOfItems;i++) {
        if (((TIRImplObject)akt.obj).isNew()) 
          nn = true;
        ((TIRImplObject)akt.obj).doCommit(normal,rep);
        if (nn)
          normal.addListItem(akt.obj);
        akt = akt.next;
        nn = false;
      }
      if (lockForMe) {
        normal.lockFor = -1L;
        lockCreate.unlock();
        lockForMe = false;
      }
    }
  }

  /* return workId for which is locked */
  public long isLockedForCreate()  throws RemoteException {
    if ((stKindImpl.value()==StateKind.sk_work))
      if (normal==null)
        return -1L;
    if (lockCreate.isLocked()) {
      return normal.lockFor;
    }
    return -1L;
  }

  /* search if in this branch exists "create lock" */
  public long canLock()  throws RemoteException {
    long a;
    if ((a=isLockedForCreate())!=-1L)  // if this module is locked
      return a;
    // search lock up
    Container con =  parent;
    while (con.get_def_kind().value()!=DefinitionKind.dk_Repository &&
           con.get_def_kind().value()!=DefinitionKind.dk_WorkRepository) {
      if ((a=((SContainer)con).isLockedForCreate())!=-1L)
        return a;
      con = ((Contained)con).get_defined_in();
    }
    // search lock down
    if ((a=canLockChild())!=-1L)
      return a;
    return -1L;
  } 
  
  /* search if in children exists "create lock" */
  public long canLockChild() throws RemoteException {
    LiItem akt = firstChild;
    long a;
    for(int i=0;i<numOfItems;i++) {
      if (akt.obj.get_def_kind().value()==DefinitionKind.dk_AModule) {
        if ((a=((AModuleDefImpl)akt.obj).isLockedForCreate())!=-1L)
          return a;
        else {
          if ((a=((AModuleDefImpl)akt.obj).canLockChild())!=-1L)
            return a;
        }
      }
    }
    return -1L;
  }

  /* it is called from create_* methods */
  boolean getCreateLock() throws RemoteException {
    if (!isNew()) {
      if (lockForMe)
        return true;
      if (lockCreate.isLocked())
        return false;
      ((WorkRepositoryImpl)rep).normalRep.lockGetCreate.lock();
      long a = canLock();
      if ((a==-1L) || (a==workId)) {
        lockForMe = true;
        lockCreate.lock();
	normal.lockFor = workId;
      }  
      ((WorkRepositoryImpl)rep).normalRep.lockGetCreate.unlock();
      return true;
    }
    return true;
  }

  public void doAbort(long workId) throws RemoteException {
    LiItem akt = firstChild;
    for(int i=0;i<numOfItems;i++) {
      //if (((TIRImplObject)akt.obj).isNew()) 
        ((TIRImplObject)akt.obj).doAbort(workId);
      akt = akt.next;
    }
    if (lockForMe) {
      normal.lockFor = -1L;
      lockCreate.unlock();
      lockForMe = false;
    }   
  }
}
