/**
 * TR2InOutRMISkel.java is a part of the SOFA project.
 * This file was created by pepan on 19.3.2003.
 */
package SOFA.SOFAnode.TR.Connector;

import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import SOFA.Connector.ConnectorTransportException;
import SOFA.Connector.Linkable;
import SOFA.SOFAnode.TR.ComponentInfo;
import SOFA.SOFAnode.TR.TRException;
import SOFA.SOFAnode.TR.Impl.BundleImpl;

/**
 * @author Petr Panuska
 */
public class TR2InOutRMISkel extends UnicastRemoteObject implements CNTR2InOutRemote, Linkable {
  private CNTR2InOut target;

  public TR2InOutRMISkel (String url) throws java.rmi.RemoteException, java.net.MalformedURLException, java.rmi.NotBoundException {
    super();
    String rmihost = System.getProperty("sofa.rmihost", "localhost");
    String rmiport = System.getProperty("sofa.rmiport", "1099");
    // because of remote registering of services
    // don't be confused with the name '.DeplDockRegistry.RgRMIDock', it has nothing to do with a Depl. Dock
    // for more info contact Petr Hnetynka
    SOFA.SOFAnode.Run.DeplDockRegistry.RgRMIDock rg = (SOFA.SOFAnode.Run.DeplDockRegistry.RgRMIDock) java.rmi.Naming.lookup("//" + rmihost + ":" + rmiport + "/RgRMIDock");
    rg.rebind(url, this);
  }

  public void link (Object target) {
    this.target = (CNTR2InOut) target;
  }

  public void storeBundle (byte[] bundleData) throws RemoteException, TRException {
    BundleImpl bundle = new BundleImpl();
    try {
      bundle._read(new cz.cuni.sofa.lib.Impl.ByteArrayInputStream(bundleData));
    } catch (IOException e) {
      throw new ConnectorTransportException("Can't serialize in the 'storeBundle'.", e);
    }
    target.storeBundle(bundle);
  }

  public byte[] getBundle (ComponentInfo[] descs, ComponentInfo[] comps, boolean inferiors) throws RemoteException, TRException {
    BundleImpl bundle = (BundleImpl) target.getBundle(descs, comps, inferiors);
    byte[] ret = null;
    try {
      cz.cuni.sofa.lib.Impl.ByteArrayOutputStream os = new cz.cuni.sofa.lib.Impl.ByteArrayOutputStream();
      bundle._write(os);
      os.close();
      ret = os.toByteArray();
    } catch (IOException e) {
      throw new ConnectorTransportException("Can't serialize in the 'getBundle'.", e);
    }
    return ret;
  }

/*
  public byte[] getMetaBundle (ComponentInfo[] descs) throws RemoteException, TRException {
    return getBundle(descs, null); // uses the getBundle method
  }

*/
  public boolean contains (ComponentInfo descrs) throws RemoteException {
    return target.contains(descrs);
  }

  public byte[] list () {
    BundleImpl bundle = (BundleImpl) target.list();
    byte[] ret = null;
    try {
      cz.cuni.sofa.lib.Impl.ByteArrayOutputStream os = new cz.cuni.sofa.lib.Impl.ByteArrayOutputStream();
      bundle._write(os);
      os.close();
      ret = os.toByteArray();
    } catch (IOException e) {
      throw new ConnectorTransportException("Can't serialize in the 'list'.", e);
    }
    return ret;
  }
}
