/* $Id: ActionToken.java,v 1.2 2004/05/20 14:23:52 bures Exp $ */

package SOFA.SOFAnode.Util;


/**
 * A String wrapper implementing action tokens with support for behavior protocol-specific
 * renaming operations.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
*/
class ActionToken implements Printer {

/**
 * public event name for easy handling of the wrapped string
*/
    public String name;

/**
 * Constructor for creating a new event name from string.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
*/
    public ActionToken( String s ) {
        name = s;
    }

/**
 * Creates an instance of an action token by transforming the action token to
 * the corresponding tau action token. ? or ! at the beginning of the action token
 * will be replaced by #.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	new tau action token
*/
    public ActionToken toTauActionToken() {
        if( name.startsWith("?") || name.startsWith( "!" ) )
            return new ActionToken((new String("#")).concat(name.substring(1)));
        else return new ActionToken("#" + name );
    }

/**
 * Creates an instance of an action token by transforming the action token to
 * the corresponding inverse action token.
 * Thus, ?a will create !a and vice versa. Other starting characters are let unchanged.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	new action token
*/
    public ActionToken toInverseActionToken() {
        String e;
        if( name.startsWith( "!" ) )
            e = (new String( "?" )).concat(name.substring(1));
	    else if( name.startsWith( "?" ) )
            e = (new String( "!" )).concat(name.substring(1));
        else e = name;
	    return new ActionToken(e);
    }

/**
 * Creates an event from the action token by removing leading ?, ! or #.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	action token as event
*/
    public ActionToken toEvent() {
        String e = name;
        if( name.startsWith( "!" ) || name.startsWith( "?" ) || name.startsWith( "#" ) )
          e = name.substring(1);
        return new ActionToken(e);
    }

/**
 * Tests, whether the action token is a request, i.e., ends with ^
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	result of the test
*/
    public boolean isRequest() {
        return name.endsWith( "^" ); 
    }

/**
 * Tests, whether the action token is a response, i.e., ends with $
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	result of the test
*/
    public boolean isResponse() {
        return name.endsWith( "$" ); 
    }

/**
 * Tests, whether the action token is a general event, i.e., ends with ~
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	result of the test
*/
    public boolean isGeneralEvent() {
        return name.endsWith( "~" ); 
    }

    public boolean isEvent() {
	return ! ( name.startsWith("?") || name.startsWith("!") || name.startsWith( "#" ));
    }

/**
 * Tests, whether the action token is a special event, ie. []
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	result of the test
*/
    public boolean isSpecial() {
        return name.startsWith( "[" ); 
    }

/**
 * Tests, whether the action token is a simple call abbreviation, i.e.,
 * it does not ends by $,^ or ~
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @return	result of the test
*/
    public boolean isSimpleCall() {
        return ! ( this.isResponse() || this.isRequest() || this.isGeneralEvent() || this.isSpecial() ); 
    }

    public String getHashKey() {
	return name.intern();
    }
    
    public void Print( int level ) {
	Debug.println( level, name );
    }
    
    public void Print( ) {
	this.Print( 1 );
    }
}