/* $Id: BehaviorProtocolCheckerFactory.java,v 1.2 2004/09/10 14:16:48 hnetynka Exp $ */
package SOFA.SOFAnode.Util;

/**
 * A factory for getting a checker implementing interface for behavior protocol conformance checker.
 *
 * @author  Stanislav Visnovsky, modified by Petr Hnetynka
 * @version 2.0.0
 * @see BehaviorProtocolChecker
*/

public class BehaviorProtocolCheckerFactory {

/**
  * Creates and returns an implementation of BehaviorProtocolChecker intarface. 
  * The <em>className</em> parameter specifies the name of the class that implements
  * BehaviorProtocolChecker interface. If the specified checker cannot be created or
  * the parameter is <em>null</em>, an instance of the default implementation is returned.
  *
  * @param className class name of the protocol checker
  * @return protocol checker instance
  */
  public static BehaviorProtocolChecker getChecker(String className) {
    if (className == null)
      return new checker();
    Class cl = null;
    try {
      cl = BehaviorProtocolCheckerFactory.class.forName(className);
    } catch (ClassNotFoundException e) {
      return new checker();
    }
    BehaviorProtocolChecker ret = null;
    try {
      ret = (BehaviorProtocolChecker) cl.newInstance();
    } catch (Exception e) {
      return new checker();
    }
    return ret;
  }
}
