/*
 * $Id: DFSRConsent.java,v 1.2 2005/07/08 12:04:11 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.DFSR;


import SOFA.SOFAnode.Util.DFSRChecker.node.ConsentNode;
import SOFA.SOFAnode.Util.DFSRChecker.node.InvalidParameterException;
import SOFA.SOFAnode.Util.DFSRChecker.state.State;


/**
 * This class perform the checking operation in each state.
 * It is called from the DFSRTraverser that implements the generic DFS algorithm.
 */
public class DFSRConsent implements IDFSRExtension {

    /**
     * Creates a new instance of DFSRConsent.
     * @param automaton the root of the parse tree
     */
    public DFSRConsent(ConsentNode automaton) {
        this.automaton = automaton;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#action(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void action(State state) throws InvalidParameterException {

    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionNew(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean actionNew(State state) throws InvalidParameterException, AcceptingStateException {
        state.label = new String("S" + timestamp);
        state.timestamp = timestamp++;
        return automaton.isAccepting(state);
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionCycle(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionCycle(State state) throws InvalidParameterException {
        if (state.getSignature().getCycleId() == Long.MAX_VALUE) {
            state.cycleStart = true;
            state.getSignature().setCycleId(state.timestamp);
        }
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionVisited(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionVisited(State state) throws InvalidParameterException {

    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionBack(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionBack(State state) throws InvalidParameterException {

    }
    
    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#reset()
     */
    public void reset() {
        timestamp = 0;
    }

    //------------------------------
    /**
     * unique cycle id generator
     */
    private long timestamp = 0;

    /**
     * The automaton represented by the consent node of the parse tree.
     */
    private ConsentNode automaton;
}