/*
 * $Id: DFSRDotVis.java,v 1.4 2005/07/08 12:04:11 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */


package SOFA.SOFAnode.Util.DFSRChecker.DFSR;


import java.io.FileOutputStream;
import java.io.IOException;


import SOFA.SOFAnode.Util.DFSRChecker.node.ActionRepository;
import SOFA.SOFAnode.Util.DFSRChecker.node.InvalidParameterException;
import SOFA.SOFAnode.Util.DFSRChecker.node.TreeNode;
import SOFA.SOFAnode.Util.DFSRChecker.state.State;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPair;

/**
 * This class produces an output for dot visualization tool (part of GraphViz by Att).
 * The visualizied subject is the transition graph represented by a TreeNode
 *  
 */
public class DFSRDotVis implements IDFSRExtension {

    /**
     * Creates a new instance of DFSRDotVis.
     * 
     * @param outputfile the name of the output file
     * @param automaton automaton to be visualized
     * @param repository an instance of ActionRepository for printing the action name.
     */
    public DFSRDotVis(FileOutputStream outputfile, TreeNode automaton, ActionRepository repository) {
        this.outputfile = outputfile;
        this.automaton = automaton;
        this.repository = repository;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#action(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void action(State state) throws InvalidParameterException {
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionNew(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean actionNew(State state) throws InvalidParameterException, CheckingException {
        state.label = new String("S" + timestamp);

        try {
            // the state
            boolean acc = automaton.isAccepting(state);
            State initstate = automaton.getInitial();
            boolean init = initstate.getSignature().equals(state.getSignature());
            if (acc)
                outputfile.write(("  \"" + state.getSignature() + "\" [label=\"S" + timestamp + "\", shape=doublecircle, style=filled, fillcolor=\"grey\"];\n").getBytes());
            else
                outputfile.write(("  \"" + state.getSignature() + "\" [label=\"S" + timestamp + "\"];\n").getBytes());

            if (init)
                if (acc)
                    outputfile.write(("  \"" + state.getSignature() + "\" [shape=box, peripheries=2, style=filled, fillcolor=\"grey\"];\n").getBytes());
                else
                    outputfile.write(("  \"" + state.getSignature() + "\" [shape=box];\n").getBytes());

            //... and its transitions
            TransitionPair[] trans = automaton.getTransitions(state).transitions;
            for (int i = 0; i < trans.length; ++i) {
                outputfile.write(("  \"" + state.getSignature() + "\" -> \"" + trans[i].state.getSignature() + "\" [label=\"" + repository.getItemString(trans[i].eventIndex) + "\"];\n").getBytes());
                if (trans[i].state.label.equals("<no_label>"))
                    outputfile.write(("  \"" + trans[i].state.getSignature() + "\" [label=\".\"];\n").getBytes());
            }

        } catch (IOException e) {
            throw new AcceptingStateException("Visualization backend: error while writing to the file.");
        }

        state.timestamp = timestamp++;

        
        return automaton.isAccepting(state);
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionCycle(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionCycle(State state) throws InvalidParameterException {
        if (state.getSignature().getCycleId() == Long.MAX_VALUE) {
            state.cycleStart = true;
            state.getSignature().setCycleId(state.timestamp);
        }
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionVisited(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionVisited(State state) throws InvalidParameterException {
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#actionBack(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public void actionBack(State state) throws InvalidParameterException {
        try {
            if (state.isAcceptingReachable()) {
                outputfile.write(("  \"" + state.getSignature() + "\" [style=filled, fillcolor=\"grey\"];\n").getBytes());
            }
        } catch (IOException e) {
        }

    }
    
    /** 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.DFSR.IDFSRExtension#reset()
     */
    public void reset() {
        timestamp = 0;
    }

    /**
     * File where the description of the automata is stored
     *  
     */
    private FileOutputStream outputfile;

    /**
     * State index
     */
    private int timestamp = 0;

    /**
     * The root if the parse tree.
     */
    private TreeNode automaton;

    /**
     * Action repository.
     */
    private ActionRepository repository;

}