/* $Id: DFSRChecker.java,v 1.4 2005/07/08 12:04:11 kofron Exp $ */

/*
 * DFSRChecker.java
 *
 * Created on 1. bezen 2004, 14:23
 *
 * @author  Jan Kofron
 * @version 1.0.0
 */

package SOFA.SOFAnode.Util.DFSRChecker;

import java.util.TreeSet;

import SOFA.SOFAnode.Util.BehaviorProtocolChecker;
import SOFA.SOFAnode.Util.SyntaxErrorException;
import SOFA.SOFAnode.Util.DFSRChecker.backend.*;
import SOFA.SOFAnode.Util.DFSRChecker.node.*;
import SOFA.SOFAnode.Util.DFSRChecker.parser.*;

/**
 * This is implementation of BehaviorProtocolChecker for SOFAnode. Frame and
 * Architecture and Interface and Frame protocols can be checked.
 * 
 * @author Jan Kofron
 * @version 1.0.0
 */
public class DFSRChecker implements BehaviorProtocolChecker {

    /** Creates a new instance of DFSRChecker */
    public DFSRChecker() {
    }

    /**
     * Not implemented...
     */
    public int StatisticsMaxStates() {
        return -1;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.BehaviorProtocolChecker#frame2arch(java.lang.String,
     *      java.lang.String[], java.lang.String, java.lang.String[])
     */
    public String frame2arch(String frameProt, String[] providesOperations, String archProt, String[] ifaceOperations) {
        ActionRepository repository = new ActionRepository();

        //debug
        /*
         * System.out.println("Arch prot: " + archProt);
         * System.out.println("Frame prot: " + frameProt);
         * System.out.println("ifaceops: "); for (int i = 0; i <
         * ifaceOperations.length; ++i) System.out.println(ifaceOperations[i]);
         * System.out.println("providesops: "); for (int i = 0; i <
         * providesOperations.length; ++i)
         * System.out.println(providesOperations[i]);
         *  
         */
        //debug
        Builder parser = new Builder(repository);

        try {
            TreeNode frame = parser.build(new StringTokenizer(frameProt));
            TreeNode arch = parser.build(new StringTokenizer(archProt));

            TreeSet ifaceops = new TreeSet();

            for (int i = 0; i < ifaceOperations.length; ++i)
                ifaceops.add(new Integer(ActionRepository.getPure(repository.getItemIndex(ifaceOperations[i]))));

            arch = new RestrictionNode(arch, ifaceops, repository);

            //ComplianceBackend ce = new ComplianceBackend(repository);
            ConsentBackend ce = new ConsentBackend(repository);

            TreeNode[] nodes = new TreeNode[2];
            nodes[1] = frame;
            nodes[0] = arch;

            String[] provarray = new String[1];

            provarray[0] = "";
            for (int i = 0; i < providesOperations.length; ++i) {
                // cut off the first ? and !
                provarray[0] += providesOperations[i].substring(1) + ",";
            }
            provarray[0] = provarray[0].substring(0, provarray[0].length() - 1);

            // that't ok, in SOFA we have no incomplete bindings so far
            CheckingResult result = ce.perform(nodes, provarray, new String[0]);
            if (result.type == CheckingResult.ERR_OK)
                return null;
            else
                return result.description;
            
        } catch (SyntaxErrorException e) {
            return "Error: SyntaxErrorException: " + e.getMessage();
        }
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.BehaviorProtocolChecker#int2frame(java.lang.String,
     *      boolean, java.lang.String[], java.lang.String)
     */
    public String int2frame(String ifaceProt, boolean provide, String[] ifaceOperations, String frameProt) {
        ActionRepository repository = new ActionRepository();

        //debug
        /*
         * System.out.println("Iface prot: " + ifaceProt);
         * System.out.println("Frame prot: " + frameProt);
         * System.out.println("ifaceops: "); for (int i = 0; i <
         * ifaceOperations.length; ++i) System.out.println(ifaceOperations[i]);
         *  
         */
        //debug

        Builder parser = new Builder(repository);

        try {
            TreeNode frame = parser.build(new StringTokenizer(frameProt));
            TreeNode iface = parser.build(new StringTokenizer(ifaceProt));

            TreeSet ifaceops = new TreeSet();

            for (int i = 0; i < ifaceOperations.length; ++i)
                ifaceops.add(new Integer(ActionRepository.getPure(repository.getItemIndex(ifaceOperations[i]))));

            frame = new RestrictionNode(frame, ifaceops, repository);

            //ComplianceBackend ce = new ComplianceBackend(repository);
            ConsentBackend ce = new ConsentBackend(repository);

            TreeNode[] nodes = new TreeNode[2];
            // compliance is checked in the way according to the value of
            // provide param
            if (provide) {
                nodes[0] = frame;
                nodes[1] = iface;
            } else {
                nodes[0] = iface;
                nodes[1] = frame;
            }

            String[] provarray = new String[1];

            provarray[0] = "";
            for (int i = 0; i < ifaceOperations.length; ++i) {
                // cut off the first ? and !
                provarray[0] += ifaceOperations[i].substring(1) + ",";
            }
            provarray[0] = provarray[0].substring(0, provarray[0].length() - 1);

            // that't ok, in SOFA we have no incomplete bindings so far
            CheckingResult result = ce.perform(nodes, provarray, new String[0]);
            if (result.type == CheckingResult.ERR_OK)
                return null;
            else
                return result.description;
        } catch (SyntaxErrorException e) {
            return "Error: SyntaxErrorException: " + e.getMessage();
        }
    }

    /** @link dependency */
    /* # Builder lnkBuilder; */

    /** @link dependency */
    /* # ComplianceBackend lnkComplianceBackend; */
}