/*
 * $Id: AtomicAction.java,v 1.1 2005/07/13 07:27:49 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.node;

import java.util.Collection;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * Extending class for a treeset.
 * Implements comparable interface and can be use for 
 * storing within a hashset or whatever
 */
public class AtomicAction extends TreeSet implements Comparable {


    /**
     * Creates a new instance of AtomicAction.
     * @param c members of c that are to be added to this AtomicAction
     */
    public AtomicAction(Collection c) {
        super(c);
    }

    /**
     * Creates a new instance of AtomicAction.
     *
     */
    public AtomicAction() {}
    
    
    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o) {
        if (o instanceof AtomicAction) {
            AtomicAction other = (AtomicAction)o;
            
            if (this.size() < other.size())
                return -1;
            else if (this.size() > other.size())
                return 1;
            
            else {
                Iterator it1 = this.iterator();
                Iterator it2 = other.iterator();
                
                while (it1.hasNext()) {
                    Object o1 = it1.next();
                    Object o2 = it2.next();
                    
                    int result = ((Comparable)o1).compareTo(o2);
                    if (result != 0)
                        return result;
                }
                
                return 0;
            }
        }
        else
            return -1;
    }
    
    
}
