/* 
 * $Id: EventNode.java,v 1.4 2005/07/08 12:04:12 kofron Exp $ 
 *
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.node;

import java.util.ArrayList;
import java.util.TreeSet;


import SOFA.SOFAnode.Util.DFSRChecker.state.SimpleState;
import SOFA.SOFAnode.Util.DFSRChecker.state.State;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPair;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPairs;
import SOFA.SOFAnode.Util.DFSRChecker.utils.AnotatedProtocol;


/**
 * This is a class extending TreeNode representing an event node within the
 * tree.
 */
public class EventNode extends TreeNode {

    /** Creates a new instance of EventNode */
    public EventNode(int eventindex, ActionRepository repository) {
        super(repository.getItemString(eventindex));
        this.eventIndex = eventindex;

        this.initState = new SimpleState(0);

        this.initTransitions = new TransitionPairs(new TransitionPair[1]);
        this.initTransitions.transitions[0] = new TransitionPair(eventindex, new SimpleState(1));

        this.secondTransitions = new TransitionPairs(new TransitionPair[0]);

        this.nodes = new TreeNode[0];

        this.weight = 2;
        
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getInitial()
     */
    public State getInitial() {
        return initState;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#isAccepting(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean isAccepting(State state) {
        return (((SimpleState) state).state == 1);
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getTransitions(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public TransitionPairs getTransitions(State state) throws InvalidParameterException {
        if (((SimpleState) state).state == 0)
            return initTransitions;
        else if (((SimpleState) state).state == 1)
            return secondTransitions;
        else
            throw new InvalidParameterException();
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getWeight()
     */
    public long getWeight() {
        return 2;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#forwardCut(java.util.TreeSet)
     */
    public TreeNode forwardCut(TreeSet livingevents) {
        if (livingevents.contains(new Integer(eventIndex)))
            return this;
        else
            return null;
    }

    /**
     * @return the symbolic name of the treenode denoting its type
     */
    public String[] getTypeName() {
        String[] result = { "Event", "" };
        return result;
    }

    /**
     * @return the eventindex
     */
    public int getEventIndex() {
        return eventIndex;
    }
    
    public int getLeafCount() {
        return 1;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getAnotatedProtocol(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public AnotatedProtocol getAnotatedProtocol(State state) {
        SimpleState sstate = (SimpleState) state;
        String result = new String();
        ArrayList indicesresult = new ArrayList();
        
        result = protocol;
        if (state != null)
            if (((SimpleState)state).state == 0)
                indicesresult.add(new Integer(0));
            else
                indicesresult.add(new Integer(result.length()));
        
        return new AnotatedProtocol(result, indicesresult);
    }
    
    
    //--------------------------------------------------------------------------
    /**
     * event index
     */
    final private int eventIndex;

    /**
     * Simple state for initial state representation
     */
    private SimpleState initState;
    
    /**
     * Transitions for initial state
     */
    final private TransitionPairs initTransitions;

    /**
     * Transitions for the second state
     */
    final private TransitionPairs secondTransitions;

}