/* 
 * $Id: ExplicitNode.java,v 1.4 2005/07/08 12:04:12 kofron Exp $ 
 *
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.node;


import java.util.TreeMap;
import java.util.TreeSet;


import SOFA.SOFAnode.Util.DFSRChecker.state.State;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPair;
import SOFA.SOFAnode.Util.DFSRChecker.state.TransitionPairs;
import SOFA.SOFAnode.Util.DFSRChecker.utils.AnotatedProtocol;

/**
 * Represents an explicit automaton for a parse subtree.
 */
public class ExplicitNode extends TreeNode {

    /** Creates a new instance of ExplicitNode */
    public ExplicitNode(State initial, TreeMap transitions, TreeSet accepting, String protocol, TreeNode original) {
        super(protocol);
        this.initial = initial;
        this.transitions = transitions;
        this.accepting = accepting;

        this.nodes = new TreeNode[0];
        this.original = original;

        this.weight = 1;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getInitial()
     */
    public State getInitial() {
        return initial;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getTransitions(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public TransitionPairs getTransitions(State state) throws InvalidParameterException {
        TransitionPair[] result = (TransitionPair[]) transitions.get(state);
        if (result != null)
            return new TransitionPairs(result);
        else
            return new TransitionPairs(new TransitionPair[0]);

    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#isAccepting(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean isAccepting(SOFA.SOFAnode.Util.DFSRChecker.state.State state) {
        return accepting.contains(state);
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getWeight()
     */
    public long getWeight() {
        return 1;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#forwardCut(java.util.TreeSet)
     */
    public TreeNode forwardCut(TreeSet livingevents) {
        // no cutting is performed here...
        return this;
    }

    /**
     * 
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#copy()
     */
    public TreeNode copy() {
        // no cutting is performed here...
        return this;
    }

    /**
     * @return the symbolic name of the treenode denoting its type
     */
    public String[] getTypeName() {
        String[] result = { "Explicit", "EXPLICIT" };
        return result;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.node.TreeNode#getAnotatedProtocol(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public AnotatedProtocol getAnotatedProtocol(State state) {
        return original.getAnotatedProtocol(state);
    }    
    //--------------------------------------------------------------------------
    /**
     * The initial state.
     */
    final private State initial;
    
    /**
     * The original parse subtree .
     */
    private TreeNode original;

    /**
     * The transition matrix.
     */

    final private TreeMap transitions;

    /**
     * The set of accepting states.
     */
    final private TreeSet accepting;


}