/* 
 * $Id: StringTokenizer.java,v 1.4 2005/07/08 12:04:12 kofron Exp $ 
 *
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.parser;

import java.io.IOException;

/**
 * This class it to replace the originally used StringReader in order to be able
 * to provide information about the index in the string where the current token
 * lies.
 *  
 */
public class StringTokenizer extends ProtocolReader {

    /**
     * Creates a new instance of StringTokenizer.
     * @param source the source string to be read.
     */
    public StringTokenizer(String source) {
        this.source = source;
    }

    /**
     * Reads the next character
     * 
     * @return the character read
     */
    public int read() {
        if (index == source.length())
            return -1;
        else
            return source.charAt(index++);
    }

    /**
     * Checks whether the source string is not yet parsed completelly
     * 
     * @return true if there are character left, false otherwise
     */
    public boolean ready() {
        return source.length() >= index;
    }

    /**
     * @return the index within the string being parsed
     * 
     */
    public int getIndex() {
        return index;
    }

    /**
     * @see java.io.Reader#close()
     */
    public void close() throws IOException {
        //nothing to be done
    }

    /**
     * @see java.io.Reader#read(char[], int, int)
     */
    public int read(char[] cbuf, int off, int len) throws IOException {
        //not implemented
        throw new IOException("Not implemented");
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.parser.ProtocolReader#resetIndex()
     */
    public void resetIndex() {
        // makes nothing
    }    
    //-----------------------------------------------------------------------------------

    /**
     * The string being parsed
     */
    private String source;

    /**
     * The current index within the string being parsed
     */
    int index = 0;



}