/*
 * $Id: DenotedState.java,v 1.1 2005/07/13 07:27:49 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */

package SOFA.SOFAnode.Util.DFSRChecker.state;

/**
 * This class is to be used by an alternative node.
 * It contains an index identifying the correct child, which state it also
 * contains. The reason for existence of this class to make a reasonable estimate
 * of the entire state signature and keep the signature size as small as possible.
 * The problem is caused by the DeterministicNode used in higher levels of 
 * parse trees, where the signatures of superstates (created as o consequence of
 * "deterministing" of the PT automaton.
 * Please note that this is not an optimization, it is necessary for the signature
 * principle to work.
 */
public class DenotedState extends State {
    
    /**
     * Creates a new instance of DenotedState.
     * 
     * @param index index of the subtree which is the state related to
     * @param state the state of a subtree
     * @param children log(number of children of this state) 
     */
    public DenotedState(int index, State state, int children) {
        this.index = index;
        this.state = state;
        this.children = children;
    }
        

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State#createSignature()
     */
    protected void createSignature() {
        if (index == -1) 
            for (int i = 0; i < children; ++i)
                signature.set(i, true);		// the signature has a slot for -1 = 11..11
        
        else
            for (int i = 0; i < children; ++i)
                signature.set(i, (index & (1 << i)) != 0 ? true : false);
        
        signature.concat(state.getSignature());
    }
    
    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State#equals(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean equals(State another) {
        if (!(another instanceof DenotedState))
            return false;

        DenotedState dstate = (DenotedState) another;

        if (this.index != dstate.index)
            return false;

        if (this.signature == null)
            this.getSignature();

        if (another.signature == null)
            another.getSignature();

        return this.getSignature().equals(dstate.getSignature());
    }      
    
    /**
     * Index of the subtree to which the state is related
     */
    final public int index;
    
    /**
     * State of a subtree
     */
    final public State state;
    
    /**
     * Number of the children - necessary for signature construction
     */
    private int children;
    

}
