/*
 * $Id: IndexState.java,v 1.4 2005/07/08 12:04:12 kofron Exp $
 * 
 * Copyright 2004
 *    Distributed Systems Research Group
 *    Department of Software Engineering
 *    Faculty of Mathematics and Physics
 *    Charles University, Prague
 *
 * Copyright 2005
 *    Formal Methods In Software Engineering Group
 *    Institute of Computer Science
 *    Academy of Sciences of the Czech Republic
 * 
 * This code was developed by Jan Kofron <kofron@nenya.ms.mff.cuni.cz>
 */


package SOFA.SOFAnode.Util.DFSRChecker.state;

/**
 * This class represents the state with an index of the subautomaton and pointer
 * to it. Note that both the index and the pointer are neccessary. The members
 * of this class are declared as public for better performance of the
 * SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.
 */
public class IndexState extends State {

    /** Creates a new instance of IndexState */
    public IndexState(int index, State[] states) {
        this.index = index;
        this.states = states;
    }

    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State#equals(SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State)
     */
    public boolean equals(State another) {
        if (!(another instanceof IndexState))
            return false;

        IndexState istate = (IndexState) another;

        if (this.index != istate.index)
            return false;

        if (this.signature == null)
            this.getSignature();

        if (another.signature == null)
            another.getSignature();

        return this.getSignature().equals(istate.getSignature());
    }
    
    /**
     * @see SOFA.SOFAnode.Util.objectweb.fractal.behprotocols.checker.state.State#createSignature()
     */
    protected void createSignature() {
        for (int i = 0; i < states.length; ++i) {
            this.signature.concat(states[i].getSignature());
        }
    }
    

    /**
     * The substates.
     */
    final public State[] states;

    /**
     * the index of the automaton
     */
    final public int index;


}