/* $Id: EdgeFactory.java,v 1.2 2004/05/20 14:23:53 bures Exp $ */

package SOFA.SOFAnode.Util;

import java.util.Hashtable;

/**
 * The factory class for creating new and copying the old edges.
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
*/
class EdgeFactory {


// size should be 3*n+2, since for every event we need emitting, absorbing and internal
// index 0 is for NULL, index 1 is for [UPDATE]

	public static ActionToken ActionTokens[];
	public static int inverseActionTokens[];
	public static int tauActionTokens[];

	private static Hashtable idxActionTokens;

	private static int nextindex;

	private static int size;

	public static int getActionTokensNum() {
	    return nextindex;
	}

	public static void initActionTokens() {
	    Debug.println( 2, "Initializing action tokens");
	    ActionTokens = new ActionToken[16];
	    inverseActionTokens = new int[16];
	    tauActionTokens = new int[16];
	    idxActionTokens = new Hashtable();
	    ActionTokens[0]=new ActionToken( "NULL" );
	    inverseActionTokens[0] = 0; // NULL is inverse to itself
	    tauActionTokens[0] = 0; // NULL is internal to itself;
	    idxActionTokens.put( new ActionToken( "NULL" ).getHashKey(), new Integer (0) );
	    ActionTokens[1]=new ActionToken( "[UPDATE]" );
	    inverseActionTokens[1] = 1; // [UPDATE] is inverse to itself
	    tauActionTokens[1] = 1; // [UPDATE] is internal to itself;
	    idxActionTokens.put( new ActionToken( "[UPDATE]" ).getHashKey(), new Integer (1) );
	    nextindex = 2;
	    size = 16;
	}	

	public static int getActionTokenIdx( ActionToken t ) {
	    Integer a = (Integer)idxActionTokens.get( t.getHashKey() );
	    if( a == null ) {
		Debug.println(3, "Not found "+t.name);
		if( nextindex >= size-2 ) { // we need more room
		    Debug.println(3,"We need more room for action tokens");

		    // enlarge action tokens
		    ActionToken newat[] = new ActionToken[ nextindex+15 ];
		    for( int i = 0; i<nextindex; i++) newat[i] = ActionTokens[i]; 
		    ActionTokens = newat;

		    // enlarge inverse action tokens
		    int newint[] = new int[ nextindex+15 ];
		    for( int i = 0; i<nextindex; i++) newint[i] = inverseActionTokens[i]; 
		    inverseActionTokens = newint;

		    // enlarge tau action tokens
		    newint = new int[ nextindex+15 ];
		    for( int i = 0; i<nextindex; i++) newint[i] = tauActionTokens[i]; 
		    tauActionTokens = newint;

		    size = nextindex+15;
		    Debug.println(3, "Action token table resized");
		}
		int ind = nextindex;
		Debug.println(2, "Putting new action token "+t.name+" at "+ind);
		ActionTokens[ind] = t;
		idxActionTokens.put( t.getHashKey(), new Integer(ind) );

		// fill-in inverse action tokens
		int inv = findInverseActionToken( t );
		Debug.println(3, "Inverse is :"+inv);
		inverseActionTokens[ind] = inv;
		if( inverseActionTokens[ind] != -1 )
		    inverseActionTokens[ inv ] = ind;

		// fill-in tau action tokens
		tauActionTokens[ind] = findTauActionToken( t );
		if( t.name.startsWith( "#" ) ) {
		    t = new ActionToken( "?"+t.name.substring( 1 ) );
		    Integer k = (Integer)idxActionTokens.get( t.getHashKey() );
		    if( k != null ) tauActionTokens[k.intValue()] = ind;
		    k = (Integer)idxActionTokens.get( t.toInverseActionToken().getHashKey() );
		    if( k != null ) tauActionTokens[k.intValue()] = ind;
		}

		nextindex ++;

		if( Debug.debug>2 ) {
		    Debug.println(3, "Actual action token table:");
		    for( int i = 0 ; i < nextindex ; i++ )
			Debug.println( 3, "   "+i+"\t"+ActionTokens[i].name+"\t inv:"+inverseActionTokens[i]+"\t tau:"+tauActionTokens[i]);
		}

		return nextindex-1;
	    } else return a.intValue();
	}

	private static int findInverseActionToken( ActionToken t ) {
	    int i;
	    ActionToken inverse = t.toInverseActionToken();
	    for( i = 0; i < nextindex ; i++ )
		if( ActionTokens[i].name.compareTo( inverse.name ) == 0 ) break;
	    if( i == nextindex ) return -1;
	    return i;
	}

	private static int findTauActionToken( ActionToken t ) {
	    int i;
	    for( i = 0; i < nextindex ; i++ )
		if( ActionTokens[i].name == t.toTauActionToken().name ) break;
	    if( i == nextindex ) return -1;
	    return i;
	}

	public static void checkTauActionToken( ActionToken t ) {
	    int i;
	    for( i = 0; i < nextindex ; i++ )
		if( ActionTokens[i].name == t.toTauActionToken().name ) break;
	    if( i == nextindex ) getActionTokenIdx( t.toTauActionToken() );
	}
	
}
