/* $Id: ConnGenPathInfo.java,v 1.2 2004/05/20 14:23:53 bures Exp $ */
package SOFA.Util;

import java.io.File;
import java.io.FileInputStream;

/** Provides path information for connector generator.
  *
  * @author Petr Hnetynka
  */
public class ConnGenPathInfo {

  public static class CPException extends RuntimeException {
    public CPException() { super(); }
    public CPException(String m) { super(m); }
    public CPException(String m, Throwable cause) { super(m, cause); }
  }

  public static java.util.Properties trIndex;
  public static String trDir;

  static {
    trDir = System.getProperty("sofa.tr.dir", null);
    if (trDir == null) {
      throw new CPException("Specify TR directrory (java property \"sofa.tr.dir\")");
    }
    trIndex = new java.util.Properties();
    try {
      FileInputStream fis = new FileInputStream(trDir+File.separator+"iface"+File.separator+"index");
      trIndex.load(fis);
      fis.close();
    } catch (java.io.IOException e) {
      throw new CPException("IOException: "+e.getMessage());
    }
  }
  
  /** Returns classpath for javac action of connector generator.
    * 
    * @param iface user interface of generated connector (more interfaces can be delimited by comma, e.g. "package.interface1,package.interface2")
    * @return classpath
    */
  public static String getConnGenClassPath(String iface) throws CPException {
    StringBuffer classpath = new StringBuffer();
    
    java.util.StringTokenizer st=new java.util.StringTokenizer(iface,",");
    while (st.hasMoreTokens()) {
      String token=st.nextToken().trim();
      String subdir = (String) trIndex.get(token);
      if (subdir == null) {
        throw new CPException("No such interface in TR: '"+token+"'");
      }
      classpath.append(trDir+File.separator+"iface"+File.separator+subdir);
      classpath.append(File.pathSeparator);
    }
    
    classpath.append(trDir+File.separator+"congen");
    classpath.append(File.pathSeparator);
    classpath.append(System.getProperty("java.class.path"));
    return classpath.toString();
  }

  public static String getRMICOutDir() {
    return trDir+File.separator+"congen";
  }

  public static String getIDLOutDir() {
    return trDir+File.separator+"congen";
  }
}
